// Navdoc.cpp : implementation of the CNavDoc class
//
/*
    Copyright (C) 1997 Ted Wright 
    NASA Business: wright@lerc.nasa.gov, Remainder: wright@en.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "Nav.h"

#include <afxtempl.h>
#include "Navdoc.h"
#include "flightpl.h"
#include "navview.h"
#include "math.h"
#include "serbgl.h"
#include "io.h"
#include "mainfrm.h"
#include "export.h"
#include "ExportProgress.h"
#include "NewList.h"
#include "formatedfile.h"

#define WM_NAV (WM_USER+500)

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CNavDoc

IMPLEMENT_DYNCREATE(CNavDoc, CDocument)

#define new DEBUG_NEW

BEGIN_MESSAGE_MAP(CNavDoc, CDocument)
	//{{AFX_MSG_MAP(CNavDoc)
	ON_COMMAND(ID_FILE_ADD, OnFileAdd)
	ON_UPDATE_COMMAND_UI(ID_FILE_ADD, OnUpdateFileAdd)
	ON_COMMAND(ID_FILE_LOAD, OnFileLoad)
	ON_COMMAND(ID_FILE_SEARCH, OnFileSearch)
	ON_COMMAND(ID_FILE_EXPORT, OnFileExport)
	ON_UPDATE_COMMAND_UI(ID_FILE_EXPORT, OnUpdateFileExport)
	ON_COMMAND(ID_MAP_INSERTGPSFIX, OnMapInsertgpsfix)
	ON_UPDATE_COMMAND_UI(ID_FILE_ADDTOSCENERYLIST_NEW, OnUpdateFileAddtoscenerylistNew)
	ON_COMMAND(ID_FILE_ADDTOSCENERYLIST_NAVLST, OnFileAddtoscenerylistNavlst)
	ON_UPDATE_COMMAND_UI(ID_FILE_ADDTOSCENERYLIST_NAVLST, OnUpdateFileAddtoscenerylistNavlst)
	ON_COMMAND(ID_FILE_LOADSCENERYLIST_NAVLST, OnFileLoadscenerylistNavlst)
	ON_COMMAND(ID_FILE_SEARCHFORBGLFILES_NAVLST, OnFileSearchforbglfilesNavlst)
	ON_COMMAND(ID_FILE_ADDTOSCENERYLIST_NEW, OnFileAddtoscenerylistNew)
	ON_COMMAND(ID_FILE_SEARCHFORBGLFILES_NEW, OnFileSearchforbglfilesNew)
	ON_COMMAND(ID_FILE_LOAD_WPT, OnFileLoadWpt)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CNavDoc construction/destruction

CNavDoc::CNavDoc() {
	dbFileIndex = 0;  // JYL
	runwayIndex.SetSize(0,2000);
	ilsIndex.SetSize(0,1000);
	vorIndex.SetSize(0,2000);
	ndbIndex.SetSize(0,4000);
	markerIndex.SetSize(0,1000);
	gpsIndex.SetSize(0,1000);
	airportIndex.SetSize(0,1000);
	atisIndex.SetSize(0,1000);
	wptIndex.SetSize(0,2000);	// JYL
	listFiles = 0;
	dbCount = 0;
	db.SetSize(0,6000);	//JYL
	idcounter = 0;
	numbercounter = 0;
	magVarTableExists = FALSE;
	mapsExist = FALSE;
	ReadMagVarTable();
	ReadMaps();
}

CNavDoc::~CNavDoc() {
	db.RemoveAll();
	db.FreeExtra();
	runwayIndex.RemoveAll();
	runwayIndex.FreeExtra();
	ilsIndex.RemoveAll();
	ilsIndex.FreeExtra();
	vorIndex.RemoveAll();
	vorIndex.FreeExtra();
	ndbIndex.RemoveAll();
	ndbIndex.FreeExtra();
	markerIndex.RemoveAll();
	markerIndex.FreeExtra();
	gpsIndex.RemoveAll();
	gpsIndex.FreeExtra();
	airportIndex.RemoveAll();
	airportIndex.FreeExtra();
	atisIndex.RemoveAll();
	atisIndex.FreeExtra();
	//++ JYL
	wptIndex.RemoveAll();
	wptIndex.FreeExtra();
}

BOOL CNavDoc::OnNewDocument() {
	if (!CDocument::OnNewDocument())
		return FALSE;
	dbCount = 0;
	db.RemoveAll();
 	db.FreeExtra();
	db.SetSize(0,5000);
	runwayIndex.RemoveAll();
	runwayIndex.FreeExtra();
	ilsIndex.RemoveAll();
	ilsIndex.FreeExtra();
	vorIndex.RemoveAll();
	vorIndex.FreeExtra();
	ndbIndex.RemoveAll();
	ndbIndex.FreeExtra();
	markerIndex.RemoveAll();
	markerIndex.FreeExtra();
	gpsIndex.RemoveAll();
	gpsIndex.FreeExtra();
	airportIndex.RemoveAll();
	airportIndex.FreeExtra();
	atisIndex.RemoveAll();
	atisIndex.FreeExtra();
	wptIndex.RemoveAll();
	wptIndex.FreeExtra();
	POSITION p = GetFirstViewPosition();
	CNavView *v = (CNavView *) GetNextView(p);
	v->init = TRUE;
	v->mag = 1;
	v->mag2 = 1;
	UpdateAllViews(NULL);
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CNavDoc serialization

void CNavDoc::Serialize(CArchive& ar) {
	if (ar.IsStoring())	{
		// TODO: add storing code here
	} else {
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CNavDoc diagnostics

#ifdef _DEBUG
void CNavDoc::AssertValid() const {
	CDocument::AssertValid();
}

void CNavDoc::Dump(CDumpContext& dc) const {
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CNavDoc commands

BOOL CNavDoc::OnOpenDocument(LPCTSTR lpszPathName) {
	if (!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;
	dbCount = 0;
	db.RemoveAll();
 	db.FreeExtra();
	db.SetSize(0,5000);
	POSITION p = GetFirstViewPosition();
	CNavView *v = (CNavView *) GetNextView(p);
	v->mag = 1;
	v->mag2 = 1;
	UpdateAllViews(NULL);

	fName = lpszPathName;
	dbFileIndex = 0;
    dbFiles[dbFileIndex] = fName;
	readFile(lpszPathName);
	SetTitle(lpszPathName);
	Index();
	v->ZoomToFit();
	return TRUE;
}

// read 3 bytes as a signed int
int CNavDoc::read3bytes(char *b) {
	char i[4];
	i[0] = 0;
	i[1] = *b;
	i[2] = *(b+1);
	i[3] = *(b+2);
	int lat = *((int *) i);
	lat >>= 8; 
	return(lat);
}

// read 3 bytes as a signed int, 1 byte as fractional part
double CNavDoc::readDM(char *b) {
	unsigned char dec = (unsigned char) b[0];
	int lat = read3bytes(&b[1]);
	if (lat > 0)
		return( (double) lat + dec / 256.0 );
	else
		return( (double) lat - dec / 256.0 );
}

// read 6 latitude bytes in "Micro" format
double CNavDoc::readMicro6Lat(char *b) {
	unsigned short fractPart;
	int intPart;
	intPart = *(int*)(b+2);
	fractPart = *(unsigned short *)b;
	if (intPart >= 0)
		return(intPart + fractPart / 65536.0);
	else
		return(intPart - fractPart / 65536.0);
}

// read 6 longitude bytes in "Micro" format ???
double CNavDoc::readMicro6Lon(char *b) {
	unsigned short fractPart;
	int intPart;
	intPart = read3bytes(b+3);
	fractPart = *(unsigned short *)b;
	if (intPart >= 0)
		return(intPart + fractPart / 65536.0);
	else
		return(intPart - fractPart / 65536.0);
}

// read 4 bytes in "Micro" format
double CNavDoc::readMicro4(char *b) {
	unsigned short fractPart;
	short intPart;
	intPart = *(short *)(b+2);
	fractPart = *(unsigned short *)b;
	if (intPart >= 0)
		return(intPart + fractPart / 65536.0);
	else
		return(intPart - fractPart / 65536.0);
}

int CNavDoc::readFile(CString filename) {
	POSITION p = GetFirstViewPosition();
	CNavView *v = (CNavView *) GetNextView(p);
	HANDLE filehandle, filemapping;
	char *bgl;
	int count = 0;
	char buffer[100];
	CString names[1000];

	char* pDot;
	pDot = strrchr (filename,'.');
	if (pDot)
	{
		CString fileExtension = pDot;
		fileExtension.MakeUpper();
		if (fileExtension != ".BGL")
		{
			LoadWayPoint(filename);
			return(0);
		}

	}

	// find file size
	WIN32_FIND_DATA fd;
	HANDLE findHandle = FindFirstFile(filename, &fd);
	if (findHandle == INVALID_HANDLE_VALUE) {
		sprintf(buffer, "Could not open %s\n", filename);
		AfxMessageBox(buffer);
		return(2);
	}	
	int size = (int) fd.nFileSizeLow; // assume its not more than 32 bits
	FindClose(findHandle);

	filehandle = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_READONLY, NULL);
	if (filehandle == INVALID_HANDLE_VALUE) {
		sprintf(buffer, "Could not open %s\n", filename);
		AfxMessageBox(buffer);
		return(2);
	}	

	filemapping = CreateFileMapping(filehandle, NULL, PAGE_READONLY, 0, 0, "bglfile");
	DWORD err = GetLastError();
	if ((filemapping != NULL) && (err == ERROR_ALREADY_EXISTS)) {
		sprintf(buffer, "File mapping failed, object already exists\n");
		AfxMessageBox(buffer);
		CloseHandle(filemapping);
		CloseHandle(filehandle);
		return(3);
	} else if (filemapping == NULL) {
		sprintf(buffer, "File mapping failed, error code %d\n", err);
		AfxMessageBox(buffer);
		CloseHandle(filehandle);
		return(3);
	}	

	bgl = (char *) MapViewOfFile(filemapping, FILE_MAP_READ, 0, 0, 0); 
	if (bgl == NULL) {
		CloseHandle(filemapping);
		CloseHandle(filehandle);
		AfxMessageBox("Could not map file\nMaybe your virtual memory setting is too low");
		return(4);
	}
	// finished mapping file to memory

  	int sect0  = *((int *) &bgl[18]);  //used
  	int sect9  = *((int *) &bgl[58]);  //used
	int sect10 = *((int *) &bgl[62]);
 	int sect11 = *((int *) &bgl[66]);  //used 
  	int sect12 = *((int *) &bgl[70]);
  	int sect13 = *((int *) &bgl[74]);  //used
 	int sect14 = *((int *) &bgl[78]);  //used
  	int sect15 = *((int *) &bgl[82]);
 	int sect16 = *((int *) &bgl[102]); //used
  	int sect17 = *((int *) &bgl[106]);
  	int sect18 = *((int *) &bgl[110]); //used
  	int sect19 = *((int *) &bgl[114]);
  	int sect20 = *((int *) &bgl[118]);
  	int sect21 = *((int *) &bgl[122]);

	if (sect0 > size)  sect0 = 0;	// limit damage caused by bogus headers
	if (sect9 > size)  sect9 = 0;	
	if (sect10 > size) sect10 = 0;	
	if (sect11 > size) sect11 = 0;	
	if (sect12 > size) sect12 = 0;	
	if (sect13 > size) sect13 = 0;	
	if (sect14 > size) sect14 = 0;	
	if (sect15 > size) sect15 = 0;	
	if (sect16 > size) sect16 = 0;	
	if (sect17 > size) sect17 = 0;	
	if (sect18 > size) sect18 = 0;	
	if (sect19 > size) sect19 = 0;	
	if (sect20 > size) sect20 = 0;	
	if (sect21 > size) sect21 = 0;	

	// try to find where section 9 ends
	if ((sect0  > sect9) && (sect0 < size))   size = sect0;
	if ((sect10 > sect9) && (sect10 < size))  size = sect10;
	if ((sect11 > sect9) && (sect11 < size))  size = sect11;
	if ((sect12 > sect9) && (sect12 < size))  size = sect12;
	if ((sect13 > sect9) && (sect13 < size))  size = sect13;
	if ((sect14 > sect9) && (sect14 < size))  size = sect14;
	if ((sect15 > sect9) && (sect15 < size))  size = sect15;
	if ((sect16 > sect9) && (sect16 < size))  size = sect16;
	if ((sect17 > sect9) && (sect17 < size))  size = sect17;
	if ((sect18 > sect9) && (sect18 < size))  size = sect18;
	if ((sect19 > sect9) && (sect19 < size))  size = sect19;
	if ((sect20 > sect9) && (sect20 < size))  size = sect20;
	if ((sect21 > sect9) && (sect21 < size))  size = sect21;

	// section 14: NDB
	if (sect14 != 0)	{
		int rangeTableOffset = 0;
		// for each latitude range
		while (bgl[sect14 + rangeTableOffset] == 0x15) {
			short minLat = *((UNALIGNED short *) &bgl[sect14 + rangeTableOffset+1]);
			short maxLat = *((UNALIGNED short *) &bgl[sect14 + rangeTableOffset+3]);
			int ndbOffset = *((UNALIGNED int *) &bgl[sect14 + rangeTableOffset+5]);
			// for each station in this range
			while (bgl[sect14 + ndbOffset] == 0x04) {
				int lat = read3bytes(&bgl[sect14 + ndbOffset + 4]);
				int lon = read3bytes(&bgl[sect14 + ndbOffset + 7]);
				// many ndb altitudes are -9999, why?
				char freqbcd = *((char *) &bgl[sect14 + ndbOffset + 1]);
				char freqhun = *((char *) &bgl[sect14 + ndbOffset + 2]);
				char freqdec = *((char *) &bgl[sect14 + ndbOffset + 3]);
				char freqten = (freqbcd & 0xF0) >> 4;
				char freqone = (freqbcd & 0x0F);
				double freq = (double) freqhun * 100. + (double) freqten * 10. + (double) freqone + (double) freqdec * .1;
				short alt = *((UNALIGNED short *) &bgl[sect14 + ndbOffset + 10]);
				char range = *((char *) &bgl[sect14 + ndbOffset + 12]);
				char ndbID[6];
				strncpy(ndbID, &bgl[sect14 + ndbOffset + 13], 5);
				ndbID[5] = 0x00;
				char ndbName[25];
				strncpy(ndbName, &bgl[sect14 + ndbOffset + 18], 24);
				ndbName[24] = 0x00;
				addDB(NDB_TYPE, (double) lat * 90.0 / 5000875.0, (double) lon / (double) 0x1000000 * 360.0, alt, freq, ndbID, ndbName, 0, 0, (double) range * 2048. / 1000.); 				

				ndbOffset += 42;
				count++;
			}
			rangeTableOffset += 9;
		}
	}

	// section 13: ATIS
	if (sect13 != 0)	{
		int rangeTableOffset = 0;
		// for each latitude range
		while (bgl[sect13 + rangeTableOffset] == 0x15) {
			short minLat = *((UNALIGNED short *) &bgl[sect13 + rangeTableOffset+1]);
			short maxLat = *((UNALIGNED short *) &bgl[sect13 + rangeTableOffset+3]);
			int ndbOffset = *((UNALIGNED int *) &bgl[sect13 + rangeTableOffset+5]);
			// for each station in this range
			while (bgl[sect13 + ndbOffset] == 0x04) {
				char freqdec = *((char *) &bgl[sect13 + ndbOffset + 1]);
				char freqhun = *((char *) &bgl[sect13 + ndbOffset + 2]);
				double lat = read3bytes(&bgl[sect13 + ndbOffset + 3]) * 90.0 / 5000875.0;
				double lon = read3bytes(&bgl[sect13 + ndbOffset + 6]) / (double) 0x1000000 * 360.0;
				int range = *((char *) &bgl[sect13 + ndbOffset + 9]) * 2048; // in meters
				unsigned short recLength = *((unsigned short *) &bgl[sect13 + ndbOffset + 10]);
				char pr1 = *((char *) &bgl[sect13 + ndbOffset + 12]); // preferred runways
				char pr2 = *((char *) &bgl[sect13 + ndbOffset + 13]);
				char pr3 = *((char *) &bgl[sect13 + ndbOffset + 14]);
				char pr4 = *((char *) &bgl[sect13 + ndbOffset + 15]);
				unsigned int allRunways = *((unsigned int *) &bgl[sect13 + ndbOffset + 12]); 
				char freqtenth    = (freqdec & 0xF0) >> 4;
				char freqhundreth = (freqdec & 0x0F);
				char freqone = (freqhun & 0x0F);
				char freqten = (freqhun & 0xF0) >> 4;
				double freq = 100 + freqten * 10. + freqone + freqtenth * .1 + freqhundreth * .01;

				// generate DMS name, since ATIS don't have names
				char buf[26];
				char dirx, diry;
				if (lon < 0.) 
					dirx = 'W'; 
				else 
					dirx = 'E';
				if (lat < 0.) 
					diry = 'S'; 
				else 
					diry = 'N';
				int xdeg, xmin, ydeg, ymin;
				double xsec, ysec, gpslat, gpslon;
				gpslat = fabs(lat);
				gpslon = fabs(lon);
				xdeg = (int) gpslon;
				xmin = (int) ((gpslon - xdeg) * 60);
				xsec = (gpslon - xdeg - xmin / 60.0) * 3600;
				ydeg = (int) gpslat;
				ymin = (int) ((gpslat - ydeg) * 60);
				ysec = (gpslat - ydeg - ymin / 60.0) * 3600;
				sprintf(buf, "ATIS %d %d'%2.0f\"%c, %d %d'%2.0f\"%c", ydeg, ymin, ysec, diry, xdeg, xmin, xsec, dirx);
				CString name(buf);

				addDB(ATIS_TYPE, lat , lon , 0, freq, "ATIS ", name.GetBuffer(1), 0, 0, allRunways); 				
				name.ReleaseBuffer();
				ndbOffset += recLength;
				count++;
			}
			rangeTableOffset += 9;
		}
	}

	// section 0: VOR and ILS
    short minS0freq = *((short *) &bgl[22]);
    short maxS0freq = *((short *) &bgl[24]);
	if (sect0 != 0)	{
		// for each frequency
		for (int i=0; i<=maxS0freq-minS0freq; i++) {
			if(bgl[sect0+5*i] == 0x01) {
				int sectOffset = *((UNALIGNED int *) &bgl[sect0+5*i+1]);
				if (sectOffset != 0) {
					double freq = 108 + (minS0freq + i) * .05;
					int rangeTableOffset = 0;
					// for each latitude range
					while (bgl[sect0 + sectOffset + rangeTableOffset] == 0x15) {
						short minLat = *((UNALIGNED short *) &bgl[sect0 + sectOffset + rangeTableOffset+1]);
						short maxLat = *((UNALIGNED short *) &bgl[sect0 + sectOffset + rangeTableOffset+3]);
						int vorOffset = *((UNALIGNED int *) &bgl[sect0 + sectOffset + rangeTableOffset+5]);
						// for each station in this range
						while ((bgl[sect0 + vorOffset] == 0x04) || (bgl[sect0 + vorOffset] == 0x05)) {
							char range = *((char *) &bgl[sect0 + vorOffset + 1]);
							short magdev = *((UNALIGNED short *) &bgl[sect0 + vorOffset + 2]);
							int lat = read3bytes(&bgl[sect0 + vorOffset + 5]);
							int lon = read3bytes(&bgl[sect0 + vorOffset + 8]);
							short alt = *((UNALIGNED short *) &bgl[sect0 + vorOffset + 11]);
							char vorID[6];
							strncpy(vorID, &bgl[sect0 + vorOffset + 15], 5);
							vorID[5] = 0x00;
							char vorName[25];
							strncpy(vorName, &bgl[sect0 + vorOffset + 20], 24);
							vorName[24] = 0x00;

							// read additional info for ILS
							if (bgl[sect0 + vorOffset] == 0x05)	{
								short approach = *((UNALIGNED short *) &bgl[sect0 + vorOffset + 13]);
								int gslat = read3bytes(&bgl[sect0 + vorOffset + 44]);
								int gslon = read3bytes(&bgl[sect0 + vorOffset + 47]);
								short gsalt = *((UNALIGNED short *) &bgl[sect0 + vorOffset + 50]);
								addDB(ILS_TYPE, (double) gslat * 90.0 / 5000875.0, (double) gslon / (double) 0x1000000 * 360.0, gsalt, freq, vorID, vorName, (double) magdev * 360./ 0x10000, (double) approach * 360. / 0x10000, range); 				
							} else {
								addDB(VOR_TYPE, (double) lat * 90.0 / 5000875.0, (double) lon / (double) 0x1000000 * 360.0, alt, freq, vorID, vorName, (double) magdev * 360./ 0x10000, 0, range); 				
							}
							count++;
							if (bgl[sect0 + vorOffset] == 0x05)	
								vorOffset += 54;
							else 
								vorOffset += 44;
						}
						rangeTableOffset += 9;
					}
				}
			}
		}
	}

	// section 16: airports
	if (sect16 != 0)	{
		int rangeTableOffset = 0;
		// for each latitude range
		while (bgl[sect16 + rangeTableOffset] == 0x15) {
			short minLat = *((UNALIGNED short *) &bgl[sect16 + rangeTableOffset + 1]);
			short maxLat = *((UNALIGNED short *) &bgl[sect16 + rangeTableOffset + 3]);
			int areaOffset = *((UNALIGNED int *) &bgl[sect16 + rangeTableOffset + 5]);
			// for each area
			while ((bgl[sect16 + areaOffset] > 0x03) && (bgl[sect16 + areaOffset] < 0x08)) {
				char areaRec = *((char *) &bgl[sect16 + areaOffset + 1]);
				// check to see if its an airport record
				if (bgl[sect16 + areaOffset] == 0x04) {	// inner marker
					int lat = read3bytes(&bgl[sect16 + areaOffset + 2]);
					int lon = read3bytes(&bgl[sect16 + areaOffset + 5]);
					short alt = *((UNALIGNED short *) &bgl[sect16 + areaOffset + 8]);
					addDB(INNER_MARKER_TYPE, (double) lat * 90.0 / 5000875.0, (double) lon / (double) 0x1000000 * 360.0, alt, 0, "", "", 0, 0, 0); 				
				}
				if (bgl[sect16 + areaOffset] == 0x05) {	// middle marker
					int lat = read3bytes(&bgl[sect16 + areaOffset + 2]);
					int lon = read3bytes(&bgl[sect16 + areaOffset + 5]);
					short alt = *((UNALIGNED short *) &bgl[sect16 + areaOffset + 8]);
					addDB(MIDDLE_MARKER_TYPE, (double) lat * 90.0 / 5000875.0, (double) lon / (double) 0x1000000 * 360.0, alt, 0, "", "", 0, 0, 0); 				
				}
				if (bgl[sect16 + areaOffset] == 0x06) {	// outer marker
					int lat = read3bytes(&bgl[sect16 + areaOffset + 2]);
					int lon = read3bytes(&bgl[sect16 + areaOffset + 5]);
					short alt = *((UNALIGNED short *) &bgl[sect16 + areaOffset + 8]);
					addDB(OUTER_MARKER_TYPE, (double) lat * 90.0 / 5000875.0, (double) lon / (double) 0x1000000 * 360.0, alt, 0, "", "", 0, 0, 0); 				
				}
				if (bgl[sect16 + areaOffset] == 0x07) {	// airport
					int lat = read3bytes(&bgl[sect16 + areaOffset + 2]);
					int lon = read3bytes(&bgl[sect16 + areaOffset + 5]);
					short alt = *((UNALIGNED short *) &bgl[sect16 + areaOffset + 9]);
  					char airportName[25];
					strncpy(airportName, &bgl[sect16 + areaOffset + 22], 24);
					airportName[24] = 0x00;
					int len = (int) strlen(airportName) + 1;
					// primary runways?
					int rid1 = bgl[sect16 + areaOffset + 22 + len] & 0x1F;
					int rid2 = bgl[sect16 + areaOffset + 23 + len] & 0x1F;
					// check to make sure this airport has not already been found
					for (int n=0; n<dbCount; n++) {
						if (db[n].type == 4) {
							// same name
							if (CString(airportName).CompareNoCase(db[n].name) == 0)
								goto alreadyGotIt;
							// so close they must? be the same
							if (fabs(db[n].lat-lat) < 1) {		// quick check - within 1 degree
								if (fabs(db[n].lon-lon) < 1) {
									if (v->gcDist(db[n].lat, db[n].lon, lat, lon) < 5000)
										goto alreadyGotIt;
								}
							}
						}
					}
					count++;
					addDB(AIRPORT_TYPE, (double) lat * 90.0 / 5000875.0, (double) lon / (double) 0x1000000 * 360.0, alt, 0, "", airportName, 0, 0, 0); 				
					alreadyGotIt:;
				}
				areaOffset += areaRec;
			}
			rangeTableOffset += 9;
		}
	}

	// section 11: airports & runways (menu items)
	if (sect11 != 0)	{
		int areaOffset = 0;
		// for each area
		while (bgl[sect11 + areaOffset] == 0x01) {
			short areaRec = *((UNALIGNED short *) &bgl[sect11 + areaOffset + 1]);
			int runwayTable = *((UNALIGNED int *) &bgl[sect11 + areaOffset + 3]);
			char* areaName = &bgl[sect11 + areaOffset + 7]; //menu section
			// for each runway in the area
			int runwayOffset = 0;
			while (bgl[sect11 + runwayTable + runwayOffset] == 0x03) {
				short runwayRec = *((UNALIGNED short *) &bgl[sect11 + runwayTable + runwayOffset + 1]);
				char* runwayName = &bgl[sect11 + runwayTable + runwayOffset + 27]; //menu item
				double lat = readDM(&bgl[sect11 + runwayTable + runwayOffset + 3]);
				double lon = readDM(&bgl[sect11 + runwayTable + runwayOffset + 7]);
				double alt = *((UNALIGNED int *) &bgl[sect11 + runwayTable + runwayOffset + 11]) / 256;
				lat *= 90.0 / 5000875.0;
				lon = lon / (double) 0x1000000 * 360;

				// trim runway from name
				CString airport = runwayName;
				int end = airport.Find("- Runway");
				if (end<0)  // can't find "Runway", try other method
					end = airport.ReverseFind('-');
				if (end<0)  // still can't find anything, try Scotland methods
					end = airport.Find("Rwy");	
				if (end<0)  // still can't find anything, try Scotland methods
					end = airport.Find("at");	
				if (end > 0)
					airport = airport.Mid(0, end);
				airport.TrimRight();
				// check to make sure this airport has not already been found
				// need to rewrite this using hash tables (see CMap)
				for (int n=0; n<dbCount; n++) {
					if (db[n].type == 4) {
						// same name
						if (airport.CompareNoCase(db[n].name) == 0)
							goto alreadyGotOne;
						// so close they must? be the same
						if (fabs(db[n].lat-lat) < 1) {		// quick check - within 1 degree
							if (fabs(db[n].lon-lon) < 1) {
								if (v->gcDist(db[n].lat, db[n].lon, lat, lon) < 5000)
									goto alreadyGotOne;
							}
						}
					}
				}
				addDB(AIRPORT_TYPE, lat, lon, (short) alt, 0, "", airport.GetBuffer(1), 0, 0, 0); 							
				airport.ReleaseBuffer();
				alreadyGotOne:;
				count++;
				runwayOffset += runwayRec;
			}
			areaOffset += areaRec;
		}
	}

	// section 9: runways
	if (sect9 != 0)	{
		// scan all of section 9 looking for the runway pattern
		for (int i=0; i<(size-sect9); i++) {
			char rwRec = *((char *) &bgl[sect9 + i]);
			if (rwRec == 68) {
				char j1 = *((char *) &bgl[sect9 + i + 1]);
				if (j1 != 0) continue;
				char j2 = *((char *) &bgl[sect9 + i + 18]);
				if (j2 != 0) continue;
				char j3 = *((char *) &bgl[sect9 + i + 19]);
				if (j3 != 0) continue;
				char j4 = *((char *) &bgl[sect9 + i + 29]);
				if (j4 != 0) continue;
				// lat and lon are center of runway
				double lat = readMicro6Lat(&bgl[sect9 + i + 2]);
				lat *= 45.0 / 5000875.0;
				double lon = readMicro6Lon(&bgl[sect9 + i + 8]);
				lon = lon / (double) 0x1000000 * 360.0;	  
				double alt = readMicro4(&bgl[sect9 + i + 14]);
				double course = *((UNALIGNED short *) &bgl[sect9 + i + 20]) * 360.0 / 0x10000;
				if (course < 0.0)
					course += 360.0;
				short length = *((UNALIGNED short *) &bgl[sect9 + i + 22]);
				short width = *((UNALIGNED short *) &bgl[sect9 + i + 24]);
				unsigned char rwID = *((unsigned char *) &bgl[sect9 + i + 27]);
				char rwID2 = ' ';
				if (rwID > 192) {
					rwID -= (unsigned char) 192;
					rwID2 = 'C';
				} else if (rwID > 128) {
					rwID -= (unsigned char) 128;
					rwID2 = 'R';
				} else if (rwID > 64) {
					rwID -=  (unsigned char) 64;
					rwID2 = 'L';
				}
				char surface = *((char *) &bgl[sect9 + i + 30]);

				// the procedure is far from perfect, so throw out "obviously" bad records
				if (rwID == 0) continue; // should be 36 for a valid runway
				if (fabs(lat) > 90.0) continue;
				if (fabs(lon) > 180.0) continue;
				if (alt > 10000) continue;
				if ((length <= 0) || (length > 20000)) continue;
				if (width <= 20) continue;
				if ((surface > 5) || (surface <0)) continue;

				char rwName[6];
				int c;
				c=sprintf(rwName, "%d", (unsigned) rwID);
				sprintf(&rwName[c], "%c", rwID2);
				addDB(RUNWAY_TYPE, lat, lon, (int) alt, (double) surface, rwName, "Runway", (double) width, course, (double) length); 

				i+=29;
				count++;
			}
		}
	}

	// section 18: MagVar info (should only be in World3.bgl or Magdec.bgl)
	if (sect18 != 0) {
		magVarTable.SetSize(181*360,1);
		unsigned short deg;
		for (int lat = -90; lat <=90; lat++) {
			for (int lon = 0; lon <360; lon++) {
				deg = *((unsigned short *) &bgl[8 + sect18 + ((lon * 181) + (lat + 90)) * 2]);	
				magVarTable[lat+90 + 181*lon] = deg; 
			}
		}
		// save magvar table here
		FILE *mv;
		POSITION p = GetFirstViewPosition();
		CNavView *v = (CNavView *) GetNextView(p);
		CString fn = v->orgDir + "\\magvar.nav";
		mv = fopen((LPCTSTR) fn, "wb");
		fwrite(&bgl[sect18],2,181*360,mv); 
		fclose(mv);
		magVarTableExists = TRUE;
	}

	// unmap file and close
	if (UnmapViewOfFile(bgl) == FALSE)
		AfxMessageBox("UnmapViewOfFile failed");
	if (CloseHandle(filemapping) == FALSE)
		AfxMessageBox("Could not close filemapping handle");
	if (CloseHandle(filehandle) == FALSE)
		AfxMessageBox("Could not close file handle");
	return(0);
}

void CNavDoc::OnFileAdd() {
	CString filelist[MaxDBfiles], path;
	int count = 0;
	CString name;
	char c;

	// build existing file list

	POSITION p = GetFirstViewPosition();
	CNavView *v = (CNavView *) GetNextView(p);
	CString fn = v->orgDir + "\\" + listFileName;
	FILE *fp = fopen((LPCTSTR) fn,"r");
	if (fp != NULL) {

		int go = TRUE;
		while (go) {		
			name = "";	// more convoluted to handle directory names with spaces
			while ((c=fgetc(fp)) != '\n') {
				if (c == EOF) {
					go = FALSE;
					break;
				}
				name = name + c;
			}
			if (name == "" && go == FALSE)
				break;
			if (name != "") {
				path = name;
				path.MakeLower();
				count++;
				if (count >= MaxDBfiles) {
					AfxMessageBox("Maximum number of scenery list files exceeded");
					AfxAbort();
				}
				filelist[count] = path;
			}
		}
		fclose(fp);

		// make sure its not already in the list
		fName.MakeLower();
		for (int i=0; i<count; i++) {
			if (filelist[i] == fName)
				return;;
		}
	}

	// add it
	int ch = 'a';
	fp = fopen((LPCTSTR) fn,"a");
	char *b = fName.GetBuffer(1);	
	fName.ReleaseBuffer();
	for(; (ch != EOF) && (*b != '\0'); b++ )
		ch = putc( *b, fp);
	putc('\n', fp);
	fclose(fp);
}

void CNavDoc::OnUpdateFileAdd(CCmdUI* pCmdUI) {
	pCmdUI->Enable(fName != "");			
}

void CNavDoc::OnFileLoad() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CString loadfile, name;
	dbCount = 0;
	db.RemoveAll();
 	db.FreeExtra();
	db.SetSize(0,5000);
	POSITION p = GetFirstViewPosition();
	CNavView *v = (CNavView *) GetNextView(p);
	v->mag = 1;
	v->mag2 = 1;
	CString fn = v->orgDir + "\\" + listFileName;
	FILE *fp = fopen((LPCTSTR) fn,"r");
	if (fp == NULL) {
		AfxMessageBox("Could not load scenery list from " + listFileName);
		return;
	}
	
	dbFileIndex = 0;

	char c;
	int go = TRUE;
	while (go) {		
		name = "";	// more convoluted to handle directory names with spaces
		while ((c=fgetc(fp)) != '\n') {
			if (c == EOF) {
				go = FALSE;
				break;
			}
			name = name + c;
		}
		if (name == "" && go == FALSE)
			break;
		if (name != "") {
			fName = name;
 			dbFiles[dbFileIndex] = fName;
			loadfile = "Loading " + fName;
			loadfile.TrimRight();
			main->SetStatusBar(loadfile);
			// take over message processing to simulate threading 
	//		MSG msg;  
	//		while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) { 
	//			if (msg.message == WM_QUIT) { 
	//		 		PostQuitMessage(1);
	//		 		return;      
	//			}	 
	//		 	TranslateMessage(&msg);
	//		 	DispatchMessage(&msg);
	//		}
			// continue
			readFile((LPCTSTR) name);
			dbFileIndex++;
			if (dbFileIndex >= MaxDBfiles) {
				AfxMessageBox("Maximum number of scenery list files exceeded");
				AfxAbort();
			}
		}
	}

	char buff[200];
	sprintf(buff, "%d files loaded, %d items in database", dbFileIndex, dbCount);
	((CMainFrame *) AfxGetMainWnd())->SetStatusBar(buff);
	fclose(fp);
	SetTitle("Scenery List: " + listFileName);
	fName = "";
	Index();
	v->ZoomToFit();
}

void CNavDoc::addDB(NavaidType_t type, double lat, double lon, int alt, double freq, char *id, char *name, double magdev, double approach, double range) {
	dbRec r;
	db.Add(r);

	db[dbCount].type = type;
	db[dbCount].lat = lat;
	db[dbCount].lon = lon;
	db[dbCount].alt = alt;
	db[dbCount].freq = freq;
	strncpy(db[dbCount].id, id, 6);
	strncpy(db[dbCount].name, name, 30);
	db[dbCount].name[30] = '\x00';
	db[dbCount].magdev = magdev;
	if (approach < 0)
		approach += 360;
	db[dbCount].approach = approach;
	db[dbCount].range = range;
	db[dbCount].file = dbFileIndex;
	dbCount++;
}

void CNavDoc::formatData(CString& details, int dbindex) {
	POSITION p = GetFirstViewPosition();
	CNavView *v = (CNavView *) GetNextView(p);
	char line[50];

	double lat = db[dbindex].lat;
	double lon = db[dbindex].lon;
	char dirx, diry;
	if (lon < 0.) 
		dirx = 'W'; 
	else 
		dirx = 'E';
	if (lat < 0.) 
		diry = 'S'; 
	else 
		diry = 'N';
	int xdeg, xmin, ydeg, ymin;
	double xsec, ysec;
	lat = fabs(lat);
	lon = fabs(lon);
	xdeg = (int) lon;
	xmin = (int) ((lon - xdeg) * 60);
	xsec = (lon - xdeg - xmin / 60.0) * 3600;
	ydeg = (int) lat;
	ymin = (int) ((lat - ydeg) * 60);
	ysec = (lat - ydeg - ymin / 60.0) * 3600;

	if (1 == db[dbindex].type) {
		sprintf(line, " NDB\r\n");
		details += line;
		sprintf(line, " Id: %s\r\n", db[dbindex].id);
		details += line;
		sprintf(line, " Full name: %s\r\n", db[dbindex].name);
		details += line;
		sprintf(line, " Frequency: %5.1f KHz\r\n", db[dbindex].freq);
		details += line;
		sprintf(line, " Latitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lat, diry, ydeg, ymin, ysec, diry);
		details += line;		 
		sprintf(line, " Longitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lon, dirx, xdeg, xmin, xsec, dirx);;
		details += line;
		sprintf(line, " Altitude: %d ft\r\n", (int) (db[dbindex].alt / .3048));
		details += line;
		sprintf(line, " Range: %d nm\r\n", (int) (db[dbindex].range/1.852));
		details += line;
	} else if (2 == db[dbindex].type) {
		sprintf(line, " ILS\r\n");
		details += line;
		sprintf(line, " Id: %s\r\n", db[dbindex].id);
		details += line;
		sprintf(line, " Full name: %s\r\n", db[dbindex].name);
		details += line;
		if (airportIndex.GetSize() > 0 ) {
			CString airport;
			double dist = 1e10, d;
	  		for (int i=0; i<airportIndex.GetSize(); i++) {
				d = v->gcDist(db[airportIndex[i]].lat, db[airportIndex[i]].lon, db[dbindex].lat, db[dbindex].lon); 
				if (d < dist) {
					dist = d;
					airport = db[airportIndex[i]].name;
				}
			}
			d = dist/1852;
			if (d > 5)
				sprintf(line, " Closest airport (%.1lf nm away): %s\r\n", d, (LPCTSTR) airport);
			else
				sprintf(line, " Closest airport: %s\r\n", (LPCTSTR) airport);
			details += line;
		}
		sprintf(line, " Frequency: %6.2f MHz\r\n", db[dbindex].freq);
		details += line;
		sprintf(line, " Latitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lat, diry, ydeg, ymin, ysec, diry);
		details += line;		 
		sprintf(line, " Longitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lon, dirx, xdeg, xmin, xsec, dirx);;
		details += line;
		sprintf(line, " Altitude: %d ft\r\n", (int) (db[dbindex].alt / .3048));
		details += line;
		sprintf(line, " Range: %d nm\r\n", (int) (db[dbindex].range/1.852));
		details += line;
		if (db[dbindex].magdev < 0.0)
			sprintf(line, " Mag Deviation: %.1f degrees (East)\r\n",  db[dbindex].magdev);
		else 
			sprintf(line, " Mag Deviation: %.1f degrees (West)\r\n",  db[dbindex].magdev);
		details += line;
		sprintf(line, " Approach course: %d degrees\r\n", (int) db[dbindex].approach);
		details += line;
	} else if (3 == db[dbindex].type) {
		sprintf(line, " VOR\r\n");
		details += line;
		sprintf(line, " Id: %s\r\n", db[dbindex].id);
		details += line;
		sprintf(line, " Full name: %s\r\n", db[dbindex].name);
		details += line;
		sprintf(line, " Frequency: %6.2f MHz\r\n", db[dbindex].freq);
		details += line;
		sprintf(line, " Latitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lat, diry, ydeg, ymin, ysec, diry);
		details += line;		 
		sprintf(line, " Longitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lon, dirx, xdeg, xmin, xsec, dirx);;
		details += line;
		sprintf(line, " Altitude: %d ft\r\n", (int) (db[dbindex].alt / .3048));
		details += line;
		sprintf(line, " Range: %d nm\r\n", (int) (db[dbindex].range/1.852));
		details += line;
		if (db[dbindex].magdev < 0.0)
			sprintf(line, " Mag Deviation: %.1f degrees (East)\r\n",  db[dbindex].magdev);
		else 
			sprintf(line, " Mag Deviation: %.1f degrees (West)\r\n",  db[dbindex].magdev);
		details += line;
	} else if (4 == db[dbindex].type) {
		sprintf(line, " Airport\r\n");
		details += line;
		sprintf(line, " Full name: %s\r\n", db[dbindex].name);
		details += line;
		sprintf(line, " Latitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lat, diry, ydeg, ymin, ysec, diry);
		details += line;		 
		sprintf(line, " Longitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lon, dirx, xdeg, xmin, xsec, dirx);;
		details += line;
		sprintf(line, " Altitude: %d ft\r\n", (int) (db[dbindex].alt / .3048));
		details += line;
	} else if (5 == db[dbindex].type) {
		sprintf(line, " Inner marker\r\n");
		details += line;
		sprintf(line, " Latitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lat, diry, ydeg, ymin, ysec, diry);
		details += line;		 
		sprintf(line, " Longitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lon, dirx, xdeg, xmin, xsec, dirx);;
		details += line;
		sprintf(line, " Altitude: %d ft\r\n", (int) (db[dbindex].alt / .3048));
		details += line;
	} else if (6 == db[dbindex].type) {
		sprintf(line, " Middle marker\r\n");
		details += line;
		sprintf(line, " Latitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lat, diry, ydeg, ymin, ysec, diry);
		details += line;		 
		sprintf(line, " Longitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lon, dirx, xdeg, xmin, xsec, dirx);;
		details += line;
		sprintf(line, " Altitude: %d ft\r\n", (int) (db[dbindex].alt / .3048));
		details += line;
	} else if (7 == db[dbindex].type) {
		sprintf(line, " Outer marker\r\n");
		details += line;
		sprintf(line, " Latitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lat, diry, ydeg, ymin, ysec, diry);
		details += line;		 
		sprintf(line, " Longitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lon, dirx, xdeg, xmin, xsec, dirx);;
		details += line;
		sprintf(line, " Altitude: %d ft\r\n", (int) (db[dbindex].alt / .3048));
		details += line;
	} else if (8 == db[dbindex].type) {
		sprintf(line, " Runway\r\n");
		details += line;
		sprintf(line, " ID: %s\r\n", db[dbindex].id);
		details += line;
		if (airportIndex.GetSize() > 0 ) {
			CString airport;
			double dist = 1e10, d;
	  		for (int i=0; i<airportIndex.GetSize(); i++) {
				d = v->gcDist(db[airportIndex[i]].lat, db[airportIndex[i]].lon, db[dbindex].lat, db[dbindex].lon); 
				if (d < dist) {
					dist = d;
					airport = db[airportIndex[i]].name;
				}
			}
			d = dist/1852;
			if (d > 5)
				sprintf(line, " Closest airport (%.1lf nm away): %s\r\n", d, (LPCTSTR) airport);
			else
				sprintf(line, " Closest airport: %s\r\n", (LPCTSTR) airport);
			details += line;
		}
		sprintf(line, " Latitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lat, diry, ydeg, ymin, ysec, diry);
		details += line;		 
		sprintf(line, " Longitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lon, dirx, xdeg, xmin, xsec, dirx);;
		details += line;
		sprintf(line, " Altitude: %d ft\r\n", (int) (db[dbindex].alt / .3048));
		details += line;
		sprintf(line, " Course: %d degrees\r\n", (int) db[dbindex].approach);
		details += line;
		sprintf(line, " Length: %d ft\r\n", (int) (db[dbindex].range ));
		details += line;
		sprintf(line, " Width: %d ft\r\n", (int) (db[dbindex].magdev ));
		details += line;
		if (0 == (int) db[dbindex].freq)
			sprintf(line, " Surface type: Mud \r\n");
		else if (1 == (int) db[dbindex].freq)
			sprintf(line, " Surface type: Concrete \r\n");
		else if (2 == (int) db[dbindex].freq)
			sprintf(line, " Surface type: Asphalt \r\n");
		else if (3 == (int) db[dbindex].freq)
			sprintf(line, " Surface type: Grass \r\n");
		else if (4 == (int) db[dbindex].freq)
			sprintf(line, " Surface type: Dirt \r\n");
		else
			sprintf(line, " Surface type: %d \r\n", (int) db[dbindex].freq);
		details += line;
	} else if (9 == db[dbindex].type) {
		sprintf(line, " GPS fix (user defined waypiont)\r\n");
		details += line;
		sprintf(line, " ID: %s\r\n", db[dbindex].id);
		details += line;
		sprintf(line, " Name: %s\r\n", db[dbindex].name);
		details += line;
		sprintf(line, " Latitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lat, diry, ydeg, ymin, ysec, diry);
		details += line;		 
		sprintf(line, " Longitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lon, dirx, xdeg, xmin, xsec, dirx);;
		details += line;
		if (db[dbindex].magdev < 0.0)
			sprintf(line, " Mag Deviation: %.1f degrees (East)\r\n",  db[dbindex].magdev);
		else 
			sprintf(line, " Mag Deviation: %.1f degrees (West)\r\n",  db[dbindex].magdev);
		details += line;
	} else if (10 == db[dbindex].type) {
		sprintf(line, " ATIS station\r\n");
		details += line;
		sprintf(line, " Frequency: %6.2f MHz\r\n", db[dbindex].freq);
		details += line;
		if (airportIndex.GetSize() > 0 ) {
			CString airport;
			double dist = 1e10, d;
	  		for (int i=0; i<airportIndex.GetSize(); i++) {
				d = v->gcDist(db[airportIndex[i]].lat, db[airportIndex[i]].lon, db[dbindex].lat, db[dbindex].lon); 
				if (d < dist) {
					dist = d;
					airport = db[airportIndex[i]].name;
				}
			}
			d = dist/1852;
			if (d > 5)
				sprintf(line, " Closest airport (%.1lf nm away): %s\r\n", d, (LPCTSTR) airport);
			else
				sprintf(line, " Closest airport: %s\r\n", (LPCTSTR) airport);
			details += line;
		}
		sprintf(line, " Nav generated name: %s\r\n", db[dbindex].name);
		details += line;
		sprintf(line, " Latitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lat, diry, ydeg, ymin, ysec, diry);
		details += line;		 
		sprintf(line, " Longitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lon, dirx, xdeg, xmin, xsec, dirx);;
		details += line;
		unsigned int allRunways = (unsigned int)db[dbindex].range;
		char pr4 = (allRunways & 0x3F000000) >> 24;
		char pr3 = (allRunways & 0x003F0000) >> 16;
		char pr2 = (allRunways & 0x00003F00) >> 8;
		char pr1 = (allRunways & 0x0000003F);
		char pr4c = (allRunways & 0xc0000000) >> 30;
		char pr3c = (allRunways & 0x00c00000) >> 22;
		char pr2c = (allRunways & 0x0000c000) >> 14;
		char pr1c = (allRunways & 0x000000c0) >> 6;
		pr1c==0?pr1c=' ':(pr1c==1?pr1c='L':(pr1c==2?pr1c='R':(pr1c='C'))); // ah, the joys of C
		pr2c==0?pr2c=' ':(pr2c==1?pr2c='L':(pr2c==2?pr2c='R':(pr2c='C')));
		pr3c==0?pr3c=' ':(pr3c==1?pr3c='L':(pr3c==2?pr3c='R':(pr3c='C')));
		pr4c==0?pr4c=' ':(pr4c==1?pr4c='L':(pr4c==2?pr4c='R':(pr4c='C')));
		sprintf(line, " Preferred runways:\r\n");
		details += line;
		sprintf(line, "   North wind: %d%c\r\n", pr1, pr1c);
		details += line;
		sprintf(line, "   East wind:  %d%c\r\n", pr2, pr2c);
		details += line;
		sprintf(line, "   South wind: %d%c\r\n", pr3, pr3c);
		details += line;
		sprintf(line, "   West wind:  %d%c\r\n", pr4, pr4c);
		details += line;
	}
	else if (11 == db[dbindex].type)
	{
		sprintf(line, " Way Point\r\n");
		details += line;
		sprintf(line, " Id: %s\r\n", db[dbindex].id);
		details += line;
		sprintf(line, " Full name: %s\r\n", db[dbindex].name);
		details += line;
		sprintf(line, " Latitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lat, diry, ydeg, ymin, ysec, diry);
		details += line;		 
		sprintf(line, " Longitude: % 8.6f %c (%d %d' %4.2f\" %c)\r\n", lon, dirx, xdeg, xmin, xsec, dirx);;
		details += line;
	}
	if (9 != db[dbindex].type )
		details += " File: " + dbFiles[db[dbindex].file];
}

void CNavDoc::OnFileSearch() {
	CArray<CString, CString> filelist;	 
	filelist.SetSize(0,50);

	CString name;
	int count = 0;
	int startCount;
	SerBgl d;

	d.m_recurse = TRUE;
	d.m_skip = TRUE;
	char buffer[100];
	GetPrivateProfileString("Doc", "lastSceneryDirectory", "c:\\", buffer, 100, "nav.ini");

	//d.m_drive = "c:\\";
	d.m_drive = buffer;
	again:;
	if (d.DoModal() == IDOK) {
		// check for a valid drive
		int result;
		CString path;
		path = d.m_drive;
		result = _access((LPCTSTR) path, 0);
		if (result == -1L) {
			char buf[300];
			sprintf(buf, "%s is not a valid directory", d.m_drive);
			AfxMessageBox(buf);
			goto again;
		}
		
		WritePrivateProfileString("Doc", "lastSceneryDirectory", d.m_drive, "nav.ini");

		// build existing file list
		POSITION p = GetFirstViewPosition();
		CNavView *v = (CNavView *) GetNextView(p);
		CString fn = v->orgDir + "\\" + listFileName;
		FILE *fp = fopen((LPCTSTR) fn,"r");
		if (fp != NULL) {
			char c;
			int go = TRUE;
			while (go) {		
				name = "";	// more convoluted to handle directory names with spaces
				while ((c=fgetc(fp)) != '\n') {
					if (c == EOF) {
						go = FALSE;
						break;
					}
					name = name + c;
				}
				if (name == "" && go == FALSE)
					break;
				if (name != "") {
					path = name;
					path.MakeLower();
					filelist.Add(path); 
					count++;
				}
			}
			fclose(fp);
		}

		CWaitCursor wait;	// display wait cursor
		startCount = count;
		findFile(d.m_drive, filelist, count, d.m_recurse, d.m_skip);

		// write new list to file
		int ch = 'a';
		fp = fopen((LPCTSTR) fn,"w");
		for(int i=0; i<count; i++) {
			char *p = filelist[i].GetBuffer(1);	
			filelist[i].ReleaseBuffer();
			for(; (ch != EOF) && (*p != '\0'); p++ )
				ch = putc(*p, fp);
			putc('\n', fp);
		}
		fclose(fp);

		CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
		if (d.m_skip) { 
			OnNewDocument(); // start off empty to clear loaded file
			dbCount = 0;
			db.RemoveAll();
			db.FreeExtra();
			db.SetSize(0,5000);
			SetTitle("Untitled");
			UpdateAllViews(NULL);
		}
		main->SetStatusBar("Ready");

		char buffer[50];
		sprintf(buffer, "%d new files found and added to scenery list %s", count-startCount, (LPCTSTR) listFileName);
		AfxMessageBox(buffer);
	}			
	filelist.RemoveAll();
	filelist.FreeExtra();
}

// recursively search for bgl files
void CNavDoc::findFile(CString path, CArray<CString, CString> &exist, int &count, int recurse, int skip) {
	if (path[path.GetLength()-1] == '\\')	// remove trailing backslash
		path = path.Left(path.GetLength()-1);

	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	char buffer[300];
	sprintf(buffer, "Searching %s", path);
	main->SetStatusBar(buffer);

	// take over message processing to simulate threading 
//	MSG msg;  
//	while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) { 
//		if (msg.message == WM_QUIT) { 
//	 		PostQuitMessage(1);
//			return;      
//		}	 
//	 	TranslateMessage(&msg);
//	 	DispatchMessage(&msg);
//	}

	int result, handle;
	_finddata_t fd;
	CString startPath, subPath;
	startPath = path + "\\*.*";
	result = handle = _findfirst((LPCTSTR) startPath, &fd);
	while ((result != -1L) && recurse) { // look for directories
		if (fd.attrib &	_A_SUBDIR) {
			if ((strcmp(fd.name, ".") != 0) && (strcmp(fd.name, "..") != 0)) {
				subPath = path + "\\" + fd.name;
				findFile(subPath, exist, count, recurse, skip);
			}
		}
		result = _findnext(handle, &fd);
	}
	_findclose(handle);
	// find files now
	startPath = path + "\\*.bgl";
	result = handle = _findfirst((LPCTSTR) startPath, &fd);
	while (result != -1) { 
		subPath = path + "\\" + fd.name;
		subPath.MakeLower();
		// make sure its not already in the list
		int found = FALSE;
		for (int i=0; i<count; i++) {
			if (exist[i] == subPath)
				found = TRUE;
		}
		if (found == FALSE) {
			if (skip) {
				// load the file and see if it adds anything to the database
				dbCount = 0;
				db.RemoveAll();
 				db.FreeExtra();
				db.SetSize(0,5000);
				readFile(subPath);
				SetTitle(subPath);
				if (dbCount > 0) { // true if it added stuff
					exist.Add(subPath);
					count++;
				}
			} else {
				exist.Add(subPath);
				count++;
			}
		}		
		result = _findnext(handle, &fd);
	}
	_findclose(handle);
}

void CNavDoc::OnUpdateFileExport(CCmdUI* pCmdUI) {
	pCmdUI->Enable(dbCount > 0);
}

void CNavDoc::OnFileExport() {
	((CMainFrame *) AfxGetMainWnd())->SetStatusBar("Exporting (can take a long time)...");
	CExport d;
	d.m_center = "";
	d.m_state  = "";
	if (d.DoModal() == IDOK) {
		CWaitCursor wait;
		if (d.m_center == "")
			d.m_center = "            ";
		if (d.m_state == "")
			d.m_state = "    ";
		FILE *fpn = fopen("nav.n", "w");
		FILE *fpa = fopen("nav.a", "w");
		FILE *fpr = fopen("nav.r", "w");
		if (d.m_headings != FALSE) { // useful column headings for debugging
			fprintf(fpn,"         1         2         3         4         5         6         7         8         9         0         1         2\n");				
			fprintf(fpn,"123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123\n");	
			fprintf(fpn,"ID    Name                                 St  Latitude        Longitude        MagV Elev    ARTCC          Freq    Type\n");				
			fprintf(fpa,"         1         2         3         4         5         6         7         8         9         0         1         2\n");				
			fprintf(fpa,"123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123\n");	
			fprintf(fpa,"ID    Name                                 St  Latitude        Longitude        MagV Elev    ARTCC         UNICOM   CTAF\n");				
			fprintf(fpr,"         1         2         3         4         5         6         7         8\n");				
			fprintf(fpr,"12345678901234567890123456789012345678901234567890123456789012345678901234567890\n");	
			fprintf(fpr,"ID    Rwy Nos    Len   Width R1# R1h R1ILS-t   R1ILS-f R2# R2h R2-ILS-t  R2ILS-f\n");				
		}
		char line[150];
		int totalILS;
		CArray<dbRec, dbRec> runways;
		runways.SetSize(0,200);
		CArray<dbRec, dbRec> ils;
		ils.SetSize(0,200);
		CArray<dbRec, dbRec> airports;
		airports.SetSize(0,200);
		CArray<CString, CString> AID;
		AID.SetSize(0,200);
		MSG msg; 
		POSITION p = GetFirstViewPosition();
		CNavView *v = (CNavView *) GetNextView(p);
		int maxIls;

		ExportProgress *prog;
		prog = new ExportProgress();
		prog->Create(IDD_EXPORTPROGRESS);
		prog->ShowWindow(SW_SHOW);

		for (int i=0; i<dbCount; i++) {
			if (prog->cancelled != FALSE)
				goto cancelled;
			prog->m_navaids.SetPos((100*i)/dbCount);

			if (v->flatWorld) {  // limit export to current window (not implemented for sphere world)
	 			if (db[i].lon < v->left) continue;	 
				if (db[i].lon > v->right) continue;
				if (db[i].lat < v->bottom) continue;
				if (db[i].lat > v->top) continue;
			}

			// take over message processing to simulate threading 
			while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) { 
				if (msg.message == WM_QUIT) { 
		 			PostQuitMessage(1);
		 			return;      
				}	 
		 		TranslateMessage(&msg);
		 		DispatchMessage(&msg);
			}
			// continue
			memset(line, ' ', 150);
			int type = db[i].type;
			double lat = fabs(db[i].lat);
			double lon = fabs(db[i].lon);
			int latd = (int) (lat);
			int latm = (int) ((lat-latd) * 60);
			double lats = ((lat-latd)*60-latm)*60;
			int lond = (int) (lon);
			int lonm = (int) ((lon-lond) * 60);
			double lons = ((lon-lond)*60-lonm)*60;
			char dirx, diry;
			if (db[i].lon < 0.) 
				dirx = 'W'; 
			else 
				dirx = 'E';
			if (db[i].lat < 0.) 
				diry = 'S'; 
			else 
				diry = 'N';
			if ((type == 1) || (type == 3)) {  // NDB, VOR
				sprintf(line,    "%s      ", db[i].id);
				sprintf(line+6,  "%s                                        ", db[i].name);
				sprintf(line+43, "%s   ", d.m_state);
				sprintf(line+47, "%c-%02d-%02d-%06.3lf    ", diry, latd, latm, lats);
				sprintf(line+63, "%c-%03d-%02d-%06.3lf    ", dirx, lond, lonm, lons);
				double mag = db[i].magdev;
				if (type != 3)  
					mag = GetMagVar(db[i].lat, db[i].lon);
				char dirm = 'E';
				if (mag > 0)
					dirm = 'W';
				sprintf(line+80, "%c%02d    ", dirm, abs((int) mag));
				if (db[i].alt != -9999)
					sprintf(line+85, "%06d    "  , (int) (db[i].alt / .3048));
				else  // -9999 is apparently used to signify an unknown value, don't convert it to feet
					sprintf(line+85, "%06d    "  , (int) (db[i].alt));
				sprintf(line+93, "%s               "  , d.m_center);
				sprintf(line+108,"%7.3lf   "  , db[i].freq);
				if (type == 1) // NDB
					sprintf(line+116,"NDB    "  , db[i].freq);
				else // can't tell VOR type, but an entry is required
					sprintf(line+116,"VOR/UNK"  , db[i].freq);
				line[123] = '\0';
				fprintf(fpn, "%s\n", line);
			} else if (type == 4) {	 // Airport	(just add to database this pass)
				airports.Add(db[i]);
				// I don't know if ID is hidden somewhere in the BGL file, so I will generate one
				CString ID = GenUniqueId();
				AID.Add(ID);
			} else if (type == 8) {  // Runway (just add to database this pass)
				runways.Add(db[i]);
			} else if (type == 2) {  // ILS    (just add to database this pass)
				ils.Add(db[i]);
			}
		}
		totalILS = ils.GetSize();
		// runway pass
	  	for (i=0; i<runways.GetSize(); i++) {
			if (prog->cancelled != FALSE)
				goto cancelled;
			prog->m_runways.SetPos((100*i)/runways.GetSize());
			// take over message processing to simulate threading 
			while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) { 
				if (msg.message == WM_QUIT) { 
		 			PostQuitMessage(1);
		 			return;      
				}	 
		 		TranslateMessage(&msg);
		 		DispatchMessage(&msg);
			}
			// continue
			memset(line, ' ', 150);
			CString ID;
			// scan existing airports for one within 3 nm of this runway
			int found = FALSE;
			for (int j=0; j<airports.GetSize(); j++) {
				double dist = v->gcDist(airports[j].lat, airports[j].lon, runways[i].lat, runways[i].lon);
				if (dist/1852.0 < 3.0) {
					found = TRUE;
					ID = AID[j];  // use ID from found airport
					break;
				}
			}
			if (found == FALSE) { // else generate new ID and new airport
				ID = GenUniqueId();
				AID.Add(ID);
				dbRec a;
				a.type = AIRPORT_TYPE;
				a.lat = runways[i].lat;
				a.lon = runways[i].lon;
				a.alt = runways[i].alt;
				strncpy(a.id, (LPCTSTR) ID, 6);
				CString name = "Name Unknown";
				strncpy(a.name, (LPCTSTR) name, 25);
				airports.Add(a);
			}
			sprintf(line,    "%s      ", ID);
			int rwID1;
			char rwID2 = ' ';
			sscanf(runways[i].id, "%d", &rwID1);
			if (((runways[i].id)[1] == 'L') || ((runways[i].id)[2] == 'L')) rwID2 = 'L';
			if (((runways[i].id)[1] == 'R') || ((runways[i].id)[2] == 'R')) rwID2 = 'R';
			if (((runways[i].id)[1] == 'C') || ((runways[i].id)[2] == 'C')) rwID2 = 'C';
			int rwApp = (int) runways[i].approach;
			if(rwID1 > 18) {
				rwApp = (rwApp + 180) % 360;
				if (rwApp == 0) rwApp = 360;
				rwID1 -= 18;
				if (rwID2 == 'L') rwID2 = 'R';
				else if (rwID2 == 'R') rwID2 = 'L';
			}
			int rwAppo = (rwApp + 180) % 360;
			if (rwAppo == 0) rwAppo = 360;
			int rwID1o = rwID1 + 18;
			char rwID2o = ' ';
			if (rwID2 == 'L') rwID2o = 'R';
			else if (rwID2 == 'R') rwID2o = 'L';
			else if (rwID2 == 'C') rwID2o = 'C';
			if (rwID2 == ' ')
				sprintf(line+6 ,    "%02d/%02d              ", rwID1, rwID1o);
			else
				sprintf(line+6 ,    "%02d%c/%02d%c              ", rwID1, rwID2, rwID1o, rwID2o);

			// scan existing ils for one within 0.6 nm of each end of this runway
			int f1found, f2found;
			f1found = f2found = FALSE;
			double f1, f2;
			int maxIls = ils.GetSize();
			for (j=0; j<maxIls; j++) {
				double dist = v->gcDist(ils[j].lat, ils[j].lon, runways[i].lat, runways[i].lon)/1852.0; // in nm
				if (dist < 3.0) {  // first just check to see if its close
					double d2 = (runways[i].range/2) * .3046 / 1852.; // half runway length in nm
					if (fabs(dist - d2) < 0.6) { // ils is in the proper ring around the runway center
						if (fabs(ils[j].approach - v->bearing) <= 5.0) { // heading is within 5 degrees
							double head = (ils[j].approach - rwApp + 360.0);
							if (head > 360.0)
								head -= 360.0;
							if ((head <= 5.0) || (head >= 355.0)) { // its the first heading
								f1found = TRUE;
								f1 = ils[j].freq;
								ils.RemoveAt(j); // remove this ils so it will not be used again
								j--;
								maxIls--;
							} else if ((head <= 185.0) || (head >= 175.0)) { // its the second heading								  // its the second heading
								f2found = TRUE;
								f2 = ils[j].freq;
								ils.RemoveAt(j); // remove this ils so it will not be used again
								j--;
								maxIls--;
							}
							if (f1found && f2found)
								break;
						}
					}
				}
			}
			
			sprintf(line+17,    "%05d      ", (int) runways[i].range);  // really length
			sprintf(line+23,    "%05d      ", (int) runways[i].magdev); // really width
			sprintf(line+29,    "%02d%c    ", rwID1, rwID2); 
			sprintf(line+33,    "%03d                           ", rwApp); 
			if (f1found) {
				// can't tell type, but an entry is required
				sprintf(line+37,"ILS/Unk         "); 
				sprintf(line+47,"%7.3lf          ", f1); 
			}
			sprintf(line+55,    "%02d%c          ", rwID1o, rwID2o); 
			sprintf(line+59,    "%03d                           ", rwAppo); 
  			if (f2found) {
				// can't tell type, but an entry is required
				sprintf(line+63,"ILS/Unk         "); 
				sprintf(line+73,"%7.3lf           ", f2); 
			}
			line[80] = '\0';
			fprintf(fpr, "%s\n", line);
		}
		// airport pass
	  	for (i=0; i<airports.GetSize(); i++) {
			if (prog->cancelled != FALSE)
				goto cancelled;
			prog->m_airports.SetPos((100*i)/airports.GetSize());
			// take over message processing to simulate threading 
			while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) { 
				if (msg.message == WM_QUIT) { 
		 			PostQuitMessage(1);
		 			return;      
				}	 
		 		TranslateMessage(&msg);
		 		DispatchMessage(&msg);
			}
			// continue
			memset(line, ' ', 150);
			int type = airports[i].type;
			double lat = fabs(airports[i].lat);
			double lon = fabs(airports[i].lon);
			int latd = (int) (lat);
			int latm = (int) ((lat-latd) * 60);
			double lats = ((lat-latd)*60-latm)*60;
			int lond = (int) (lon);
			int lonm = (int) ((lon-lond) * 60);
			double lons = ((lon-lond)*60-lonm)*60;
			char dirx, diry;
			if (airports[i].lon < 0.) 
				dirx = 'W'; 
			else 
				dirx = 'E';
			if (airports[i].lat < 0.) 
				diry = 'S'; 
			else 
				diry = 'N';
			CString ID = AID[i];
			sprintf(line,    "%s      ", ID);
			sprintf(line+6,  "%s                                        ", airports[i].name);
			sprintf(line+43, "%s   ", d.m_state);
			sprintf(line+47, "%c-%02d-%02d-%06.3lf    ", diry, latd, latm, lats);
			sprintf(line+63, "%c-%03d-%02d-%06.3lf    ", dirx, lond, lonm, lons);
			double mag = airports[i].magdev;
			mag = GetMagVar(airports[i].lat, airports[i].lon);
			char dirm = 'E';
			if (mag > 0)
				dirm = 'W';
			sprintf(line+80, "%c%02d    ", dirm, abs((int) mag));
			if (airports[i].alt != -9999)
				sprintf(line+85, "%06d    "  , (int) (airports[i].alt / .3048));
			else  // -9999 is apparently used to signify an unknown value, don't convert it to feet
				sprintf(line+85, "%06d    "  , (int) (airports[i].alt));
			sprintf(line+93, "%s               "  , d.m_center);
			// INICOM and CTAF information would go here if it was available
			line[123] = '\0';
			fprintf(fpa, "%s\n", line);
		}
		maxIls = ils.GetSize();
		if (maxIls > 0) { // some ils were not accounted for, write to error file
			char line[120];
			FILE *fpe = fopen("nav.e", "w");
			fprintf(fpe, "%d of %d ILSs were not accounted for:\n", maxIls, totalILS);
			for (int j=0; j<maxIls; j++) {
				CString details;
				sprintf(line, " Id: %s, ", ils[j].id);
				details += line;
				sprintf(line, " Full name: %s, ", ils[j].name);
				details += line;
				sprintf(line, " Freq: %6.2f, ", ils[j].freq);
				details += line;
				sprintf(line, " Lat: % 8.4f, ", ils[j].lat);
				details += line;		 
				sprintf(line, " Lon: % 8.4f, ", ils[j].lon);;
				details += line;
				sprintf(line, " Approach: %d\n", (int) ils[j].approach);
				details += line;
				fprintf(fpe, "%s", details);
			}
			fclose(fpe);
		} else 
			remove("nav.e"); // delete old file if present
cancelled:
		delete prog;
		fclose(fpn);
		fclose(fpa);
		fclose(fpr);
		runways.RemoveAll();
		runways.FreeExtra();
		ils.RemoveAll();
		ils.FreeExtra();
		airports.RemoveAll();
		airports.FreeExtra();
		AID.RemoveAll();
		AID.FreeExtra();
	}
	((CMainFrame *) AfxGetMainWnd())->SetStatusBar("Ready");
}

CString CNavDoc::GenUniqueId() { // generate a unique Airport ID counting backward from ZZZ
	int i3 = idcounter % 26;
	int i2 = idcounter/26 % 26;
	int i1 = idcounter/(26*26) % 26;
	idcounter++;
	return(CString('Z' - i1)+CString('Z' - i2)+CString('Z' - i3));
}

CString CNavDoc::GenUniqueNumber() {
	numbercounter++;
	char buf[10];
	sprintf(buf, "%d", numbercounter);
	CString t(buf); 
	return(t);
}
 
//
// JYL return gps fix index
//
int CNavDoc::OnMapInsertgpsfix(double lat, double lon) {
	POSITION p = GetFirstViewPosition();
	CNavView *v = (CNavView *) GetNextView(p);
	CString id = GenUniqueNumber();
	char buf[26];

	// generate DMS name
	char dirx, diry;
	if (lon < 0.) 
		dirx = 'W'; 
	else 
		dirx = 'E';
	if (lat < 0.) 
		diry = 'S'; 
	else 
		diry = 'N';
	int xdeg, xmin, ydeg, ymin;
	double xsec, ysec, gpslat, gpslon;
	gpslat = fabs(lat);
	gpslon = fabs(lon);
	xdeg = (int) gpslon;
	xmin = (int) ((gpslon - xdeg) * 60);
	xsec = (gpslon - xdeg - xmin / 60.0) * 3600;
	ydeg = (int) gpslat;
	ymin = (int) ((gpslat - ydeg) * 60);
	ysec = (gpslat - ydeg - ymin / 60.0) * 3600;
	sprintf(buf, "%d %d'%2.0f\"%c, %d %d'%2.0f\"%c", ydeg, ymin, ysec, diry, xdeg, xmin, xsec, dirx);
	CString name(buf);
	double magdev = GetMagVar(lat, lon);
	addDB(GPS_FIX, lat, lon, 0, 0, id.GetBuffer(1), name.GetBuffer(1), magdev, 0, 0);
	id.ReleaseBuffer();
	name.ReleaseBuffer();
	gpsIndex.Add(dbCount - 1);
	return dbCount -1;
}

void CNavDoc::OnMapInsertgpsfix() {
	POSITION p = GetFirstViewPosition();
	CNavView *v = (CNavView *) GetNextView(p);
	double lat, lon;
	if (v->flatWorld) {
		lat = (v->top + v->bottom)/2.0;	
		lon = (v->left + v->right)/2.0;
	} else {
		lat = v->centerLat;	
		lon = v->centerLon;
	}
	OnMapInsertgpsfix(lat, lon);
	v->abort = TRUE; // abort old redraw if necessary
	v->Invalidate(TRUE);
}

double CNavDoc::GetMagVar(double lat, double lon) { 
	// use data from MagVar table if it is available
	if (magVarTableExists == TRUE) {
		lat = floor(lat+.5); // round to nearest integer
		lon = floor(lon+.5);
		int d = magVarTable[(int) lat+90 + 181*(((int) lon+360) % 360)];
		double magV = (360.0 * d) / 0x10000;
		if (magV > 180)
			magV -= 360;
		return(-magV);
	}
	// use very slow searching-for-nearest-VOR algorithm if MagVar table doesn't exist
	int found = FALSE;
	double mdev = 0.0;					// will return 0 if not found
	double startDist = 10001750.0 * 2;	// halfway around Earth	(maximum distance)
	double newdist;
	MSG msg; 
	POSITION p = GetFirstViewPosition();
	CNavView *v = (CNavView *) GetNextView(p);
	for (int k=0; k<dbCount; k++) {
		if (db[k].type != 3) goto ndbnext;
		// take over message processing to simulate threading 
		while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) { 
			if (msg.message == WM_QUIT) { 
				PostQuitMessage(1);
				return(0.0);      
			}	 
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		// continue
		newdist = v->gcDist(lat, lon, db[k].lat, db[k].lon);
		if (newdist < startDist) {
			startDist = newdist;
			found = TRUE;
			mdev = db[k].magdev;
		}
		ndbnext:;
	} 
	return(mdev); // whether found = TRUE or not
}

void CNavDoc::ReadMagVarTable()	{
	if (magVarTableExists == TRUE)
		return;
	char currentDir[260];
	GetCurrentDirectory(260, currentDir);
	CString navDir = ((CNavApp *) AfxGetApp())->GetNavDirectory();
	SetCurrentDirectory(navDir);
	FILE *mv;
	if ((mv = fopen("magvar.nav", "rb")) == NULL) {
		magVarTableExists = FALSE;
		return;
	}
	unsigned short buf[181*360];
	fread(&buf[0],2,181*360,mv); 
	fclose(mv);
	SetCurrentDirectory(currentDir);
	magVarTable.SetSize(181*360,1);
	for (int lat = -90; lat <=90; lat++) 
		for (int lon = 0; lon <360; lon++) 
			magVarTable[lat+90 + 181*lon] = buf[lat+90 + 181*lon]; 
	magVarTableExists = TRUE;
}

void CNavDoc::ReadMaps()	{
	if (mapsExist == TRUE)
		return;
	mapTable.SetSize(0,20000);
	FILE *mv;
	int handle, lat, lon;
	_finddata_t fd;
	char currentDir[260];
	GetCurrentDirectory(260, currentDir);
	CString navDir = ((CNavApp *) AfxGetApp())->GetNavDirectory();
	SetCurrentDirectory(navDir);
	handle = _findfirst("*.map", &fd);
	int keepgoing = handle;
	while (keepgoing != -1L) { 
		if ((mv = fopen(fd.name, "rb")) == NULL) {
			mapsExist = FALSE;
			CString err = "Error opening map file ";
			err += fd.name;
			AfxMessageBox(err);
			return;
		}
		mapTable.SetSize(fd.size/4,10000);
		int i = 0;
		while(!feof(mv)) {
			i++;
			//if (i > 60000)
			//	break;
 			fread(&lat, 1, 4, mv);
			if(feof(mv))
				break;
 			fread(&lon, 1, 4, mv);
			mapTable.Add(lat);
			mapTable.Add(lon);
		}
		fclose(mv);
		keepgoing = _findnext(handle, &fd);
	}
	_findclose(handle);
	SetCurrentDirectory(currentDir);
	mapsExist = TRUE;
}

BOOL CNavDoc::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) {
	// If pHandlerInfo is NULL, then handle the message
	if (pHandlerInfo == NULL) {
		// Filter the commands sent to a list file menu option
		for (UINT i = WM_NAV; i < WM_NAV+3*(UINT)listFiles; i++) {
			if (nID == i) {
				if (nCode == CN_COMMAND) {
					// Handle WM_COMMAND message
					// build list of files	
					CArray<CString, CString> filelist;	 
					filelist.SetSize(0,5);
					int result, handle;
					_finddata_t fd;
					CString fileSpec;
					fileSpec = "*.lst";
					char currentDir[260];
					GetCurrentDirectory(260, currentDir);
					CString navDir = ((CNavApp *) AfxGetApp())->GetNavDirectory();
					SetCurrentDirectory(navDir);
					result = handle = _findfirst((LPCTSTR) fileSpec, &fd);
					while (result != -1L) { 
						if (_stricmp(fd.name, "Nav.lst") != 0)
							filelist.Add(fd.name);
						result = _findnext(handle, &fd);
					}
					_findclose(handle);
  					SetCurrentDirectory(currentDir);
					if (i-WM_NAV < (UINT)listFiles) {
						listFileName = filelist[i-WM_NAV];
						OnFileAdd();
					} else if (i-WM_NAV < 2*(UINT)listFiles) {
						listFileName = filelist[(i-WM_NAV)%listFiles];
						OnFileSearch();
					} else {
						listFileName = filelist[(i-WM_NAV)%listFiles];
						OnFileLoad();
					}
					filelist.RemoveAll();
					filelist.FreeExtra();
				} else if (nCode == CN_UPDATE_COMMAND_UI) {
					// Update UI element state
					if (i-WM_NAV < (UINT)listFiles)
						((CCmdUI*)pExtra)->Enable(fName != "");			
					else
						((CCmdUI*)pExtra)->Enable(TRUE);
				}
				return TRUE;
			}
		}
	}

	// If we didn't process the command, call the base class
	// version of OnCmdMsg so the message-map can handle the message
	return CDocument::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void CNavDoc::OnUpdateFileAddtoscenerylistNew(CCmdUI* pCmdUI) {
	// build list of files	
	CArray<CString, CString> filelist;	 
	filelist.SetSize(0,5);
	int result, handle;
	_finddata_t fd;
	CString fileSpec;
	fileSpec = "*.lst";
	char currentDir[260];
	GetCurrentDirectory(260, currentDir);
	CString navDir = ((CNavApp *) AfxGetApp())->GetNavDirectory();
	SetCurrentDirectory(navDir);
	result = handle = _findfirst((LPCTSTR) fileSpec, &fd);
	while (result != -1L) { 
		if (_stricmp(fd.name, "Nav.lst") != 0)
			filelist.Add(fd.name);
		result = _findnext(handle, &fd);
	}
	_findclose(handle);
  	SetCurrentDirectory(currentDir);

	// Locate the File submenu
	CMenu* pFileMenu = NULL;
	CMenu* pTopMenu = AfxGetMainWnd()->GetMenu();
	int iPos;
	for (iPos = pTopMenu->GetMenuItemCount()-1; iPos >= 0; iPos--) {
		CMenu* pMenu = pTopMenu->GetSubMenu(iPos);
		if (pMenu && pMenu->GetMenuItemID(0) == ID_FILE_OPEN) {
			pFileMenu = pMenu;
			break;
		}
	}
	ASSERT(pFileMenu != NULL);
	pTopMenu = pFileMenu;
	pFileMenu = NULL;
	for (iPos = pTopMenu->GetMenuItemCount()-1; iPos >= 0; iPos--) {
		CMenu* pMenu = pTopMenu->GetSubMenu(iPos);
		if (pMenu && pMenu->GetMenuItemID(0) == ID_FILE_ADDTOSCENERYLIST_NEW) {
			pFileMenu = pMenu;
			break;
		}
	}
	ASSERT(pFileMenu != NULL);

	// Update the Add submenu 
	// First, delete all items but ID_FILE_LIST_OPEN_NAVLST at position 0
	for (iPos = pFileMenu->GetMenuItemCount()-1; iPos > 0; iPos--)
		pFileMenu->DeleteMenu(iPos, MF_BYPOSITION);
	// Then, add an item for each available file
	pFileMenu->AppendMenu(MF_SEPARATOR);
	pFileMenu->AppendMenu(MF_STRING, ID_FILE_ADDTOSCENERYLIST_NAVLST, "N&av.lst");
	for (int i = 0; i < filelist.GetSize(); i++)	{
		pFileMenu->AppendMenu(MF_STRING, WM_NAV+i, (LPCTSTR) filelist[i]);
	}
	listFiles = i;

	pFileMenu = NULL;
	for (iPos = pTopMenu->GetMenuItemCount()-1; iPos >= 0; iPos--) {
		CMenu* pMenu = pTopMenu->GetSubMenu(iPos);
		if (pMenu && pMenu->GetMenuItemID(0) == ID_FILE_SEARCHFORBGLFILES_NEW) {
			pFileMenu = pMenu;
			break;
		}
	}
	ASSERT(pFileMenu != NULL);
	// Update the Search submenu 
	// First, delete all items but ID_FILE_LIST_SAVE_NEW at position 0
	for (iPos = pFileMenu->GetMenuItemCount()-1; iPos > 0; iPos--)
		pFileMenu->DeleteMenu(iPos, MF_BYPOSITION);
	// Then, add a separator and an item for each available file
	pFileMenu->AppendMenu(MF_SEPARATOR);
	pFileMenu->AppendMenu(MF_STRING, ID_FILE_SEARCHFORBGLFILES_NAVLST, "N&av.lst");
	for (i = 0; i < filelist.GetSize(); i++)	{
		pFileMenu->AppendMenu(MF_STRING, WM_NAV+i+listFiles, (LPCTSTR) filelist[i]);
	}

	pFileMenu = NULL;
	for (iPos = pTopMenu->GetMenuItemCount()-1; iPos >= 0; iPos--) {
		CMenu* pMenu = pTopMenu->GetSubMenu(iPos);
		if (pMenu && pMenu->GetMenuItemID(0) == ID_FILE_LOADSCENERYLIST_NAVLST) {
			pFileMenu = pMenu;
			break;
		}
	}
	ASSERT(pFileMenu != NULL);
	// Update the Load submenu 
	// First, delete all items but Nav.lst at position 0
	for (iPos = pFileMenu->GetMenuItemCount()-1; iPos > 0; iPos--)
		pFileMenu->DeleteMenu(iPos, MF_BYPOSITION);
	// Then, add an item for each available file
	for (i = 0; i < filelist.GetSize(); i++)	{
		pFileMenu->AppendMenu(MF_STRING, WM_NAV+i+2*listFiles, (LPCTSTR) filelist[i]);
	}

	filelist.RemoveAll();
	filelist.FreeExtra();

	pCmdUI->Enable(fName != "");			
}

void CNavDoc::OnFileAddtoscenerylistNavlst() {
	listFileName = "Nav.lst";
	OnFileAdd();
}

void CNavDoc::OnUpdateFileAddtoscenerylistNavlst(CCmdUI* pCmdUI) {
	pCmdUI->Enable(fName != "");			
}

void CNavDoc::OnFileLoadscenerylistNavlst() {
	OnFileLoad("Nav.lst");
}

void CNavDoc::OnFileSearchforbglfilesNavlst() {
	listFileName = "Nav.lst";
	OnFileSearch();
}

void CNavDoc::OnFileAddtoscenerylistNew() {
	if (newList() == TRUE)	
		OnFileAdd();
}

void CNavDoc::OnFileSearchforbglfilesNew() {
	if (newList() == TRUE)
		OnFileSearch();		
}

int CNavDoc::newList() {
	NewList d;
	d.m_name = "Untitled.lst";
	if (d.DoModal() == IDOK) {
		CString fileName = d.m_name;
		// make sure the name ends with .lst
		CString ext = d.m_name.Right(4);
		if (ext.CompareNoCase(".lst") != 0)
			fileName = fileName + ".lst";
		// make sure the file doesn't already exist
		// build list of files	
		CArray<CString, CString> filelist;	 
		filelist.SetSize(0,5);
		int result, handle;
		_finddata_t fd;
		CString fileSpec;
		fileSpec = "*.lst";
		char currentDir[260];
		GetCurrentDirectory(260, currentDir);
		CString navDir = ((CNavApp *) AfxGetApp())->GetNavDirectory();
		SetCurrentDirectory(navDir);
		result = handle = _findfirst((LPCTSTR) fileSpec, &fd);
		while (result != -1L) { 
			filelist.Add(fd.name);
			result = _findnext(handle, &fd);
		}
		_findclose(handle);
  		SetCurrentDirectory(currentDir);
		for (int i = 0; i < filelist.GetSize(); i++) {
			if (fileName == filelist[i]) {
				AfxMessageBox(fileName + " already exists!");
				filelist.RemoveAll();
				filelist.FreeExtra();
				return(FALSE);
			}
		}
		filelist.RemoveAll();
		filelist.FreeExtra();
		// set the filename
		listFileName = fileName;
		return(TRUE);
	} else 
		return(FALSE);
}

void CNavDoc::Index() {
	runwayIndex.RemoveAll();
	runwayIndex.FreeExtra();
	ilsIndex.RemoveAll();
	ilsIndex.FreeExtra();
	vorIndex.RemoveAll();
	vorIndex.FreeExtra();
	ndbIndex.RemoveAll();
	ndbIndex.FreeExtra();
	markerIndex.RemoveAll();
	markerIndex.FreeExtra();
	gpsIndex.RemoveAll();
	gpsIndex.FreeExtra();
	airportIndex.RemoveAll();
	airportIndex.FreeExtra();
	atisIndex.RemoveAll();
	atisIndex.FreeExtra();
	//++ JYL
	wptIndex.RemoveAll();
	wptIndex.FreeExtra();
	//-- JYL
	for (int i=0; i<dbCount; i++) 
		if (1 == db[i].type)
			ndbIndex.Add(i);
		else if (2 == db[i].type)
			ilsIndex.Add(i);
		else if (3 == db[i].type)
			vorIndex.Add(i);
		else if (4 == db[i].type)
			airportIndex.Add(i);
		else if (5 == db[i].type)
			markerIndex.Add(i);
		else if (6 == db[i].type)
			markerIndex.Add(i);
		else if (7 == db[i].type)
			markerIndex.Add(i);
		else if (8 == db[i].type)
			runwayIndex.Add(i);
		else if (9 == db[i].type)
			gpsIndex.Add(i);   
		else if (10 == db[i].type)
			atisIndex.Add(i);  
		else if (11 == db[i].type)
			wptIndex.Add(i);
}

void CNavDoc::OnFileLoad(CString fileListName) {
	listFileName = fileListName;
	OnFileLoad();
}

void CNavDoc::OnFileLoadWpt() 
{
	CString wptFileName;
	CFileDialog searchWayPtFile(TRUE,"txt","*.txt",
									(OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST),
									" txt (*.txt) | *.txt ||");
	if (searchWayPtFile.DoModal() == IDOK)
	{
		wptFileName = searchWayPtFile.GetPathName( );
	}
	else
		return;

	LoadWayPoint(wptFileName);
}

BOOL CNavDoc::ConvertCoordinate(CString& coordToConvert,double* converted)
{
	char NSEW;

	NSEW = coordToConvert.GetAt(0);
	coordToConvert = coordToConvert.Mid(1);
	BOOL positive = TRUE;

	switch(NSEW)
	{
	case 'N':
		positive = TRUE;
		break;
	case 'S':
		positive = FALSE;
		break;
	case 'E':
		positive = TRUE;
		break;
	case 'W':
		positive = FALSE;
		break;
	default:
		return FALSE;
	}

	int degrees;
	int minutes;
	int seconds;

	int nextPoint = coordToConvert.Find('.');
	if (nextPoint < 0)
		return FALSE;
	degrees = atoi(coordToConvert.Mid(0,nextPoint));
	coordToConvert = coordToConvert.Mid(nextPoint+1);

	nextPoint = coordToConvert.Find('.');
	if (nextPoint < 0)
		return FALSE;
	minutes = atoi(coordToConvert.Mid(0,nextPoint));
	coordToConvert = coordToConvert.Mid(nextPoint+1);

	seconds = atoi(coordToConvert);

	*converted = (double) degrees + (double) minutes/60 + (double) seconds/3600;

	if (!positive)
		*converted = (double) -1 * *converted;
	
	return TRUE;
}

void CNavDoc::LoadWayPoint(CString wptFileName)
{

	CString errorMessage;
	FormatedFile f1(',','#');
	if( !f1.Open( wptFileName,
		CFile::modeRead | CFile::typeText ) ) 
	{
		if (! errorMessage.LoadString( IDS_ERR_OPEN_WPT_FILE ))
		{
			AfxMessageBox("Unable to open way point file\n Error loading IDS_ERR_OPEN_WPT_FILE");
		}
		AfxMessageBox(errorMessage);
		#ifdef _DEBUG
		afxDump << "Unable to open way point file : " << wptFileName << "\n";
		#endif
		return ;
	}

	CString type;
	CString id;
	CString lat;
	CString longit;
	double latitude;
	double longitude;

	BOOL allFieldsHere = TRUE;

	dbFiles[dbFileIndex] = wptFileName;

	while (f1.NextRecord())
	{
		allFieldsHere = f1.ReadNextField(type);
		allFieldsHere = f1.ReadNextField(id);
		id = id + "00000";
		id = id.Mid(0,5);
		allFieldsHere = f1.ReadNextField(lat);
		allFieldsHere = f1.ReadNextField(longit);
		if (allFieldsHere)
		{
			if (ConvertCoordinate(lat,&latitude))
			{
				if (ConvertCoordinate(longit,&longitude))
				{
					addDB(WAY_POINT_TYPE,latitude,longitude,0, 0, id.GetBuffer(10), 
						"                                ",
						0,0,0);
					id.ReleaseBuffer();
					wptIndex.Add(dbCount - 1);
				}
			}

		}
	}

	f1.Close();
	dbFileIndex++;
	POSITION p = GetFirstViewPosition();
	CNavView *v = (CNavView *) GetNextView(p);
	v->ZoomToFit();
}