// Navdoc.h : interface of the CNavDoc class
//
/////////////////////////////////////////////////////////////////////////////
//++ added JYL
enum NavaidType_t {
	NDB_TYPE			= 1,
	ILS_TYPE			= 2,
	VOR_TYPE			= 3,
	AIRPORT_TYPE		= 4,
	INNER_MARKER_TYPE	= 5,
	MIDDLE_MARKER_TYPE	= 6,
	OUTER_MARKER_TYPE	= 7,
	RUNWAY_TYPE			= 8,
	GPS_FIX				= 9,
	ATIS_TYPE			= 10,
	WAY_POINT_TYPE		= 11

};
//--
struct dbRec {
	NavaidType_t type;
	double lat;
	double lon;
	int alt;
	double freq;
	char id[6];
	char name[31];
	double magdev;
	double approach;
	double range;
	short file;	
};

#define MaxDBfiles 2000

class CNavDoc : public CDocument {
protected: // create from serialization only
	CNavDoc();
	DECLARE_DYNCREATE(CNavDoc)

// Attributes
public:
	CArray<int, int> runwayIndex;
	CArray<int, int> ilsIndex;
	CArray<int, int> vorIndex;
	CArray<int, int> ndbIndex;
	CArray<int, int> markerIndex;
	CArray<int, int> gpsIndex;
	CArray<int, int> airportIndex;
	CArray<int, int> atisIndex;
	CArray<int, int> wptIndex;
	CString listFileName;
	int listFiles;
	int numbercounter;
	int mapsExist;
	CArray<int, int> mapTable;
	void ReadMagVarTable();
	int magVarTableExists;
	int idcounter;
	int dbCount;
	CArray<dbRec, dbRec> db;
	CArray<unsigned short, unsigned short> magVarTable;
	CString fName;
	int dbFileIndex;
	CString dbFiles[MaxDBfiles];
	BOOL ConvertCoordinate(CString& coordToConvert,double* converted);

// Operations
public:
	int OnMapInsertgpsfix(double lat, double lon);
	void ReadMaps();
	CString GenUniqueId();
	CString GenUniqueNumber();
	void Index();
	int newList();
	BOOL OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo);
	double GetMagVar(double lat, double lon);
	int read3bytes(char *b);
	double readDM(char *b);
	double readMicro4(char *b);
	double readMicro6Lat(char *b);
	double readMicro6Lon(char *b);
	int readFile(CString filename);
	void addDB(	NavaidType_t type, double lat, double lon, int alt, double freq, char *id, char *name, double magdev, double approach, double range);
	void formatData(CString& out, int index);
	void findFile(CString path, CArray<CString, CString> &exist, int &count, int recurse, int skip);
	void OnFileLoad(CString fileListName);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CNavDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CNavDoc();
	virtual void Serialize(CArchive& ar);   // overridden for document i/o
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CNavDoc)
	afx_msg void OnFileAdd();
	afx_msg void OnUpdateFileAdd(CCmdUI* pCmdUI);
	afx_msg void OnFileLoad();
	afx_msg void OnFileSearch();
	afx_msg void OnFileExport();
	afx_msg void OnUpdateFileExport(CCmdUI* pCmdUI);
	afx_msg void OnMapInsertgpsfix();
	afx_msg void OnUpdateFileAddtoscenerylistNew(CCmdUI* pCmdUI);
	afx_msg void OnFileAddtoscenerylistNavlst();
	afx_msg void OnUpdateFileAddtoscenerylistNavlst(CCmdUI* pCmdUI);
	afx_msg void OnFileLoadscenerylistNavlst();
	afx_msg void OnFileSearchforbglfilesNavlst();
	afx_msg void OnFileAddtoscenerylistNew();
	afx_msg void OnFileSearchforbglfilesNew();
	afx_msg void OnFileLoadWpt();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
	void LoadWayPoint(CString filename);
};
