// Navview.cpp : implementation of the CNavView class
//
/*
    Copyright (C) 1997 Ted Wright 
    NASA Business: wright@lerc.nasa.gov, Remainder: wright@en.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "Nav.h"

#include "Navdoc.h"
#include "flightpl.h" 
#include "Navview.h"

#include "settitle.h"
#include "datadial.h"
#include "search.h"
#include "mainfrm.h"
#include "details.h"
#include "autoopt.h"
#include "direct.h"
#include "FTDensity.h"
#include "Transition.h"
#include <afxdlgs.h>
#include "CenterAt.h"
#include "LinkOptions.h"
#include "Fs6State.h"
#include "Fs98State.h"
#include "FS6IPC_Client.h"
#include "InsertGpsFixDial.h"
/*
#define root2		 1.414213562
#define pi			 3.1415946536
#define degToRad     (pi / 180.0)
#define degToMeters  (10001750.0 / 90.0)
#define earthRadius	 (180 * degToMeters / pi)
*/

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CNavView

IMPLEMENT_DYNCREATE(CNavView, CView)

#define new DEBUG_NEW

BEGIN_MESSAGE_MAP(CNavView, CView)
	//{{AFX_MSG_MAP(CNavView)
	ON_COMMAND(ID_VIEW_ZOOMIN, OnViewZoomin)
	ON_COMMAND(ID_VIEW_ZOOMOUT, OnViewZoomout)
	ON_WM_LBUTTONDOWN()
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOMOUT, OnUpdateViewZoomout)
	ON_WM_LBUTTONDBLCLK()
	ON_WM_SIZE()
	ON_COMMAND(ID_VIEW_GRIDLINES, OnViewGridlines)
	ON_UPDATE_COMMAND_UI(ID_VIEW_GRIDLINES, OnUpdateViewGridlines)
	ON_COMMAND(ID_VIEW_NDB, OnViewNdb)
	ON_COMMAND(ID_VIEW_ILS, OnViewIls)
	ON_COMMAND(ID_VIEW_VOR, OnViewVor)
	ON_COMMAND(ID_VIEW_AIRPORTS, OnViewAirports)
	ON_UPDATE_COMMAND_UI(ID_VIEW_AIRPORTS, OnUpdateViewAirports)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ILS, OnUpdateViewIls)
	ON_UPDATE_COMMAND_UI(ID_VIEW_NDB, OnUpdateViewNdb)
	ON_UPDATE_COMMAND_UI(ID_VIEW_VOR, OnUpdateViewVor)
	ON_COMMAND(ID_VIEW_ID, OnViewId)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ID, OnUpdateViewId)
	ON_UPDATE_COMMAND_UI(ID_VIEW_FREQUENCY, OnUpdateViewFrequency)
	ON_COMMAND(ID_VIEW_FREQUENCY, OnViewFrequency)
	ON_COMMAND(ID_EDIT_CHANGEFONT, OnEditChangefont)
	ON_COMMAND(ID_EDIT_SETTITLE, OnEditSettitle)
	ON_COMMAND(ID_VIEW_SEARCH, OnViewSearch)
	ON_COMMAND(ID_VIEW_DATABASE, OnViewDatabase)
	ON_COMMAND(ID_EDIT_SAVEOPTIONS, OnEditSaveoptions)
	ON_WM_RBUTTONDBLCLK()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_VIEW_FLATWORLD, OnViewFlatworld)
	ON_UPDATE_COMMAND_UI(ID_VIEW_FLATWORLD, OnUpdateViewFlatworld)
	ON_COMMAND(ID_EDIT_FLIGHTPLAN, OnEditFlightplan)
	ON_UPDATE_COMMAND_UI(ID_EDIT_FLIGHTPLAN, OnUpdateEditFlightplan)
	ON_COMMAND(ID_PLAN_PLANOPTIONS, OnPlanPlanoptions)
	ON_COMMAND(ID_MAP_SHOW_PLANDATAONLY, OnMapShowPlandataonly)
	ON_UPDATE_COMMAND_UI(ID_MAP_SHOW_PLANDATAONLY, OnUpdateMapShowPlandataonly)
	ON_COMMAND(ID_VIEW_MARKERS, OnViewMarkers)
	ON_UPDATE_COMMAND_UI(ID_VIEW_MARKERS, OnUpdateViewMarkers)
	ON_COMMAND(ID_MAP_SHOW_PLANINFORMATION, OnMapShowPlaninformation)
	ON_UPDATE_COMMAND_UI(ID_MAP_SHOW_PLANINFORMATION, OnUpdateMapShowPlaninformation)
	ON_COMMAND(ID_FILE_IMPORT, OnFileImport)
	ON_UPDATE_COMMAND_UI(ID_MAP_CLEARFLIGHTTRACK, OnUpdateMapClearflighttrack)
	ON_COMMAND(ID_MAP_CLEARFLIGHTTRACK, OnMapClearflighttrack)
	ON_COMMAND(ID_MAP_SETFLIGHTTRACKDENSITY, OnMapSetflighttrackdensity)
	ON_UPDATE_COMMAND_UI(ID_FILE_IMPORT, OnUpdateFileImport)
	ON_COMMAND(ID_MAP_LARGESYMBOLS, OnMapLargesymbols)
	ON_UPDATE_COMMAND_UI(ID_MAP_LARGESYMBOLS, OnUpdateMapLargesymbols)
	ON_COMMAND(ID_MAP_SETTRANSITION, OnMapSettransition)
	ON_COMMAND(ID_MAP_ZOOMTOFIT, OnMapZoomtofit)
	ON_UPDATE_COMMAND_UI(ID_MAP_ZOOMTOFIT, OnUpdateMapZoomtofit)
	ON_COMMAND(ID_MAP_SHOW_MAPSYMBOLS_AIRPORTZONES, OnMapShowMapsymbolsAirportzones)
	ON_UPDATE_COMMAND_UI(ID_MAP_SHOW_MAPSYMBOLS_AIRPORTZONES, OnUpdateMapShowMapsymbolsAirportzones)
	ON_COMMAND(ID_MAP_SHOW_MAPS, OnMapShowMaps)
	ON_UPDATE_COMMAND_UI(ID_MAP_SHOW_MAPS, OnUpdateMapShowMaps)
	ON_COMMAND(ID_MAP_CENTERAT, OnMapCenterat)
	ON_COMMAND(ID_MAP_ATIS, OnMapAtis)
	ON_UPDATE_COMMAND_UI(ID_MAP_ATIS, OnUpdateMapAtis)
	ON_COMMAND(ID_FS6_LINK, OnFs6Link)
	ON_UPDATE_COMMAND_UI(ID_FS6_LINK, OnUpdateFs6Link)
	ON_COMMAND(ID_FS6_SET, OnFs6Set)
	ON_COMMAND(ID_FS6_FS6LINKOPTIONS, OnFs6Fs6linkoptions)
	ON_COMMAND(ID_FS6_FLYTO, OnFs6Flyto)
	ON_UPDATE_COMMAND_UI(ID_FS6_FLYTO, OnUpdateFs6Flyto)
	ON_COMMAND(ID_HELPVIEWDOCS, OnHelpviewdocs)
	ON_WM_KEYDOWN()
	ON_WM_SETFOCUS()
	ON_WM_LBUTTONUP()
	ON_COMMAND(ID_VIEW_WAYPT, OnViewWaypt)
	ON_UPDATE_COMMAND_UI(ID_VIEW_WAYPT, OnUpdateViewWaypt)
	//}}AFX_MSG_MAP
	ON_WM_DROPFILES()
//	ON_COMMAND(WM_DROPFILES, CView::OnDropFiles)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

struct path_t {
	long time;			// timestamp
    double lat;			// latitude
    double lon;        	// longitude
};

CArray<path_t, path_t> fltPath;
CRITICAL_SECTION crit;

// define a structure for storing flight data
struct {
  DWORD dwLatLo;			// latitude  (low dword)
  DWORD dwLatHi;			// latitude  (high dword)
  DWORD dwLonLo;			// longitude (low dword)
  DWORD dwLonHi;			// longitude (high dword)
  DWORD dwAltLo;			// altitude  (low dword)
  DWORD dwAltHi;			// altitude  (high dword)
  DWORD dwHeading;			// heading
  DWORD dwTAS;				// true airspeed (m/s * 256)
  WORD  wMagVar;			// magnetic variation 
  DWORD	dwAPMaster;         // autopilot master switch (0 or 1)
  DWORD dwAPWingLeveler;    // autopilot wing leveler enable (0 or 1)
  DWORD dwAPHeadingHold;    // autopilot heading hold enable (0 or 1)
  DWORD dwAPHeading;        // autopilot heading to hold (actual heading * 0x10000/360.0)
  DWORD dwAPAttitudeHold;   // autopilot attitude (pitch & bank) hold enable (0 or 1)
} flightData;

// Define a structure to hold data specific to each version of FS
// This structure helps ease compatibility with both FS95 and FS98.
struct FSInfo {
  const char* pszWindowClass;	// name of the window class
  // state data offsets
  DWORD offset_dwLatLo;
  DWORD offset_dwLatHi;
  DWORD offset_dwLonLo;
  DWORD offset_dwLonHi;
  DWORD offset_dwAltLo;
  DWORD offset_dwAltHi;
  DWORD offset_dwHeading;
  DWORD offset_dwTAS;
  DWORD offset_wMagVar;
  DWORD offset_dwAPMaster;           
  DWORD offset_dwAPWingLeveler;      
  DWORD offset_dwAPHeadingHold;      
  DWORD offset_dwAPHeading;          
  DWORD offset_dwAPAttitudeHold;     
};

// FS95 information
FSInfo fs95Info = {
  "FS6MAIN",					// name of the window class
  // state data offsets
  FIELD_OFFSET(FS6StateData, dwLatLo),                            
  FIELD_OFFSET(FS6StateData, dwLatHi),                                
  FIELD_OFFSET(FS6StateData, dwLonLo),
  FIELD_OFFSET(FS6StateData, dwLonHi),
  FIELD_OFFSET(FS6StateData, dwAltLo),
  FIELD_OFFSET(FS6StateData, dwAltHi),
  FIELD_OFFSET(FS6StateData, dwHeading),
  FIELD_OFFSET(FS6StateData, dwTAS),
  FIELD_OFFSET(FS6StateData, wMagVar),
  FIELD_OFFSET(FS6StateData, dwAPMaster),                            
  FIELD_OFFSET(FS6StateData, dwAPWingLeveler),                                
  FIELD_OFFSET(FS6StateData, dwAPHeadingHold),
  FIELD_OFFSET(FS6StateData, dwAPHeading),
  FIELD_OFFSET(FS6StateData, dwAPAttitudeHold),
};

// FS98 information
FSInfo fs98Info = {
  "FS98MAIN",					// name of the window class
  // state data offsets
  FIELD_OFFSET(FS98StateData, dwLatLo),                            
  FIELD_OFFSET(FS98StateData, dwLatHi),                                
  FIELD_OFFSET(FS98StateData, dwLonLo),
  FIELD_OFFSET(FS98StateData, dwLonHi),
  FIELD_OFFSET(FS98StateData, dwAltLo),
  FIELD_OFFSET(FS98StateData, dwAltHi),
  FIELD_OFFSET(FS98StateData, dwHeading),
  FIELD_OFFSET(FS98StateData, dwTAS),
  FIELD_OFFSET(FS98StateData, wMagVar),
  FIELD_OFFSET(FS98StateData, dwAPMaster),                            
  FIELD_OFFSET(FS98StateData, dwAPWingLeveler),                                
  FIELD_OFFSET(FS98StateData, dwAPHeadingHold),
  FIELD_OFFSET(FS98StateData, dwAPHeading),
  FIELD_OFFSET(FS98StateData, dwAPAttitudeHold),
};

UINT LinkThread( LPVOID pParam ) {
	CNavView *pView = (CNavView *) pParam;
	WaitForInputIdle(GetCurrentProcess(), 10000);
	FS6IPC_Client *ipcClient;		// create the IPC client
	FSInfo* pFSInfo;				// set this pointer to either FS95 or FS98 state offsets 
	HWND hWndFs6;
	while(pView->link == TRUE) {
		if (pView->linked == FALSE) {	// look for FS95 or FS98 program
			if (hWndFs6 = FindWindowEx(NULL, NULL, fs95Info.pszWindowClass, NULL))
				pFSInfo = &fs95Info;
			else if (hWndFs6 = FindWindowEx(NULL, NULL, fs98Info.pszWindowClass, NULL))
				pFSInfo = &fs98Info;
			if (hWndFs6 != NULL) {		// found it
				pView->linked = TRUE;
				pView->linkFirst = TRUE;
//				pView->Invalidate(TRUE);
				ipcClient = new FS6IPC_Client;
				ipcClient->Start(hWndFs6);
			} else {					// wait a while
				Sleep(5000);
			}
		} else {						// try to get data from FS
			// get data
			WaitForInputIdle(GetCurrentProcess(),60000);
			Sleep(pView->linkUpdateTime * 1000);
			ipcClient->BeginRequest();
			EnterCriticalSection(&crit);
			ipcClient->ReadStateData(pFSInfo->offset_dwLatLo, 6 * sizeof(DWORD), &flightData.dwLatLo);
			ipcClient->ReadStateData(pFSInfo->offset_dwHeading, sizeof(DWORD), &flightData.dwHeading);
			ipcClient->ReadStateData(pFSInfo->offset_dwTAS, sizeof(DWORD), &flightData.dwTAS);
			ipcClient->ReadStateData(pFSInfo->offset_wMagVar, sizeof(WORD), &flightData.wMagVar);
			if (ipcClient->SendRequest()) {
				// process the data we just read
				LeaveCriticalSection(&crit);
				double lat, lon;
				int intPart = flightData.dwLatHi;
				if (intPart >= 0)
					lat = intPart + (double)flightData.dwLatLo / (double)0x100000000;
				else		 
					lat = intPart - (double)flightData.dwLatLo / (double)0x100000000;
				lat *= 45.0 / 5000875.0;

				intPart = flightData.dwLonHi;
				if (intPart >= 0)
					lon = intPart + (double)flightData.dwLonLo / (double)0x100000000;
				else
					lon = intPart - (double)flightData.dwLonLo / (double)0x100000000;
				lon *= 90.0 / (256.0 * 4194304.0);

				double alt = ((double)flightData.dwAltHi + (double)flightData.dwAltLo / (double)0x100000000) * 3.28084;
				double head = 360 * (double)flightData.dwHeading / (double)0x100000000;
				double tas = ((double)flightData.dwTAS / 256.0) * 3600 / (degToMeters/60); // kts
				double mag = ((double)flightData.wMagVar * 360.0) / 0x10000;
				if (mag > 180) mag = mag-360;
				mag *= -1.0;

				pView->linkHeading = head;
				pView->linkAlt = alt;
				pView->linkTAS = tas;
				pView->linkLat = lat;
				pView->linkLon = lon;
				pView->linkMag = mag;
				if (pView->linkFirst) {
					pView->linkFirst = FALSE;
					pView->DrawLink(FALSE);
				} else {
					pView->DrawLink(TRUE);
				}
			} else {
				pView->linked = FALSE; // FS did not respond
				LeaveCriticalSection(&crit);
				ipcClient->Stop();
				delete ipcClient;
			}
		}
	}
	if (pView->linked == TRUE) {
		pView->linked = FALSE;
		ipcClient->Stop();
		delete ipcClient;
	}
	return(0);
}

UINT FlyThread( LPVOID pParam ) {
	CNavView *pView = (CNavView *) pParam;
	WaitForInputIdle(GetCurrentProcess(), 10000);
	FSInfo* pFSInfo;	  // set this pointer to either FS95 or FS98 state offsets 
	HWND hWndFs6;
	double lastDistance = 0;
	while (pView->flying) {
		if (hWndFs6 = FindWindowEx(NULL, NULL, fs95Info.pszWindowClass, NULL))
			pFSInfo = &fs95Info;
		else if (hWndFs6 = FindWindowEx(NULL, NULL, fs98Info.pszWindowClass, NULL))
			pFSInfo = &fs98Info;
		if (hWndFs6 == NULL) {		
			pView->hflythread = 0;
			pView->flying = FALSE;
			AfxMessageBox("Can't find FlightSim. Is it running?");
			return(0);
		}
		FS6IPC_Client ipcClient;	
		EnterCriticalSection(&crit);
		ipcClient.Start(hWndFs6);
		ipcClient.ReadStateData(pFSInfo->offset_dwLatLo, 6 * sizeof(DWORD), &flightData.dwLatLo);
		ipcClient.ReadStateData(pFSInfo->offset_wMagVar, sizeof(WORD), &flightData.wMagVar);
		if (ipcClient.SendRequest()) {
			double lat, lon;
			int intPart = flightData.dwLatHi;
			if (intPart >= 0)
				lat = intPart + (double)flightData.dwLatLo / (double)0x100000000;
			else		 
				lat = intPart - (double)flightData.dwLatLo / (double)0x100000000;
			lat *= 45.0 / 5000875.0;
			intPart = flightData.dwLonHi;
			if (intPart >= 0)
				lon = intPart + (double)flightData.dwLonLo / (double)0x100000000;
			else
				lon = intPart - (double)flightData.dwLonLo / (double)0x100000000;
			lon *= 90.0 / (256.0 * 4194304.0);
			double mag = ((double)flightData.wMagVar * 360.0) / 0x10000;
			if (mag > 180) mag = mag-360;
			mag *= -1.0;
			double dist = pView->gcDist(lat, lon, pView->flyingLat, pView->flyingLon);
			if (lastDistance == 0)
				lastDistance = dist;
			if (pView->flytoPlan && pView->planning)
			{
				//
				// get a new fly route if we flyToPlan
				// we get a new fly point
				// if dist is less than number of miles to turn before point
				// or if plane is getting far from the point
				// (we do not take in account this last when the distance is more
				// than 25 km. i.e. Sharp turn.
				//
				if ((dist < pView->linkFlyNbMilesToTurnBefore * 1852) || 
						((dist > lastDistance) && (dist < 25000)))
					pView->planD->SelectNextToFlyTo(pView);
				// recalculate the distance (to get the course)
				dist = pView->gcDist(lat, lon, pView->flyingLat, pView->flyingLon);

			}
			lastDistance = dist;
			double course = pView->bearing+mag;
			if (course < 0) course += 360;
			if (course > 360) course -= 360;

			flightData.dwAPHeading = (DWORD) (course * 0x10000 / 360.0);
			ipcClient.BeginRequest();
			ipcClient.WriteStateData(pFSInfo->offset_dwAPHeading, sizeof(DWORD), &flightData.dwAPHeading);
			// turn on the nav hold switches or just update the heading?
			if (pView->linkFlySetMode) {
				flightData.dwAPMaster = 1;           
				flightData.dwAPWingLeveler = 0;     
				flightData.dwAPHeadingHold = 1;      
				flightData.dwAPAttitudeHold = 0;;     
				ipcClient.WriteStateData(pFSInfo->offset_dwAPMaster,         sizeof(DWORD), &flightData.dwAPMaster);
				ipcClient.WriteStateData(pFSInfo->offset_dwAPWingLeveler,    sizeof(DWORD), &flightData.dwAPWingLeveler);
				ipcClient.WriteStateData(pFSInfo->offset_dwAPHeadingHold,    sizeof(DWORD), &flightData.dwAPHeadingHold);
				ipcClient.WriteStateData(pFSInfo->offset_dwAPAttitudeHold,   sizeof(DWORD), &flightData.dwAPAttitudeHold);
			}
			ipcClient.SendRequest();
		}
		ipcClient.Stop();
		LeaveCriticalSection(&crit);
		WaitForSingleObject(pView->hflyevent, pView->linkFlyUpdateTime * 1000);
	}
	pView->hflythread = 0;
	return(0);
}

/////////////////////////////////////////////////////////////////////////////
// CNavView construction/destruction
	  
CNavView::CNavView() {
	orgDir = ((CNavApp *) AfxGetApp())->GetNavDirectory();
	init = TRUE;
	initAfterPrint = FALSE;
	fontSet = FALSE;
	title = "";
	search = "";
	measuring = FALSE;
	planning = FALSE;
	showPlanning = TRUE;
	plancount = 0;
	planOnly = FALSE;
	planInfo = TRUE;
	abort = FALSE;
	flightPaths = 0;
	flightTrackDensity = 10; // 10 seconds
	largeSymbols = TRUE;
	symbolTransition = 6;
	left = -180;
	right = 180;
	top = 90;
	bottom = -90;
	mag = 1;
	mag2 = 1;
	centerLat = 0;
	centerLon = 0;
	Mready = FALSE;
	linked = FALSE;
	linkUpdateTime = 5;
	linkFlyUpdateTime = 60;
	linkFlySetMode = FALSE;
	linkFlyNbMilesToTurnBefore = 1;
	xLinkExtent = yLinkExtent = 0;
	flying = FALSE;
	InitializeCriticalSection(&crit);
	hflythread = 0;
	hflyevent = CreateEvent(NULL, FALSE, FALSE, "flyevent");

	CFont font;		// initialize default font
	font.CreateFont(-11,0,0,0,FW_NORMAL,0,0,0,ANSI_CHARSET,3,2,1,34,"Arial");
	font.GetLogFont(&lf);

	// set OS version
    OSVERSIONINFO vi;
	vi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if (GetVersionEx(&vi)) {
		if (vi.dwPlatformId == VER_PLATFORM_WIN32_NT)
			OS = 1; // NT
		else if (vi.dwPlatformId == VER_PLATFORM_WIN32s)
			OS = 3; // Win32s
		else 
			OS = 2; // Win95
	} else
		OS = 3; // call failed, assume Win32s

	// dummy values, will be overwritten
	grid = 0;
	vor = 0;
	ils = 0;
	ndb = 0;
	id = 0;
	freq = 0;
	airports = 0;
	markers = 0;
	atis = 1;
	flatWorld = 1;
	planVOR = 1;
	planNDB = 0;
	planAirport = 0;
	planContinue = 1;
	showAirportZones = 1;
	showMaps = 1;
	idealDist = 139.0; // km, about 75 nm
	link = 1;
	wpt = 1;
	fromNavAid = FALSE;
	navAidIndex = -1;

	flytoPlan = FALSE;
}

CNavView::~CNavView() {
	if (linked) {
		linked = FALSE;
		WaitForSingleObject(hthread, INFINITE); 
		CloseHandle(hthread);
	}
	if (hflythread != 0) {
		flying = FALSE;
		SetEvent(hflyevent);
		WaitForSingleObject(hflythread, INFINITE); 
		CloseHandle(hflythread);
	}
	CloseHandle(hflyevent);	
	DeleteCriticalSection(&crit);
	if (flightPaths > 0) {
		fltPath.RemoveAll();
		fltPath.FreeExtra();
		flightPaths--;
	}
	windtable.RemoveAll();
}

/////////////////////////////////////////////////////////////////////////////

int CNavView::LtoC(double xl, double yl, int &xc, int &yc) {
	if (flatWorld) {
		xc = (int) ((xl - left)/(right-left)*screenWidth);
		yc = (int) ((top - yl)/(top - bottom)*screenHeight);
		if ((xl<left) || (xl>right) || (yl<bottom) || (yl>top))
			return(FALSE);
		return(TRUE);
	} else {
		ASSERT(convX > 0);
		ASSERT(convY > 0);
		if (!Mready)
			Mcalc();
		double xo,yo,zo,xs,ys,zs;
		StoCe(xo,yo,zo,yl,xl);			// convert to normalized cartesian
		zs = xo*M31 + yo*M32 + zo*M33;  // rotate to viewpoint 
		xs = xo*M11 + zo*M13;			// (M12 skipped, should always be 0)
		xs *= convX * screenWidth/2; 
		xc = screenWidth/2  + (int) (xs);
		ys = xo*M21 + yo*M22 + zo*M23;
		ys *= convY * screenHeight/2;
		yc = screenHeight/2 - (int) (ys);
		if (zs < 0)
			return FALSE;				// dark side of the earth
		if (fabs(xs) > screenWidth/2)
			return(FALSE);
		if (fabs(ys) > screenHeight/2)
			return(FALSE);
		return TRUE;
	}
}

int CNavView::CtoL(int xc, int yc, double &xl, double &yl) {
	if (flatWorld) {
		xl = left+((double) xc)/screenWidth*(right-left);
		yl = top-((double) yc)/screenHeight*(top - bottom);
		return(TRUE);
	} else {	
		ASSERT(convX > 0);
		ASSERT(convY > 0);
		if (!Mready)
			Mcalc();
		double xo,yo,zo,xs,ys,zs,t;
		double nx = xc - screenWidth/2;		// center screen is 0,0
		double ny = screenHeight/2 - yc;
		xs = nx / (convX * screenWidth/2);
		ys = ny / (convY * screenHeight/2);
		t = xs*xs+ys*ys;
		if (t > 1)							// invalid point in orbit
			return(FALSE);					
		zs = sqrt(1 - t);					// positive root selects this side of earth
		xo = xs*M11 + ys*M21 + zs*M31;
		yo =          ys*M22 + zs*M32;		// (M12 skipped, should always be 0)
		zo = xs*M13 + ys*M23 + zs*M33;
		CtoSe(xo,yo,zo,yl,xl);				// convert from normalized cartesian
		fixup(xl, yl);						// normalize angles
		return(TRUE); 
	}
}

void CNavView::fixup(double &x, double &y) {  // fixup wrap-around
	if (y > 90) {
		y -= (y - 90) * 2;
		x += 180;
	} else if (y < -90) {
		y += (y + 90) * 2;
		x += 180;
	}
	while (x > 180)
		x -= 360;
	while (x < -180)
		x += 360;
}

void CNavView::OnDraw(CDC* pDC) {
	EnterCriticalSection(&crit);
	int dcNumber = pDC->SaveDC();
	abort = FALSE;
	linkFirst = TRUE;
	CNavDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (planning && !pDC->IsPrinting())
		planD->update();

	CArray<idPlace, idPlace> idlist;	 
	idlist.SetSize(0,5);

	xLinkExtent = yLinkExtent = 0;
	GetClientRect(clientRect);
	pDC->FillSolidRect(clientRect, RGB(255,255,255));

	CFont font, titleFont;
	LOGFONT lfnew = lf;
	if (pDC->IsPrinting()) {	// make font larger on printer
		lfnew.lfHeight *= 3;
	}
	font.CreateFontIndirect(&lfnew);
	lfnew.lfHeight *= 2;		// double size for title
	titleFont.CreateFontIndirect(&lfnew);
		
	if (pDC->IsPrinting()) {
		screenWidth = pDC->GetDeviceCaps(HORZRES);
		screenHeight = pDC->GetDeviceCaps(VERTRES);
		Size(screenWidth, screenHeight);
		initAfterPrint = TRUE;
	} else {
		screenWidth = clientRect.right;
		screenHeight = clientRect.bottom;
		if (initAfterPrint) {
			Size(screenWidth, screenHeight);
			initAfterPrint = FALSE;
		}
	}

	CRgn clipper; // clip drawing to displayable area
	clipper.CreateRectRgn(0, 0, screenWidth, screenHeight);
	pDC->SelectClipRgn(&clipper, RGN_AND);

	if (init) {
		init = FALSE;

		char buffer[100];
		float temp;
		flatWorld = GetPrivateProfileInt("View", "flatWorld", 1, "nav.ini");
 		GetPrivateProfileString("View", "idealDist", "139", buffer, 30, "nav.ini");
		sscanf(buffer, "%f", &temp);
		idealDist = (double) temp;

		grid = GetPrivateProfileInt("View", "grid", 1, "nav.ini");
		vor = GetPrivateProfileInt("View", "vor", 1, "nav.ini");
		ils = GetPrivateProfileInt("View", "ils", 0, "nav.ini");
		ndb = GetPrivateProfileInt("View", "ndb", 1, "nav.ini");
		airports = GetPrivateProfileInt("View", "airports", 1, "nav.ini");
		markers = GetPrivateProfileInt("View", "markers", 0, "nav.ini");
		atis = GetPrivateProfileInt("View", "atis", 1, "nav.ini");
		id = GetPrivateProfileInt("View", "id", 0, "nav.ini");
		freq = GetPrivateProfileInt("View", "freq", 0, "nav.ini");
		planVOR = GetPrivateProfileInt("View", "planVOR", 1, "nav.ini");
		planNDB = GetPrivateProfileInt("View", "planNDB", 0, "nav.ini");
		planAirport = GetPrivateProfileInt("View", "planAirport", 0, "nav.ini");
		planContinue = GetPrivateProfileInt("View", "planContinue", 1, "nav.ini");
		showAirportZones = GetPrivateProfileInt("View", "showAirportZones", 1, "nav.ini");
		showMaps = GetPrivateProfileInt("View", "showMaps", 1, "nav.ini");
		link = GetPrivateProfileInt("View", "Link", 1, "nav.ini");
		linkUpdateTime = GetPrivateProfileInt("View", "linkUpdateTime", 5, "nav.ini");
		linkFlyUpdateTime = GetPrivateProfileInt("View", "linkFlyUpdateTime", 10, "nav.ini");
		linkFlySetMode = GetPrivateProfileInt("View", "linkFlySetMode", 0, "nav.ini");

		GetPrivateProfileString("View", "linkFlyNbMilesToTurnBefore", "1", buffer, 30, "nav.ini");
		sscanf(buffer, "%f", &temp);
		linkFlyNbMilesToTurnBefore = (double) temp;

		flightTrackDensity = GetPrivateProfileInt("View", "flightTrackDensity", 10, "nav.ini");
		largeSymbols = GetPrivateProfileInt("View", "largeSymbols", 1, "nav.ini");
		plancount = 0;
		windtable.RemoveAll();
		if (planning) { // close flight plan window, if open
			planning = FALSE;
			planD->DestroyWindow();	// let it deallocate its own memory
		}
		if (pDoc->dbCount == 0) {
			left = -180;
			right = 180;
			top = 90;
			bottom = -90;
			mag = 1;
			mag2 = 1;
			centerLat = 0;
			centerLon = 0;
			convX = convY = mag2;
			if (screenWidth > screenHeight) 
				convX *= screenHeight / (double) screenWidth;
			else 
				convY *= screenWidth / (double) screenHeight;
		}
		Size(screenWidth, screenHeight);
		if (link && !linked) { // start link thread
			hthread = AfxBeginThread(LinkThread, this);
			SetThreadPriority(hthread, THREAD_PRIORITY_BELOW_NORMAL);
		}
	}

	int vorsize; 
	if (largeSymbols)
		vorsize = screenWidth / 15;
	else
		vorsize = screenWidth / 40;

	CPen *oldPen;
	CPen redPen(PS_SOLID, 0, RGB(255, 0, 0));
	CPen greenPen(PS_SOLID, 0, RGB(0, 192, 0));
	CPen bluePen(PS_SOLID, 0, RGB(0, 0, 255));
	CPen grayPen(PS_SOLID, 0, RGB(192, 192, 192));
	CPen orangePen(PS_SOLID, 0, RGB(255, 192, 0));
	CPen wayPtPen(PS_SOLID,0,RGB(255,0,255));

	int oldMode = pDC->SetBkMode(TRANSPARENT);

	CFont* oldFont;
	oldFont = (CFont *) pDC->SelectObject(&font);

	int rx1, rx2, ry1, ry2; 
	if (flatWorld) {
		LtoC(-180, 90, rx1, ry1);
		LtoC(180, -90, rx2, ry2);
		pDC->Rectangle(rx1, ry1, rx2, ry2);
	} else {
		// draw earth outline
		double cx = screenWidth  / 2;
		double cy = screenHeight / 2;
		double hx = convX * screenWidth / 2;
		double hy = convY * screenHeight / 2;
		pDC->Ellipse((int)(cx-hx), (int)(cy-hy), (int)(cx+hx), (int)(cy+hy));
	}
	
	char buf[30];

	if (pDoc->mapsExist && showMaps) 
		DrawMaps(pDC);

	if (grid)
		DrawGrid(pDC);

	CBrush *oldBrush;
	int i;

	// airport zones
	if (airports && showAirportZones) {
		oldBrush = (CBrush *) pDC->SelectStockObject(LTGRAY_BRUSH);
		if (mag > symbolTransition)
			oldPen = (CPen *) pDC->SelectStockObject(NULL_PEN);
		else
			oldPen = (CPen *) pDC->SelectObject(&grayPen);


		for (i=0; i<pDoc->dbCount; i++) {
			if (planOnly) {
				int proceed = FALSE;
				for (int q=0; q<plancount; q++)
					if (i == plan[q]) {
						proceed = TRUE;
						break;
					}
				if (!proceed)
					goto nextDB1;
			}

			if (!pDC->IsPrinting()) {
				if (drawMessageLoop() == TRUE) { // clean up and exit
					pDC->SelectObject(oldPen);
					pDC->SelectObject(oldBrush);
					pDC->SelectObject(oldFont);
					pDC->SetBkMode(oldMode);
					pDC->RestoreDC(dcNumber);
					if (id) {
						idlist.RemoveAll();
						idlist.FreeExtra();
					}
					LeaveCriticalSection(&crit);
					return;
				}
			}
	
			int x,y;
 			if ((pDoc->db[i].lon) < left) goto nextDB1;
			if ((pDoc->db[i].lon) > right) goto nextDB1;
			if ((pDoc->db[i].lat) < bottom) goto nextDB1;
			if ((pDoc->db[i].lat) > top) goto nextDB1;
			if (!LtoC(pDoc->db[i].lon, pDoc->db[i].lat, x, y)) goto nextDB1;

			// ILS, Runway and airport have zones
			if ((2 == pDoc->db[i].type) || (8 == pDoc->db[i].type) || (4 == pDoc->db[i].type)) {	
				if (mag > symbolTransition) {
					double ldist;
					int l = distToPixels(2000); // l = 2 km
					int lx = l;
 					if(flatWorld) { // calculate latitude distortion due to flat map
						ldist = 1.0/cos(pDoc->db[i].lat * degToRad);
						lx = (int) (lx * ldist);
					}
					pDC->Ellipse(x-lx, y-l, x+lx, y+l);
				} else {
					pDC->MoveTo(x-1, y-1);
					pDC->LineTo(x+3, y+3);
					pDC->MoveTo(x+2, y-1);
					pDC->LineTo(x-2, y+3);
				}
			}
			nextDB1:;
		}
		pDC->SelectObject(oldBrush);
		pDC->SelectObject(oldPen);
	}

	if (planning) {
		oldPen = (CPen *) pDC->SelectStockObject(BLACK_PEN);
		int oldx, oldy, x, y;
		for (int j=0; j<planD->plantable.GetSize(); j++) {
			if (!LtoC(planD->plantable[j].lon, planD->plantable[j].lat, x, y)) {
				if (j > 0) { // make sure last point was not shown
					int jx, jy;
					if (!LtoC(planD->plantable[j-1].lon, planD->plantable[j-1].lat, jx, jy))
						goto nextLine;
				} else goto nextLine;
			}
			if (j > 0) {
				if (planD->plantable[j].index < 0) {
					pDC->MoveTo(x, y+3);
					pDC->LineTo(x+3, y);
					pDC->LineTo(x, y-3);
					pDC->LineTo(x-3, y);
					pDC->LineTo(x, y+3);
					CString label;
					if (planD->plantable[j].index == -1) 
						label = "TOC";
					else if (planD->plantable[j].index == -2) 
						label = "BOD";
					else if (planD->plantable[j].index == -3) 
						label = "TOC/BOD";
					double head = planD->plantable[j].correctedheading;
					if (((int) head)%180 > 90) {
						pDC->TextOut(x+10, y-10, (LPCTSTR) label);
					} else {
						pDC->TextOut(x+5, y+5, (LPCTSTR) label);
					}
				}
				gcLine(oldx, oldy, x, y, pDC);
				if (planInfo) {
					int mx = (x+oldx)/2;
					int my = (y+oldy)/2;
					char buff[30]; 
					double dis = planD->plantable[j].dist;
					int offset = min((int)sqrt((double)distToPixels(dis*1852.0)), 15);
					double head = planD->plantable[j].correctedheading;
					CSize tSize; 
					int chars = sprintf(buff, "%1.1f nm @ %1.0f", dis, head);
					tSize = pDC->GetOutputTextExtent(buff, chars);
					if (((int) head)%180 > 90) {
						idPlace idP = {mx, my-offset, mx+tSize.cx, my-offset+tSize.cy};
						idPlacement(mx, my, idP, &idlist);
						pDC->TextOut(idP.x+2, idP.y, buff);
					} else {
						idPlace idP = {mx, my+offset, mx+tSize.cx, my+offset+tSize.cy};
						idPlacement(mx, my, idP, &idlist);
						pDC->TextOut(idP.x+2, idP.y, buff);
					}

				}
			}
			nextLine:;
			oldx = x;
			oldy = y;
		}
		pDC->SelectObject(oldPen);
	}
	
	// ATIS
	if (atis) {
		oldPen = pDC->SelectObject(&redPen);
		for (i=0; i<pDoc->atisIndex.GetSize(); i++) {
			if (planOnly) {
				int proceed = FALSE;
				for (int q=0; q<plancount; q++)
					if (pDoc->atisIndex[i] == plan[q]) {
						proceed = TRUE;
						break;
				}
				if (!proceed)
					continue;
			}
			int x, y;
			if (!LtoC(pDoc->db[pDoc->atisIndex[i]].lon, pDoc->db[pDoc->atisIndex[i]].lat, x, y)) continue;
			if (mag > symbolTransition) {
				pDC->MoveTo(x-1, y-1);
				pDC->LineTo(x+2, y-1);
				pDC->LineTo(x+2, y+2);
				pDC->LineTo(x-1, y+2);
				pDC->LineTo(x-1, y-1);
				if (id || freq) {
					int offset = 10;
					int chars;
					char buf[50];
					CSize tSize; 
					if (freq && id)	
						chars = sprintf(buf, "ATIS %6.2f", pDoc->db[pDoc->atisIndex[i]].freq);
					else if (id) 
						chars = sprintf(buf, "ATIS");
					else if (freq) 
						chars = sprintf(buf, "%6.2f", pDoc->db[pDoc->atisIndex[i]].freq);
					tSize = pDC->GetOutputTextExtent(buf, chars); 
					idPlace idP2 = {x, y, x+1, y+1};
					idlist.Add(idP2);
					idPlace idP = {x+offset, y+offset, x+offset+tSize.cx+4, y+offset+tSize.cy};
					idPlacement(x, y, idP, &idlist);
					pDC->Rectangle(idP.x, idP.y, idP.sx, idP.sy);
					pDC->TextOut(idP.x+2, idP.y, buf);
				}
			} else {
				pDC->MoveTo(x-1, y);
				pDC->LineTo(x+2, y);
				pDC->MoveTo(x, y-1);
				pDC->LineTo(x, y+2);
			}
		}
		pDC->SelectObject(oldPen);
	}

	if(ndb) {
		oldBrush = (CBrush *) pDC->SelectStockObject(WHITE_BRUSH);
		oldPen = pDC->SelectObject(&greenPen);
		for (i=0; i<pDoc->ndbIndex.GetSize(); i++) {
			if (planOnly) {
				int proceed = FALSE;
				for (int q=0; q<plancount; q++)
					if (pDoc->ndbIndex[i] == plan[q]) {
						proceed = TRUE;
						break;
				}
				if (!proceed)
					continue;
			}

			if (!pDC->IsPrinting()) {
				if (drawMessageLoop() == TRUE) { // clean up and exit
					pDC->SelectObject(oldBrush);
					pDC->SelectObject(oldPen);
					pDC->SetBkMode(oldMode);
					pDC->RestoreDC(dcNumber);
					if (id) {
						idlist.RemoveAll();
						idlist.FreeExtra();
					}
					LeaveCriticalSection(&crit);
					return;
				}
			}

			int x,y;
			if (!LtoC(pDoc->db[pDoc->ndbIndex[i]].lon, pDoc->db[pDoc->ndbIndex[i]].lat, x, y)) continue;

			// draw a 3 by 3 pixel cross
			int offset = 0;
			if (mag > symbolTransition) {
				pDC->SelectObject(oldBrush);
				oldBrush = (CBrush *) pDC->SelectStockObject(LTGRAY_BRUSH);
				pDC->Ellipse(x-vorsize/6, y-vorsize/6, x+vorsize/6, y+vorsize/6);
				offset = vorsize/6-2;
				pDC->SelectObject(oldBrush);
				oldBrush = (CBrush *) pDC->SelectStockObject(WHITE_BRUSH);
				pDC->Ellipse(x-vorsize/12, y-vorsize/12, x+vorsize/12, y+vorsize/12);
			} else {
				pDC->MoveTo(x-1, y);
				pDC->LineTo(x+2, y);
				pDC->MoveTo(x, y-1);
				pDC->LineTo(x, y+2);
			}
			if ((largeSymbols) && (mag > symbolTransition)) {
				if (id || freq) {
					int chars, chars2;
					CString s1, s2;
					char buf2[50];
					CSize tSize, tSize2;
					s1 = pDoc->db[pDoc->ndbIndex[i]].name;
					s1.TrimRight();
					s2 = pDoc->db[pDoc->ndbIndex[i]].id;
					s2.TrimRight();
					if (freq && id)	{
						chars = sprintf(buf, "%s", (LPCTSTR) s1);
						chars2 = sprintf(buf2, "%6.2f %s", pDoc->db[pDoc->ndbIndex[i]].freq, (LPCTSTR) s2);
						tSize = pDC->GetOutputTextExtent(buf, chars); 
						tSize2 = pDC->GetOutputTextExtent(buf2, chars2);
						if (tSize2.cx > tSize.cx)
							tSize.cx = tSize2.cx;
					} else if (id) {
						chars = sprintf(buf, "%s", (LPCTSTR) s1);
						chars2 = sprintf(buf2, "%s", (LPCTSTR) s2);
						tSize = pDC->GetOutputTextExtent(buf, chars); 
						tSize2 = pDC->GetOutputTextExtent(buf2, chars2); 
						if (tSize2.cx > tSize.cx)
							tSize.cx = tSize2.cx;
					} else if (freq) {
						chars = sprintf(buf, "%6.2f", pDoc->db[pDoc->ndbIndex[i]].freq);
						tSize = pDC->GetOutputTextExtent(buf, chars); 
						tSize2.cx = tSize2.cy = 0;
					}
					idPlace idP2 = {x, y, x+1, y+1};
					idlist.Add(idP2);
					idPlace idP = {x-offset, y+offset/2, x-offset+tSize.cx+4, y+offset/2+tSize.cy+tSize2.cy};
					idPlacement(x, y, idP, &idlist);
					pDC->Rectangle(idP.x, idP.y, idP.x+tSize.cx+4, idP.y+tSize.cy+tSize2.cy);
					pDC->TextOut(idP.x+2, idP.y, buf);
					if (!freq || id) 
						pDC->TextOut(idP.x+2, idP.y+tSize2.cy, buf2);
				}
			} else {
				if (id || freq) {
					if (freq && id)
						sprintf(buf, "%s (%6.2f)", pDoc->db[pDoc->ndbIndex[i]].id, pDoc->db[pDoc->ndbIndex[i]].freq);
					else if (id)
						sprintf(buf, "%s", pDoc->db[pDoc->ndbIndex[i]].id);
					else if (freq)
						sprintf(buf, "%6.2f", pDoc->db[pDoc->ndbIndex[i]].freq);
					pDC->TextOut(x, y+offset+4, buf);
				}
			}
		}
		pDC->SelectObject(oldBrush);
		pDC->SelectObject(oldPen);
	}

//++ JYL
	if(wpt) {
		oldBrush = (CBrush *) pDC->SelectStockObject(WHITE_BRUSH);
		oldPen = pDC->SelectObject(&wayPtPen);
		for (i=0; i<pDoc->wptIndex.GetSize(); i++) {
			if (planOnly) {
				int proceed = FALSE;
				for (int q=0; q<plancount; q++)
					if (pDoc->wptIndex[i] == plan[q]) {
						proceed = TRUE;
						break;
				}
				if (!proceed)
					continue;
			}

			if (!pDC->IsPrinting()) {
				if (drawMessageLoop() == TRUE) { // clean up and exit
					pDC->SelectObject(oldBrush);
					pDC->SelectObject(oldPen);
					pDC->SetBkMode(oldMode);
					pDC->RestoreDC(dcNumber);
					if (id) {
						idlist.RemoveAll();
						idlist.FreeExtra();
					}
					LeaveCriticalSection(&crit);
					return;
				}
			}

			int x,y;
			if (!LtoC(pDoc->db[pDoc->wptIndex[i]].lon, pDoc->db[pDoc->wptIndex[i]].lat, x, y)) continue;

			// draw a 3 by 3 pixel cross
			int offset = 0;
				pDC->MoveTo(x-1, y);
				pDC->LineTo(x+2, y);
				pDC->MoveTo(x, y-1);
				pDC->LineTo(x, y+2);
			if (mag > symbolTransition) 
			{
				int width = vorsize / 12;
				int height = vorsize / 6;

				pDC->MoveTo(x-width, y);
				pDC->LineTo(x, y+height);
				pDC->LineTo(x+width, y);
				pDC->LineTo(x, y-height);
				pDC->LineTo(x-width, y);
			}

			if ((largeSymbols) && (mag > symbolTransition)) {
				if (id ) 
				{
					int chars, chars2;
					CString s1, s2;
					char buf2[50];
					CSize tSize, tSize2;
					s1 = pDoc->db[pDoc->wptIndex[i]].name;
					s1.TrimRight();
					s2 = pDoc->db[pDoc->wptIndex[i]].id;
					s2.TrimRight();

					if (id) 
					{
						chars = sprintf(buf, "%s", (LPCTSTR) s1);
						chars2 = sprintf(buf2, "%s", (LPCTSTR) s2);
						tSize = pDC->GetOutputTextExtent(buf, chars); 
						tSize2 = pDC->GetOutputTextExtent(buf2, chars2); 
						if (tSize2.cx > tSize.cx)
							tSize.cx = tSize2.cx;
					}
					idPlace idP2 = {x, y, x+1, y+1};
					idlist.Add(idP2);
					idPlace idP = {x-offset, y+offset/2, x-offset+tSize.cx+4, y+offset/2+tSize.cy+tSize2.cy};
					idPlacement(x, y, idP, &idlist);
					pDC->Rectangle(idP.x, idP.y, idP.x+tSize.cx+4, idP.y+tSize.cy+tSize2.cy);
					if (s1 != "")
					{
						pDC->TextOut(idP.x+2, idP.y, buf);
						if (s2 != "")
							pDC->TextOut(idP.x+2, idP.y+tSize2.cy, buf2);
					}
					else
					{
						if (s2 != "")
							pDC->TextOut(idP.x+2, idP.y, buf2);
					}

				}
			} 
			else 
			{
					if (id)
					{
						sprintf(buf, "%s", pDoc->db[pDoc->wptIndex[i]].id);
						pDC->TextOut(x, y+offset+4, buf);
					}
			}
		}
		pDC->SelectObject(oldBrush);
		pDC->SelectObject(oldPen);
	}

	// -- JYL

	if (vor) { 
		oldPen = pDC->SelectObject(&bluePen);
		oldBrush = (CBrush *) pDC->SelectStockObject(NULL_BRUSH);
		for (i=0; i<pDoc->vorIndex.GetSize(); i++) {
			if (planOnly) {
				int proceed = FALSE;
				for (int q=0; q<plancount; q++)
					if (pDoc->vorIndex[i] == plan[q]) {
						proceed = TRUE;
						break;
				}
				if (!proceed)
					continue;
			}

			if (!pDC->IsPrinting()) {
				if (drawMessageLoop() == TRUE) { // clean up and exit
					pDC->SelectObject(oldBrush);
					pDC->SelectObject(oldPen);
					pDC->SetBkMode(oldMode);
					pDC->RestoreDC(dcNumber);
					if (id) {
						idlist.RemoveAll();
						idlist.FreeExtra();
					}
					LeaveCriticalSection(&crit);
					return;
				}
			}

			int x,y;
			if (!LtoC(pDoc->db[pDoc->vorIndex[i]].lon, pDoc->db[pDoc->vorIndex[i]].lat, x, y)) continue;

			// draw a 3 by 3 pixel box
			int offset = 0;
			pDC->MoveTo(x-2, y-2);
			pDC->LineTo(x+1, y-2);
			pDC->LineTo(x+1, y+1);
			pDC->LineTo(x-2, y+1);
			pDC->LineTo(x-2, y-2);
			if (mag > symbolTransition) {
				pDC->Ellipse(x-vorsize, y-vorsize, x+vorsize, y+vorsize);
				pDC->MoveTo(x,y);
				double lx = (vorsize * -sin(pDoc->db[pDoc->vorIndex[i]].magdev * degToRad));
				double ly = (vorsize * -cos(pDoc->db[pDoc->vorIndex[i]].magdev * degToRad));
				pDC->LineTo(x + (int) lx, y + (int) ly);
				int tick = 30;
				if (largeSymbols) 
					tick = 10;
				for (int a = 0; a<180; a+=tick) {
					lx = vorsize * sin((pDoc->db[pDoc->vorIndex[i]].magdev + a) * degToRad);
					ly = vorsize * cos((pDoc->db[pDoc->vorIndex[i]].magdev + a) * degToRad);
					if (a%30 == 0) {	 // big ticks (30 degrees)
						pDC->MoveTo(x + (int) lx, y + (int) ly);
						pDC->LineTo(x + (int) (lx * .80), y + (int) (ly * .80));
						pDC->MoveTo(x - (int) lx, y - (int) ly);
						pDC->LineTo(x - (int) (lx * .80), y - (int) (ly * .80));
					} else {			 // medium ticks (10 degrees)
						pDC->MoveTo(x + (int) lx, y + (int) ly);
						pDC->LineTo(x + (int) (lx * .9), y + (int) (ly * .9));
						pDC->MoveTo(x - (int) lx, y - (int) ly);
						pDC->LineTo(x - (int) (lx * .9), y - (int) (ly * .9));
					}
					if (largeSymbols) {	 // small ticks (5 degrees)
						lx = vorsize * sin((pDoc->db[pDoc->vorIndex[i]].magdev + a + tick/2) * degToRad);
						ly = vorsize * cos((pDoc->db[pDoc->vorIndex[i]].magdev + a + tick/2) * degToRad);
						pDC->MoveTo(x + (int) lx, y + (int) ly);
						pDC->LineTo(x + (int) (lx * .96), y + (int) (ly * .96));
						pDC->MoveTo(x - (int) lx, y - (int) ly);
						pDC->LineTo(x - (int) (lx * .96), y - (int) (ly * .96));
					}
				}
				offset = vorsize - 2;
			}
			if ((largeSymbols) && (mag > symbolTransition)) {
				if (id || freq) {
					int chars, chars2;
					CString s1, s2;
					char buf2[50];
					CSize tSize, tSize2;
					oldBrush = (CBrush *) pDC->SelectStockObject(WHITE_BRUSH);
					s1 = pDoc->db[pDoc->vorIndex[i]].name;
					s1.TrimRight();
					s2 = pDoc->db[pDoc->vorIndex[i]].id;
					s2.TrimRight();
					if (freq && id)	{
						chars = sprintf(buf, "%s", (LPCTSTR) s1);
						chars2 = sprintf(buf2, "%6.2f %s", pDoc->db[pDoc->vorIndex[i]].freq, (LPCTSTR) s2);
						tSize = pDC->GetOutputTextExtent(buf, chars); 
						tSize2 = pDC->GetOutputTextExtent(buf2, chars2);
						if (tSize2.cx > tSize.cx)
							tSize.cx = tSize2.cx;
					} else if (id) {
						chars = sprintf(buf, "%s", (LPCTSTR) s1);
						chars2 = sprintf(buf2, "%s", (LPCTSTR) s2);
						tSize = pDC->GetOutputTextExtent(buf, chars); 
						tSize2 = pDC->GetOutputTextExtent(buf2, chars2); 
						if (tSize2.cx > tSize.cx)
							tSize.cx = tSize2.cx;
					} else if (freq) {
						chars = sprintf(buf, "%6.2f", pDoc->db[pDoc->vorIndex[i]].freq);
						tSize = pDC->GetOutputTextExtent(buf, chars); 
						tSize2.cx = tSize2.cy = 0;
					}
					idPlace idP2 = {x, y, x+1, y+1};
					idlist.Add(idP2);
					idPlace idP = {x-offset, y+offset/2, x-offset+tSize.cx+4, y+offset/2+tSize.cy+tSize2.cy};
					idPlacement(x, y, idP, &idlist);
					pDC->Rectangle(idP.x, idP.y, idP.x+tSize.cx+4, idP.y+tSize.cy+tSize2.cy);
					pDC->TextOut(idP.x+2, idP.y, buf);
					if (!freq || id) 
						pDC->TextOut(idP.x+2, idP.y+tSize2.cy, buf2);
					pDC->SelectObject(oldBrush);
				}
			} else {
				if (id || freq) {
					if (freq && id)
						sprintf(buf, "%s (%6.2f)", pDoc->db[pDoc->vorIndex[i]].id, pDoc->db[pDoc->vorIndex[i]].freq);
					else if (id)
						sprintf(buf, "%s", pDoc->db[pDoc->vorIndex[i]].id);
					else if (freq)
						sprintf(buf, "%6.2f", pDoc->db[pDoc->vorIndex[i]].freq);
					pDC->TextOut(x, y+offset+4, buf);
				}
			}
		}
		pDC->SelectObject(oldBrush);
		pDC->SelectObject(oldPen);
	}
	
	if (ils) {	
		oldPen = pDC->SelectObject(&redPen);
		for (i=0; i<pDoc->ilsIndex.GetSize(); i++) {
			if (planOnly) {
				int proceed = FALSE;
				for (int q=0; q<plancount; q++)
					if (pDoc->ilsIndex[i] == plan[q]) {
						proceed = TRUE;
						break;
				}
				if (!proceed)
					continue;
			}

			if (!pDC->IsPrinting()) {
				if (drawMessageLoop() == TRUE) { // clean up and exit
					pDC->SelectObject(oldPen);
					pDC->SetBkMode(oldMode);
					pDC->RestoreDC(dcNumber);
					if (id) {
						idlist.RemoveAll();
						idlist.FreeExtra();
					}
					LeaveCriticalSection(&crit);
					return;
				}
			}

			int x,y;
			if (!LtoC(pDoc->db[pDoc->ilsIndex[i]].lon, pDoc->db[pDoc->ilsIndex[i]].lat, x, y)) continue;

			// draw a 3 by 3 pixel diagonal cross
			pDC->MoveTo(x-1, y-1);
			pDC->LineTo(x+3, y+3);
			pDC->MoveTo(x+2, y-1);
			pDC->LineTo(x-2, y+3);
			if (mag > symbolTransition) {
				double ldist; 
				int l = distToPixels(2000); // l = 2km
				int lx = (int) (l * sin(pDoc->db[pDoc->ilsIndex[i]].approach * degToRad));
				int ly = (int) (l * -cos(pDoc->db[pDoc->ilsIndex[i]].approach * degToRad));
 				if(flatWorld) { // calculate latitude distortion due to flat map
					ldist = 1.0/cos(pDoc->db[pDoc->ilsIndex[i]].lat * degToRad);
					lx = (int) (lx * ldist);
				}
				pDC->MoveTo(x, y);
				pDC->LineTo(x-lx-ly/10, y-ly+lx/10);
				pDC->LineTo(x-lx+ly/10, y-ly-lx/10);
				pDC->LineTo(x, y);
				if (id || freq) {
					int chars;
					CSize tSize; 
					if (freq && id)
						chars = sprintf(buf, "%s (%6.2f)", pDoc->db[pDoc->ilsIndex[i]].id, pDoc->db[pDoc->ilsIndex[i]].freq);
					else if (id)
						chars = sprintf(buf, "%s", pDoc->db[pDoc->ilsIndex[i]].id);
					else if (freq)
						chars = sprintf(buf, "%6.2f", pDoc->db[pDoc->ilsIndex[i]].freq);
					tSize = pDC->GetOutputTextExtent(buf, chars);
					int cx = (int) (x-lx);
					int cy = (int) (y-ly);
					idPlace idP = {(int) (x-1.5*lx), (int) (y-1.5*ly), (int) (x-1.5*lx)+tSize.cx+4, (int) (y-1.5*ly)+tSize.cy};
					idPlacement(cx, cy, idP, &idlist, 1);
					pDC->Rectangle(idP.x, idP.y, idP.x+tSize.cx+4, idP.y+tSize.cy);
					// find closest corner
					int ccx = idP.x;
					int ccy = idP.y;
					double dn, d = sqrt((double)(idP.x-cx)*(idP.x-cx)+(idP.y-cy)*(idP.y-cy));
					if ((dn = sqrt((double)(idP.x-cx)*(idP.x-cx)+(idP.y+tSize.cy-cy)*(idP.y+tSize.cy-cy))) < d) {
						d = dn;
						ccx = idP.x;
						ccy = idP.y+tSize.cy;
					}
					if ((dn = sqrt((double)(idP.x+tSize.cx+4-cx)*(idP.x+tSize.cx+4-cx)+(idP.y-cy)*(idP.y-cy))) < d) {
						d = dn;
						ccx = idP.x+tSize.cx+4;
						ccy = idP.y;
					}
					if ((dn = sqrt((double)(idP.x+tSize.cx+4-cx)*(idP.x+tSize.cx+4-cx)+(idP.y+tSize.cy-cy)*(idP.y+tSize.cy-cy))) < d) {
						d = dn;
						ccx = idP.x+tSize.cx+4;
						ccy = idP.y+tSize.cy;
					}
					pDC->MoveTo(cx, cy);
					pDC->LineTo(ccx, ccy);
					pDC->TextOut(idP.x+2, idP.y, buf);
				}
			}
		}	
		pDC->SelectObject(oldPen);
	}	
		
	if (airports && (mag > symbolTransition)) {	// Runway
		for (i=0; i<pDoc->runwayIndex.GetSize(); i++) {
			if (planOnly) {
				int proceed = FALSE;
				for (int q=0; q<plancount; q++)
					if (pDoc->runwayIndex[i] == plan[q]) {
						proceed = TRUE;
						break;
				}
				if (!proceed)
					continue;
			}

			if (!pDC->IsPrinting()) {
				if (drawMessageLoop() == TRUE) { // clean up and exit
					pDC->SetBkMode(oldMode);
					pDC->RestoreDC(dcNumber);
					if (id) {
						idlist.RemoveAll();
						idlist.FreeExtra();
					}
					LeaveCriticalSection(&crit);
					return;
				}
			}

			int x,y;
			if (!LtoC(pDoc->db[pDoc->runwayIndex[i]].lon, pDoc->db[pDoc->runwayIndex[i]].lat, x, y)) continue;

			double ldist;
			int l = distToPixels(pDoc->db[pDoc->runwayIndex[i]].range * 0.3048);
			int lx = (int) ((double) l * sin(pDoc->db[pDoc->runwayIndex[i]].approach * degToRad));
			int ly = (int) ((double) l * -cos(pDoc->db[pDoc->runwayIndex[i]].approach * degToRad));
 			if(flatWorld) { // calculate latitude distortion due to flat map
				ldist = 1.0/cos(pDoc->db[pDoc->runwayIndex[i]].lat * degToRad);
				lx = (int) (lx * ldist);
			}

			if (OS != 3) { // not Win32s (it can't handle the wide pen for some reason)
				// Pen for runway width (when mag > 10)
				int rwWidth = 1;
				if (mag > 10) 
					rwWidth = max(1, distToPixels(pDoc->db[pDoc->runwayIndex[i]].magdev * 0.3048));
				LOGBRUSH lb;
				lb.lbStyle = BS_SOLID;
				lb.lbColor = RGB(0, 0, 0);
				CPen widePen(PS_GEOMETRIC | PS_ENDCAP_SQUARE | PS_JOIN_BEVEL, rwWidth, &lb);
				
				if (mag > 10) 
					oldPen = (CPen *) pDC->SelectObject(&widePen);	
				else 
					oldPen = (CPen *) pDC->SelectStockObject(BLACK_PEN);	// switch to black
				pDC->MoveTo(x-lx/2, y-ly/2);		// lat and lon are center of runway
				pDC->LineTo(x+lx/2, y+ly/2);
				pDC->SelectObject(oldPen);

			} else {  // OS is Win32s (no wide runways)
				oldPen = (CPen *) pDC->SelectStockObject(BLACK_PEN);	// switch to black
				pDC->MoveTo(x-lx/2, y-ly/2);		// lat and lon are center of runway
				pDC->LineTo(x+lx/2, y+ly/2);
				pDC->SelectObject(oldPen);
			}
		}	
	}
	
	if (airports && id && (mag > symbolTransition)) {	// airport name
		for (i=0; i<pDoc->airportIndex.GetSize(); i++) {
			if (planOnly) {
				int proceed = FALSE;
				for (int q=0; q<plancount; q++)
					if (pDoc->airportIndex[i] == plan[q]) {
						proceed = TRUE;
						break;
				}
				if (!proceed)
					continue;
			}

			if (!pDC->IsPrinting()) {
				if (drawMessageLoop() == TRUE) { // clean up and exit
					pDC->SetBkMode(oldMode);
					pDC->RestoreDC(dcNumber);
					if (id) {
						idlist.RemoveAll();
						idlist.FreeExtra();
					}
					LeaveCriticalSection(&crit);
					return;
				}
			}

			int x,y;
			if (!LtoC(pDoc->db[pDoc->airportIndex[i]].lon, pDoc->db[pDoc->airportIndex[i]].lat, x, y)) continue;
			int chars;
			CSize tSize;
			int l = distToPixels(2500);	// l = 2.5km
			l = max(l, 10);
			chars=sprintf(buf, "%s", pDoc->db[pDoc->airportIndex[i]].name);
			tSize = pDC->GetOutputTextExtent(buf, chars); 
			idPlace idP = {x, y+l+1, x+tSize.cx, y+l+1+tSize.cy};
			idPlacement(x, y, idP, &idlist);
			pDC->TextOut(idP.x, idP.y, buf);
		}
	}

	if (markers && (mag > symbolTransition)) {	// inner, middle and outer markers
		oldBrush = (CBrush *) pDC->SelectStockObject(LTGRAY_BRUSH);
		for (i=0; i<pDoc->markerIndex.GetSize(); i++) {
			if (planOnly) {
				int proceed = FALSE;
				for (int q=0; q<plancount; q++)
					if (pDoc->markerIndex[i] == plan[q]) {
						proceed = TRUE;
						break;
				}
				if (!proceed)
					continue;
			}

			if (!pDC->IsPrinting()) {
				if (drawMessageLoop() == TRUE) { // clean up and exit
					pDC->SelectObject(oldBrush);
					pDC->SetBkMode(oldMode);
					pDC->RestoreDC(dcNumber);
					if (id) {
						idlist.RemoveAll();
						idlist.FreeExtra();
					}
					LeaveCriticalSection(&crit);
					return;
				}
			}

			int x,y;
			if (!LtoC(pDoc->db[pDoc->markerIndex[i]].lon, pDoc->db[pDoc->markerIndex[i]].lat, x, y)) continue;

			if (5 == pDoc->db[pDoc->markerIndex[i]].type)
				oldPen = pDC->SelectObject(&orangePen);
			else if (6 == pDoc->db[pDoc->markerIndex[i]].type)
				oldPen = pDC->SelectObject(&greenPen);
			else if (7 == pDoc->db[pDoc->markerIndex[i]].type)
				oldPen = pDC->SelectObject(&redPen);
			pDC->Rectangle(CRect(x-2, y-2, x+2, y+2));
			pDC->SelectObject(oldPen);
		} 
 		pDC->SelectObject(oldBrush);
	}	

	oldPen = (CPen *) pDC->SelectObject(&orangePen);
	for (i=0; i<pDoc->gpsIndex.GetSize(); i++) {		// gps
		if (planOnly) {
			int proceed = FALSE;
			for (int q=0; q<plancount; q++)
				if (pDoc->gpsIndex[i] == plan[q]) {
					proceed = TRUE;
					break;
			}
			if (!proceed)
				continue;
		}

		if (!pDC->IsPrinting()) {
			if (drawMessageLoop() == TRUE) { // clean up and exit
				pDC->SelectObject(oldPen);
				pDC->SetBkMode(oldMode);
				pDC->RestoreDC(dcNumber);
				if (id) {
					idlist.RemoveAll();
					idlist.FreeExtra();
				}
				LeaveCriticalSection(&crit);
				return;
			}
		}

		int x,y;
		if (!LtoC(pDoc->db[pDoc->gpsIndex[i]].lon, pDoc->db[pDoc->gpsIndex[i]].lat, x, y)) continue;

		pDC->MoveTo(x, y-5);
		pDC->LineTo(x, y+5);
		pDC->MoveTo(x-5, y);
		pDC->LineTo(x+5, y);
		pDC->MoveTo(x-5, y);
		pDC->LineTo(x, y-5);
		pDC->LineTo(x+5, y);
		pDC->LineTo(x, y+5);
		pDC->LineTo(x-5, y);
		if (id) {
			sprintf(buf, "GPS fix %s", pDoc->db[pDoc->gpsIndex[i]].id);
			pDC->TextOut(x-5, y+6, buf);
		}
	}
	pDC->SelectObject(oldPen);

	pDC->SelectObject(oldFont);
	if (title != "") {
		oldFont = (CFont *) pDC->SelectObject(&titleFont);
		int chars;
		chars = sprintf(buf, "%s", title);
		CSize tSize = pDC->GetOutputTextExtent(buf, chars); 
		pDC->TextOut(screenWidth/2-tSize.cx/2, 1, buf); // center top
		pDC->SelectObject(oldFont);
	}
 
	if (flying) { // draw FlyTo target
		int x,y;
		if (LtoC(flyingLon, flyingLat, x, y)) {
			CBrush redBrush(RGB(255, 0, 0));
			oldPen = pDC->SelectObject(&redPen);
			oldBrush = (CBrush *) pDC->SelectObject(&redBrush);
			pDC->Ellipse(x-8, y-8, x+8, y+8);
			pDC->SelectObject(oldBrush);
			oldBrush = (CBrush *) pDC->SelectStockObject(WHITE_BRUSH);
			pDC->Ellipse(x-6, y-6, x+6, y+6);
			pDC->SelectObject(oldBrush);
			oldBrush = (CBrush *) pDC->SelectObject(&redBrush);
			pDC->Ellipse(x-3, y-3, x+3, y+3);
			pDC->SelectObject(oldBrush);
			pDC->SelectObject(oldPen);
		}
	}

	if (flightPaths > 0) { // draw imported flight path
		int x, y;
		oldPen = pDC->SelectObject(&redPen);
		int starttime = flightTrackDensity;
	  	for (int k=0; k<fltPath.GetSize(); k++) {
			double lat = fltPath[k].lat;
			double lon = fltPath[k].lon;
			LtoC(lon, lat, x, y);
			if (k == 0)
				pDC->MoveTo(x, y);
			else {
				if (k >= starttime) {
					starttime = k + flightTrackDensity;
					pDC->LineTo(x, y);
				}
			}
		}
		pDC->SelectObject(oldPen);
	}

	if (!pDC->IsPrinting()) { // draw center mark on screen
		int centerX = screenWidth/2;
		int centerY = screenHeight/2;
		pDC->MoveTo(centerX-2, centerY);
		pDC->LineTo(centerX+3, centerY);
		pDC->MoveTo(centerX, centerY-2);
		pDC->LineTo(centerX, centerY+3);
	}

	if (id) {
		idlist.RemoveAll();
		idlist.FreeExtra();
	}

	clipper.DeleteObject();
	pDC->SelectClipRgn(NULL);

	pDC->SetBkMode(oldMode);
	pDC->RestoreDC(dcNumber);

	LocationBar();
	LeaveCriticalSection(&crit);
}

/////////////////////////////////////////////////////////////////////////////
// CNavView printing

BOOL CNavView::OnPreparePrinting(CPrintInfo* pInfo) {
	pInfo->SetMinPage(1);
	pInfo->SetMaxPage(1);
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CNavView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/) {
	// TODO: add extra initialization before printing
}

void CNavView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/) {
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CNavView diagnostics

#ifdef _DEBUG
void CNavView::AssertValid() const {
	CView::AssertValid();
}

void CNavView::Dump(CDumpContext& dc) const	{
	CView::Dump(dc);
}

CNavDoc* CNavView::GetDocument() { // non-debug version is inline
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CNavDoc)));
	return (CNavDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CNavView message handlers

void CNavView::OnViewZoomin() {
	double xd = (right - left) / 4.;
	double yd = (top - bottom) / 4.;
	right -= xd;
	left += xd;
	top -= yd;
	bottom += yd;

	convX *= 2;
	convY *= 2;

	mag += 1;
	mag2 *= 2;
	LocationBar();
	Mready = FALSE;
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);	
}

void CNavView::OnUpdateViewZoomout(CCmdUI* pCmdUI) {
	pCmdUI->Enable(mag != 1);		
}

void CNavView::OnViewZoomout() {
	if(mag > 1) {
		double xd = (right - left) / 2.;
		double yd = (top - bottom) / 2.;
		right += xd;
		left -= xd;
		top += yd;
		bottom -= yd;

		convX /= 2;
		convY /= 2;

		mag -= 1;
		mag2 /= 2;
		LocationBar();
		Mready = FALSE;
		abort = TRUE; // abort old redraw if necessary
		Invalidate(TRUE);
	}	
}

void CNavView::OnLButtonDown(UINT nFlags, CPoint point) {
	TRACE("\nCNavView::OnLButtonDown");
	double rx, ry;
	if (!CtoL(point.x, point.y, rx, ry))
		return;
	double yd = (top + bottom)/2.0 - ry;
	double xd = (right + left)/2.0 - rx;
	left -= xd;
	right -= xd;
	top -= yd;
	bottom -= yd;

	centerLat = ry;
	centerLon = rx;

	LocationBar();
	Mready = FALSE;
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}


void CNavView::OnLButtonDblClk(UINT nFlags, CPoint point) {
	//OnLButtonDown(nFlags, point); // same as single click
	TRACE("\nCNavView::OnLButtonDblClk");
}

void CNavView::OnSize(UINT nType, int cx, int cy) {
	CView::OnSize(nType, cx, cy);

	Size(cx, cy);
	Mready = FALSE;
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnViewGridlines() {
	grid = !grid;
	char buffer[30];
	sprintf(buffer, "%d", grid);
	WritePrivateProfileString("View", "grid", buffer, "nav.ini");
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnUpdateViewGridlines(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(grid);
}

void CNavView::OnViewNdb() {
	ndb = !ndb;	
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnUpdateViewNdb(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(ndb);	
}

void CNavView::OnViewMarkers() {
	markers = !markers;	
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnUpdateViewMarkers(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(markers);	
}

void CNavView::OnMapAtis() {
	atis = !atis;	
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnUpdateMapAtis(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(atis);	
}

void CNavView::OnViewIls() {
	ils = !ils;
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnUpdateViewIls(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(ils);	
}

void CNavView::OnViewVor() {
	vor = !vor;	
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnUpdateViewVor(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(vor);	
}

void CNavView::OnViewAirports() {
	airports = !airports;	
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnUpdateViewAirports(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(airports);	
}

void CNavView::OnViewId() {
	id = !id;	
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnUpdateViewId(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(id);	
}

void CNavView::OnViewFrequency() {
	freq = !freq;	
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnUpdateViewFrequency(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(freq);	
}

void CNavView::OnEditChangefont() {
	CFontDialog d(&lf, CF_SCREENFONTS|CF_INITTOLOGFONTSTRUCT, NULL, this);
	if (d.DoModal() == IDOK) {
		// asserts in debug mode - I don't know why, it seems to work
		d.GetCurrentFont(&lf);
		Invalidate(TRUE);
	}
}

void CNavView::Size(int cx, int cy) {
	if ((cx == 0) && (cy == 0)) // happens during initialization
		return;
	convX = convY = mag2;
	if (cx > cy) 
		convX *= cy / (double) cx;
	else 
		convY *= cx / (double) cy;

	if (!flatWorld) 
		return;

	double xcent = (right + left)/2.0;
	double ycent = (top + bottom)/2.0;
	double ratio = (double) cx / cy;

	// proportion correctly
	if (ratio > 2) {
		top = 90;
		bottom = -90;
		left = -top * ratio;
		right = top * ratio;		
	} else {
		left = -180;
		right = 180;		
		top = right / ratio;
		bottom = -right / ratio;
	}
	centerLat = xcent;
	centerLon = ycent;

	// scale
	for (int i=1; i<mag; i++) {
		double xd = (right - left) / 4.;
		double yd = (top - bottom) / 4.;
		right -= xd;
		left += xd;
		top -= yd;
		bottom += yd;
	}

	// center
	double xd = (right + left) / 2.;
	double yd = (top + bottom) / 2.;
	right += xcent-xd;
	left += xcent-xd;
	top += ycent-yd;
	bottom += ycent-yd;
}

void CNavView::OnEditSettitle() {
	SetTitle d;	

	d.m_title = title;
	if (d.DoModal() == IDOK) {
		title = d.m_title;
		Invalidate(TRUE);
	}
}

void CNavView::OnViewSearch() {
	DataDialog d;	
	Search d2;
	
	if (d2.DoModal() == IDOK) {
		search = d2.m_target;
		if (search != "")	
			d.DoModal();
	} 
	search = "";
}

void CNavView::OnViewDatabase() {
	DataDialog d;	
	d.DoModal();
}

void CNavView::OnEditSaveoptions() {
	char buffer[50];
/*	sprintf(buffer, "%f", top);
	WritePrivateProfileString("View", "top", buffer, "nav.ini");
	sprintf(buffer, "%f", bottom);
	WritePrivateProfileString("View", "bottom", buffer, "nav.ini");
	sprintf(buffer, "%f", right);
	WritePrivateProfileString("View", "right", buffer, "nav.ini");
	sprintf(buffer, "%f", left);
	WritePrivateProfileString("View", "left", buffer, "nav.ini");
	sprintf(buffer, "%d", mag);
	WritePrivateProfileString("View", "mag", buffer, "nav.ini");
	sprintf(buffer, "%d", mag2);
	WritePrivateProfileString("View", "mag2", buffer, "nav.ini");
*/
	sprintf(buffer, "%d", largeSymbols);
	WritePrivateProfileString("View", "largeSymbols", buffer, "nav.ini");
	sprintf(buffer, "%d", grid);
	WritePrivateProfileString("View", "grid", buffer, "nav.ini");
	sprintf(buffer, "%d", vor);
	WritePrivateProfileString("View", "vor", buffer, "nav.ini");
	sprintf(buffer, "%d", ils);
	WritePrivateProfileString("View", "ils", buffer, "nav.ini");
	sprintf(buffer, "%d", ndb);
	WritePrivateProfileString("View", "ndb", buffer, "nav.ini");
	sprintf(buffer, "%d", airports);
	WritePrivateProfileString("View", "airports", buffer, "nav.ini");
	sprintf(buffer, "%d", markers);
	WritePrivateProfileString("View", "markers", buffer, "nav.ini");
	sprintf(buffer, "%d", atis);
	WritePrivateProfileString("View", "atis", buffer, "nav.ini");
	sprintf(buffer, "%d", id);
	WritePrivateProfileString("View", "id", buffer, "nav.ini");
	sprintf(buffer, "%d", freq);
	WritePrivateProfileString("View", "freq", buffer, "nav.ini");
	sprintf(buffer, "%d", flatWorld);
	WritePrivateProfileString("View", "flatWorld", buffer, "nav.ini");
	sprintf(buffer, "%d", planVOR);
	WritePrivateProfileString("View", "planVOR", buffer, "nav.ini");
	sprintf(buffer, "%d", planNDB);
	WritePrivateProfileString("View", "planNDB", buffer, "nav.ini");
	sprintf(buffer, "%d", planAirport);
	WritePrivateProfileString("View", "planAirport", buffer, "nav.ini");
	sprintf(buffer, "%d", planContinue);
	WritePrivateProfileString("View", "planContinue", buffer, "nav.ini");
	sprintf(buffer, "%f", idealDist);
	WritePrivateProfileString("View", "idealDist", buffer, "nav.ini");
	sprintf(buffer, "%d", showAirportZones);
	WritePrivateProfileString("View", "showAirportZones", buffer, "nav.ini");
	sprintf(buffer, "%d", showMaps);
	WritePrivateProfileString("View", "showMaps", buffer, "nav.ini");
	sprintf(buffer, "%d", link);
	WritePrivateProfileString("View", "Link", buffer, "nav.ini");
	sprintf(buffer, "%d", linkUpdateTime);
	WritePrivateProfileString("View", "linkUpdateTime", buffer, "nav.ini");
	sprintf(buffer, "%d", linkFlyUpdateTime);
	WritePrivateProfileString("View", "linkFlyUpdateTime", buffer, "nav.ini");
	sprintf(buffer, "%d", linkFlySetMode);
	WritePrivateProfileString("View", "linkFlySetMode", buffer, "nav.ini");
	sprintf(buffer, "%.2f", linkFlyNbMilesToTurnBefore);
	WritePrivateProfileString("View", "linkFlyNbMilesToTurnBefore", buffer, "nav.ini");
	sprintf(buffer, "%d", flightTrackDensity);
	WritePrivateProfileString("View", "flightTrackDensity", buffer, "nav.ini");
}
int CNavView::SearchNavAidNearPoint(double startDist, CPoint point)
{
	CNavDoc* doc = GetDocument();
	double clat;
	double clon;
	if (!CtoL(point.x, point.y, clon, clat))
		return -1;

	int found = FALSE;
	int index, d1, d2;
	double dist;
		
	for (int i=0; i<doc->dbCount; i++) {
		if ((doc->db[i].type == 1) && (!ndb)) goto next;
		if ((doc->db[i].type == 2) && (!ils)) goto next;
		if ((doc->db[i].type == 3) && (!vor)) goto next;
		if ((doc->db[i].type == 4) && (!airports)) goto next;
		if ((doc->db[i].type == 5) && (!markers)) goto next;
		if ((doc->db[i].type == 6) && (!markers)) goto next;
		if ((doc->db[i].type == 7) && (!markers)) goto next;
		if ((doc->db[i].type == 8) && (!airports)) goto next;
		if ((doc->db[i].type == 10) && (!atis)) goto next;
		if ((doc->db[i].type == 11) && (!wpt)) goto next;
		// limit to window being viewed	 
		if (flatWorld) {
			if ((doc->db[i].lon) < left) goto next;
			if ((doc->db[i].lon) > right) goto next;
			if ((doc->db[i].lat) < bottom) goto next;
			if ((doc->db[i].lat) > top) goto next;
		} else 
			if(LtoC(doc->db[i].lon, doc->db[i].lat, d1, d2) == FALSE) goto next;
	
		dist = gcDist(clat, clon, doc->db[i].lat, doc->db[i].lon);
		if (dist < startDist) {
			startDist = dist;
			found = TRUE;
			index = i;
		}
		next:;
	}
	if (found)
		return index;
	else
		return -1;

}
void CNavView::OnRButtonDblClk(UINT nFlags, CPoint point) {
	// find the nearest thing in the database and show details
	measuring = FALSE;
	CNavDoc* doc = GetDocument();

	double startDist = gcDist(top, left, bottom, right);

	int index;
	index = SearchNavAidNearPoint(startDist,point);
	if (index >= 0) {
		CString details = "";
		doc->formatData(details, index);
		Details d;
		d.index = index;
		d.m_text = details;
		d.DoModal();
	}

	CView::OnRButtonDblClk(nFlags, point);
}

// great circle distance (in meters)
double CNavView::gcDist(double lat1, double lon1, double lat2, double lon2) {
	double dlon = (lon2-lon1) * degToRad;
	lat1 *= degToRad;
	lat2 *= degToRad;

	// calculate bearing in case anyone cares 
	bearing = atan2(sin(dlon), (-sin(lat1)*cos(dlon) + tan(lat2)*cos(lat1))) / degToRad;
	if (bearing < 0)
		bearing += 360;

	// calculate distance 
	// bad algorithm
 	// return(earthRadius*acos(sin(lat1)*sin(lat2) + cos(lat1)*cos(lat2)*cos(dlon)));

	// good algorithm
	double la = sin((lat2-lat1)/2);
	double lo = sin(dlon/2);
	double x = la*la + cos(lat1) * cos(lat2) * lo*lo;
	return(2*earthRadius*asin(sqrt(x)));
}

void CNavView::gcDistInterpolate(double lat1, double lon1, double lat2, double lon2, double percent, double &lat3, double &lon3) {
	double radius = 180 * degToMeters / 3.1415926536;
	double x1, y1, z1, x2, y2, z2;
	StoC(y1, z1, x1, lat1, lon1, radius);
	StoC(y2, z2, x2, lat2, lon2, radius);
	// rough approximation for now
	CtoS(y1+(y2-y1)*percent, z1+(z2-z1)*percent, x1+(x2-x1)*percent, lat3, lon3, radius);
}

// great circle distance (in meters) and bearing (in radians), and projection (meters)
int CNavView::gcDistDraw(double lat1, double lon1, double lat2, double lon2, double &dist, double &bearing, double &proj) {
	double dlon = (lon2-lon1) * degToRad;
	lat1 *= degToRad;
	lat2 *= degToRad;
	double la = sin((lat2-lat1)/2);
	double lo = sin(dlon/2);
	double x = sqrt(la*la + cos(lat1) * cos(lat2) * lo*lo); // sin (half-angle subtended)
	double a = asin(x);
	dist = 2*earthRadius*a;
	if (dist > dmax)
		return FALSE;	
	proj = earthRadius*sin(2*a);
	bearing = atan2(sin(dlon), (-sin(lat1)*cos(dlon) + tan(lat2)*cos(lat1)));
	return(TRUE);
}

// spherical to cartesian conversion (y=top, x=right, z=out-of-screen)
void CNavView::StoC(double &x, double &y, double &z, double lat, double lon, double r) {
	z = r * cos(lon * degToRad) * sin((90 - lat) * degToRad);
	x = r * sin(lon * degToRad) * sin((90 - lat) * degToRad);
	y = r * cos((90 - lat) * degToRad); 
}

// cartesian to spherical conversion (y=top, x=right, z=out-of-screen)
void CNavView::CtoS(double x, double y, double z, double &lat, double &lon, double &r) {
	lon = atan2(x, z) / degToRad;
	r = sqrt(x*x + y*y + z*z);
	lat = 90 - acos(y/r) / degToRad;
}

// spherical to cartesian conversion for normalized radius (y=top, x=right, z=out-of-screen)
void CNavView::StoCe(double &x, double &y, double &z, double lat, double lon) {
	z = cos(lon * degToRad) * sin((90 - lat) * degToRad);
	x = sin(lon * degToRad) * sin((90 - lat) * degToRad);
	y = cos((90 - lat) * degToRad); 
}

// cartesian to spherical conversion for normalized radius  (y=top, x=right, z=out-of-screen)
void CNavView::CtoSe(double x, double y, double z, double &lat, double &lon) {
	lon = atan2(x, z) / degToRad;
	lat = 90 - acos(y) / degToRad;
}

// draw a line along the great circle between two screen points (lat-lon pairs)
void CNavView::gcLine(int x1, int y1, int x2, int y2, CDC *pdc) {
	if ((x1==x2) &&(y1==y2))
		return;
	CDC *dc;
	if (pdc == NULL)
		dc = GetDC();
	else
		dc = pdc;
	CPen *oldPen = (CPen *) dc->SelectStockObject(BLACK_PEN);
	int oldROP = dc->SetROP2(R2_NOTXORPEN);
	dc->MoveTo(x1, y1);
	double slat;
	double slon;
	CtoL(x1, y1, slon, slat);
	double elat;
	double elon;
	CtoL(x2, y2, elon, elat);
	double radius = 180 * degToMeters / 3.1415926536;
	double bx, by, bz, ex, ey, ez, dx, dy, dz, px, py, pz, plat, plon, oldx;
	int res = 3;
	int pointx, pointy;
	StoC(by, bz, bx, slat, slon, radius);
	StoC(ey, ez, ex, elat, elon, radius);
	int d = (int) sqrt((double)(x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
	int steps = d/res;
	oldx = 0;
	bearing = 0;
	for (int i=1; i<steps; i++) {
		dx = (ex - bx)/steps;
		dy = (ey - by)/steps;
		dz = (ez - bz)/steps;
		px = bx+i*dx;
		py = by+i*dy;
		pz = bz+i*dz;
		CtoS(py, pz, px, plat, plon, radius);
		if (i==1) {	// calculate bearing in case anyone cares 
			bearing = atan2(plon-slon, plat-slat) / degToRad;
			if (bearing < 0)
				bearing += 360;
		}
		LtoC(plon, plat, pointx, pointy);
		if ((fabs(plon - oldx) > 100) && (flatWorld)) 
			dc->MoveTo(pointx, pointy);	// skip across the map edge
		else
			dc->LineTo(pointx, pointy);
		oldx = plon;
	} 
	dc->LineTo(x2, y2);
	dc->SelectObject(oldPen);
	dc->SetROP2(oldROP);
	if (pdc == NULL)
		ReleaseDC(dc);
}

void CNavView::OnRButtonDown(UINT nFlags, CPoint point) {
	measuring = TRUE;
	fromNavAid = FALSE;
	// look for a navaid not far from here
	double startDist;
	double startLat;
	double startLong;
	double endLat;
	double endLong;

	CtoL(point.x - 5, point.y - 5, startLong, startLat);
	CtoL(point.x + 5, point.y + 5, endLong, endLat);

	startDist = gcDist(startLat,startLong,endLat,endLong);

	navAidIndex = SearchNavAidNearPoint(startDist,point);
	imx = omx = point.x;
	imy = omy = point.y;	
	if (navAidIndex > 0)
	{
		fromNavAid = TRUE;
		int x,y;
		CNavDoc* pDoc = GetDocument();

		if (LtoC(pDoc->db[navAidIndex].lon, pDoc->db[navAidIndex].lat, x, y))
		{
			imx = omx = x;
			imy = omy = y;	
		}
	}
}

void CNavView::OnRButtonUp(UINT nFlags, CPoint point) 
{
	// if navaid not far don't ask for a fix

	measuring = FALSE; 
	gcLine(imx, imy, omx, omy, NULL); // undraw old line
	LocationBar();					  // reset status bar

	if (fromNavAid)
	{
		double startDist;
		double startLat;
		double startLong;
		double endLat;
		double endLong;

		CtoL(point.x - 5, point.y - 5, startLong, startLat);
		CtoL(point.x + 5, point.y + 5, endLong, endLat);

		startDist = gcDist(startLat,startLong,endLat,endLong);

		int index;

		index = SearchNavAidNearPoint(startDist,point);
		imx = omx = point.x;
		imy = omy = point.y;	
		if ( index < 0)
		{
			InsertGpsFixDial gpsFixDial;
			CNavDoc* pDoc = GetDocument();


			// ok ask if we want to insert a fix
			double lon2,lat2;
			if (CtoL(point.x, point.y, lon2, lat2))
			{
				CString navAidDetails;
				pDoc->formatData(navAidDetails,navAidIndex);
				gpsFixDial.m_navaidDetails = navAidDetails;
				gpsFixDial.m_gpsfix_distance = gcDist( pDoc->db[navAidIndex].lat,pDoc->db[navAidIndex].lon, lat2, lon2) / 1852;
				//bearing = (int)pDoc->GetMagVar(pDoc->db[navAidIndex].lat,pDoc->db[navAidIndex].lon);
				if (bearing > 360)
					bearing -= 360;
				if (bearing < 0)
					bearing += 360;
				gpsFixDial.m_gpsfix_heading = bearing;
				gpsFixDial.fromLat =  pDoc->db[navAidIndex].lat;
				gpsFixDial.fromLong =  pDoc->db[navAidIndex].lon;
				gpsFixDial.DoModal();
			}
		}
	}
	fromNavAid = FALSE;
}

void CNavView::OnMouseMove(UINT nFlags, CPoint point) {
	if (measuring) {
		CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
		double dist, lat1, lon1, lat2, lon2, magv;
		if (!CtoL(imx, imy, lon1, lat1))
			return;
		if (!CtoL(point.x, point.y, lon2, lat2))
			return;
		gcLine(imx, imy, omx, omy, NULL);		   // undraw old line
		gcLine(imx, imy, point.x, point.y, NULL);  // draw new line
		if ((imy == point.y) &&	(imx == point.x))
			dist = 0;
		else
			dist = gcDist(lat1, lon1, lat2, lon2);
		CNavDoc* doc = GetDocument();
		magv = doc->GetMagVar(lat1, lon1) + bearing;
		if (magv > 360.0)
			magv -= 360.0;
		if (magv < 0.0)
			magv += 360.0;

		// display on status bar
		char newPosition[100];

		double newLat2 = lat1;
		double newLong2 = lon1;

		ConvertDistHeadToCoord(dist,bearing,newLat2,newLong2);
		sprintf(newPosition,"-> % 8.6f, % 8.6f",newLat2,newLong2);
		char buffer[200];
		sprintf(buffer, "(% 8.6f, % 8.6f) to (% 8.6f, % 8.6f), %10.1f nm,  %1.1f degrees (%1.1f magnetic) %s", lat1, lon1, lat2, lon2, dist/1852, bearing, magv,newPosition);

		main->SetStatusBar(buffer);

		omx = point.x;
		omy = point.y;
	}
}

void CNavView::OnViewFlatworld(){
	flatWorld = !flatWorld;	
	Size(screenWidth, screenHeight);
	abort = TRUE; // abort old redraw if necessary
	Mready = FALSE;
	Invalidate(TRUE);
}

void CNavView::OnUpdateViewFlatworld(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(flatWorld);	 
}

void CNavView::OnEditFlightplan() {
	if (planning) {
		planning = FALSE;
		planD->DestroyWindow();	// let it deallocate its own memory
	} else {
		planning = TRUE;
		planD = new FlightPlan;	// corresponding delete is in FlightPlan::PostNcDestroy()
		planD->Create(IDD_FLIGHTPLAN);
	}
	Invalidate(TRUE);
}

void CNavView::OnUpdateEditFlightplan(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(planning);
}

void CNavView::OnPlanPlanoptions() {
	planOpts();
}

void CNavView::planOpts() {
	AutoOpt d;
	d.m_ideal = floor(idealDist/1.852); // convert from km to nm & round
	d.m_vor = planVOR;
	d.m_ndb = planNDB;
	d.m_airports = planAirport;
	d.m_continue = planContinue;

	if (d.DoModal() == IDOK) {
		idealDist = d.m_ideal * 1.852; // convert back to km
		planVOR = d.m_vor;
		planNDB = d.m_ndb;
		planAirport = d.m_airports;
		planContinue = d.m_continue;
	}		
}

void CNavView::OnMapShowPlandataonly() {
	planOnly = !planOnly;	
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnUpdateMapShowPlandataonly(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(planOnly);	
}

void CNavView::OnMapShowPlaninformation() {
	planInfo = !planInfo;	
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnUpdateMapShowPlaninformation(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(planInfo);				  	
}

void CNavView::LocationBar() {
	// update status bar with location information
	char  buffer[120], dirx, diry;
	double cx , cy;
	if (flatWorld) {
		cx = (left+right)/2.0;
		cy = (top+bottom)/2.0;
	} else {
		cx = centerLon;
		cy = centerLat;
	}
	if (cx < 0.) 
		dirx = 'W'; 
	else 
		dirx = 'E';
	if (cy < 0.) 
		diry = 'S'; 
	else 
		diry = 'N';
	double cxx = fabs(cx);
	int cxd = (int) (cxx);
	int cxm = (int) ((cxx-cxd) * 60);
	int cxs = (int) (((cxx-cxd)*60-cxm)*60);
	double cyy = fabs(cy);
	int cyd = (int) (cyy);
	int cym = (int) ((cyy-cyd) * 60);
	int cys = (int) (((cyy-cyd)*60-cym)*60);
	int chars;
	if (flatWorld)
		chars = sprintf(buffer, "center:% 8.6f, % 8.6f (%d %d' %d\"%c, %d %d' %d\"%c),  size: %d by %d nm", cy, cx, cyd, cym, cys, diry, cxd, cxm, cxs, dirx,
			(int)((right-left)*degToMeters/1852), (int)((top-bottom)*degToMeters/1852));
	else
		chars = sprintf(buffer, "center:% 8.6f, % 8.6f (%d %d' %d\"%c, %d %d' %d\"%c)", cy, cx, cyd, cym, cys, diry, cxd, cxm, cxs, dirx);
	((CMainFrame *) AfxGetMainWnd())->SetStatusBar(buffer);
}

// define flight data structure	for FSasmLib version 6.0.0
#pragma	pack(1)	// needed to correct alignment problems
typedef struct tagFLTREC_DATA {
  // FsasmLib.dll version
  WORD   wVersion1;   // 6
  WORD   wVersion2;   // 0
  WORD   wVersion3;   // 0
  WORD   wVersion4;   // build id
  // time of day
  WORD   wClockHour;  // hour
  WORD   wClockMin;   // minute
  WORD   wClockSec;   // second
  // position
  DWORD  dwLatLo;     // latitude  (low dword)
  DWORD  dwLatHi;     // latitude  (high dword)
  DWORD  dwLonLo;     // longitude (low dword)
  DWORD  dwLonHi;     // longitude (high dword)
  DWORD  dwAltLo;     // altitude  (low dword)
  DWORD  dwAltHi;     // altitude  (high dword)
  // orientation
  // angles: 180 deg == 0x8000
  DWORD  dwPitch;      // pitch
  DWORD  dwBank;       // bank
  DWORD  dwHeading;    // heading
  // airspeed
  // units: 256 * meters/second
  DWORD  dwTAS;        // true
  DWORD  dwIAS;        // indicated
  // fuel tank quantities 
  // percentages: 100% == 0x800000
  DWORD  dwFuelLeftPct;     // left
  DWORD  dwFuelLeftAuxPct;  // left aux
  DWORD  dwFuelRightPct;    // right
  DWORD  dwFuelRightAuxPct; // right aux
  // gear position
  // up == 0, down == 0xFFFF
  WORD   wGear1;
  WORD   wGear2;
  WORD   wGear3;
} FLTREC_DATA;

void CNavView::OnFileImport() {
	static char BASED_CODE szFilter[] = "FltRec Files (*.dat)|*.dat|All Files (*.*)|*.*||";
	CFileDialog d(TRUE, ".dat", NULL, OFN_LONGNAMES | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY, szFilter);	
	if (d.DoModal() == IDOK) {
		CString filename = d.GetPathName();

		// based on code from Adam Szofran <zof@halcyon.com>, author of fltrec
		FILE* in;
		FLTREC_DATA fltdat;
		long  time = 0;

		if ( (in = fopen(filename, "rb")) == NULL) {
			char buf[50];
			sprintf(buf, "Cannot open %s\n", filename);
			AfxMessageBox(buf);
			return;
		}

		if (flightPaths > 0) {
			fltPath.RemoveAll();
			fltPath.FreeExtra();
			flightPaths--;
		}
		fltPath.SetSize(0,100);
		struct path_t path;
		int size = sizeof(FLTREC_DATA); // may not work due to alignment problems?
		while ( fread( &fltdat, size, 1, in) == 1) { // read data from the file & convert
			double lat, lon;
			int intPart = fltdat.dwLatHi;
			if (intPart >= 0)
				lat = intPart + (double)fltdat.dwLatLo / (double)0x100000000;
			else		 
				lat = intPart - (double)fltdat.dwLatLo / (double)0x100000000;
			lat *= 45.0 / 5000875.0;

			intPart = fltdat.dwLonHi;
			if (intPart >= 0)
				lon = intPart + (double)fltdat.dwLonLo / (double)0x100000000;
			else
				lon = intPart - (double)fltdat.dwLonLo / (double)0x100000000;
			lon *= 90.0 / (256.0 * 4194304.0);
			
			path.lat = lat;
			path.lon = lon;
			path.time = time;
			fltPath.Add(path);
			time++;
		}
		fclose(in);
		flightPaths++;
		abort = TRUE; // abort old redraw if necessary
		Invalidate(TRUE);
	}
}
#pragma	pack()

void CNavView::OnUpdateFileImport(CCmdUI* pCmdUI) {
	pCmdUI->Enable(flightPaths == 0);
}

void CNavView::OnUpdateMapClearflighttrack(CCmdUI* pCmdUI) {
	pCmdUI->Enable(flightPaths > 0);
}

void CNavView::OnMapClearflighttrack() {
	if (flightPaths > 0) {
		fltPath.RemoveAll();
		fltPath.FreeExtra();
		flightPaths--;
		abort = TRUE; // abort old redraw if necessary
		Invalidate(TRUE);
	}
}

void CNavView::OnMapSetflighttrackdensity() {
	CFTDensity d;

	d.m_density = flightTrackDensity;
	if (d.DoModal() == IDOK) { 
		flightTrackDensity = d.m_density;
		char buffer[30];
		sprintf(buffer, "%d", flightTrackDensity);
		WritePrivateProfileString("View", "flightTrackDensity", buffer, "nav.ini");
		if (flightPaths > 0) {
			abort = TRUE; // abort old redraw if necessary
			Invalidate(TRUE);
		}
	}
}

int CNavView::distToPixels(double dist) { // convert a distance (in meters) to pixels
	const double c  = 10001750.0*4;  // circumference of earth (meters)
	const double c2 = 10001750.0*2;  // half circumference of earth (meters)
	if (flatWorld)
		if (screenWidth < screenHeight*2)
			return((int) (dist * screenWidth * (mag2) / c));
		else
			return((int) (dist * screenHeight * (mag2) / c2));
	else {
		double d = 2*earthRadius * asin(1.0/mag2);
		return((int) (dist * min(screenWidth, screenHeight) / d));
	}
}

void CNavView::OnMapLargesymbols() {
	largeSymbols = !largeSymbols;
	char buffer[30];
	sprintf(buffer, "%d", largeSymbols);
	WritePrivateProfileString("View", "largeSymbols", buffer, "nav.ini");
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnUpdateMapLargesymbols(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(largeSymbols);	
}

void CNavView::OnMapSettransition() {
	Transition d;
	d.m_trans = symbolTransition;
	if (d.DoModal() == IDOK) {
		symbolTransition = d.m_trans;
		abort = TRUE; // abort old redraw if necessary
		Invalidate(TRUE);
	}
}

void CNavView::OnMapZoomtofit() {
	ZoomToFit();
}

void CNavView::OnUpdateMapZoomtofit(CCmdUI* pCmdUI) {
	CNavDoc* doc = GetDocument();
	pCmdUI->Enable(doc->dbCount > 0);
}

void CNavView::ZoomToFit() {
	CNavDoc* doc = GetDocument();
	double minLat, maxLat, maxLon, minLon;
	mag = 1;
	mag2 = 1;
	minLat =  90.;
	bottom = -90.;
	maxLat = -90.;
	top = 	  90.;
	minLon =  180.;
	left =   -180.;
	maxLon = -180.;
	right =   180.;

	CRect r;		
	GetClientRect(r);
	Size(r.right, r.bottom);

	if (doc->dbCount == 0) { // just in case
		Mready = FALSE;
		Invalidate(TRUE);
		return;
	}
	for (int i=0; i<doc->dbCount; i++) {
		if ((doc->db[i].lat) < minLat)
			minLat = doc->db[i].lat;
		if ((doc->db[i].lat) > maxLat)
			maxLat = doc->db[i].lat;
		if ((doc->db[i].lon) < minLon)
			minLon = doc->db[i].lon;
		if ((doc->db[i].lon) > maxLon)
			maxLon = doc->db[i].lon;
	}

	// set new center
	double rx, ry, xd, yd;
	rx = (minLon+maxLon)/2;
	ry = (minLat+maxLat)/2;
	left   += rx;
	right  += rx;
	top    += ry;
	bottom += ry;

	// set new magnification
	if (doc->dbCount == 1) {
		mag = 10;
		mag2 = 512;
	} else {
		int repeat = TRUE;
		while (repeat) {
			// zoom in
			xd = (right - left) / 4.;
			yd = (top - bottom) / 4.;
			right -= xd;
			left += xd;
			top -= yd;
			bottom += yd;
			mag += 1;
			mag2 *= 2;
			convX *= 2;
			convY *= 2;

			if (mag > 15)
				repeat = FALSE;
			// check to see if we went too far
			if ((left > minLon)||(right < maxLon)||(bottom > minLat)||(top < maxLat))
				repeat = FALSE;
		}
		// undo one step
		xd = (right - left) / 2.;
		yd = (top - bottom) / 2.;
		right += xd;
		left -= xd;
		top += yd;
		bottom -= yd;
		mag -= 1;
		mag2 /= 2;
	}

	centerLat = ry;
	centerLon = rx;
	convX = convY = mag2;
	if (screenWidth > screenHeight) 
		convX *= screenHeight / (double) screenWidth;
	else 
		convY *= screenWidth / (double) screenHeight;

	if (mag == 1) { // always center map at mag=1
		bottom = -90.;
		top = 	  90.;
		left =   -180.;
		right =   180.;

		Size(r.right, r.bottom);
	}

	Mready = FALSE;
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnMapShowMapsymbolsAirportzones() {
	showAirportZones = !showAirportZones;	
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnUpdateMapShowMapsymbolsAirportzones(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(showAirportZones);	
}

void CNavView::OnMapShowMaps() {
	showMaps = !showMaps;	
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

void CNavView::OnUpdateMapShowMaps(CCmdUI* pCmdUI) {
	CNavDoc* pDoc = GetDocument();
	pCmdUI->Enable(pDoc->mapsExist);
	pCmdUI->SetCheck(showMaps && pDoc->mapsExist);	
}

void CNavView::CenterAt(double lat, double lon) {
	centerLat = lat;
	centerLon = lon;

	double xd = (right + left)/2.0 - lon;
	double yd = (top + bottom)/2.0 - lat;
	left -= xd;
	right -= xd;
	top -= yd;
	bottom -= yd;

	LocationBar();
	Mready = FALSE;
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
}

int CNavView::drawMessageLoop() {	// returns TRUE if abort was detected
	MSG msg;  
	while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) { 
		if (msg.message == WM_QUIT) { 
			PostQuitMessage(1);
		}
		if ((msg.message == WM_PAINT) && (abort == TRUE)) { 
			// abort this and start over
			abort = FALSE;
			return(TRUE);
		}
		TranslateMessage(&msg);
		DispatchMessage(&msg);	
	}
	return(FALSE);
}

void CNavView::OnMapCenterat() {
	CCenterAt d;
	
	d.m_lon = (right + left)/2.0;
	d.m_lat = (top + bottom)/2.0;

	if (d.DoModal() == IDOK)
		CenterAt(d.m_lat, d.m_lon);
}

double CNavView::sign(double x) {
	if (x<0.0)
		return -1.0;
	else
		return 1.0;
}

void CNavView::DrawMaps(CDC* pDC) {
	CNavDoc* pDoc = GetDocument();
	if (pDoc->mapsExist) {
		CPen grayPen(PS_SOLID, 0, RGB(192, 192, 192));
		CPen *oldPen = pDC->SelectObject(&grayPen);
		if (flatWorld) 	{
			int lat, lon, leftd, rightd, topd, bottomd, lastlon=0, rx, ry;
			double latf, lonf;
			double convertLat = 90.0/0x80000000; 
			double convertLon = 180.0/0x80000000;
			if (left < -180.0)
				leftd = 0x80000000;
			else
				leftd = (int) (left / convertLon);
			if (right >= 180.0)
				rightd = 0x7fffffff;
			else
				rightd = (int) (right / convertLon);
			if (top >= 90.0)
				topd = 0x7fffffff;
			else
				topd = (int) (top / convertLat);
			if (bottom < -90.0)
				bottomd = 0x80000000;
			else
				bottomd = (int) (bottom / convertLat);
			int loop = pDoc->mapTable.GetSize();
			int lastEdgeFlag = FALSE;
			int edgeFlag = FALSE;
			int moveFlag = TRUE;
			int wrapFlag = FALSE;
			int lastx, lasty;
			int decimate = 1;
			if (mag == 1)
				decimate = 4;
			else if (mag == 2)
				decimate = 3;
			else if (mag == 3)
				decimate = 2;
			int deccounter = decimate;
			oldPen = pDC->SelectObject(&grayPen);
			for (int m=0; m<loop; m+=2) {
				lat = pDoc->mapTable[m];
				lon = pDoc->mapTable[m+1];
				if (lat == 0) {
					if (lon == 0) {
						moveFlag = TRUE;
						deccounter = decimate;
						continue;
					}
				} else if ((decimate > 1) && (deccounter != decimate)) {
					if(deccounter == 0) {
						deccounter = decimate;
						continue;
					}
					deccounter--;
					continue;
				} else if (lon < leftd) 
					edgeFlag = TRUE;
				else if (lon > rightd) 
					edgeFlag = TRUE;
				else if (lat < bottomd) 
					edgeFlag = TRUE;
				else if (lat > topd) 
					edgeFlag = TRUE;
				if (mag == 1) { // check for date-line wrap around
					if (!moveFlag) {
						if ((lon > 0x70000000) && (lastlon < -0x70000000))
							wrapFlag = TRUE;
						else if ((lon < -0x70000000) && (lastlon > 0x70000000))
							wrapFlag = TRUE;
					}
				}
				latf = lat * convertLat;
				lonf = lon * convertLon;
				LtoC(lonf, latf, rx, ry);
				if (moveFlag)
					pDC->MoveTo(rx,ry);
				else if (edgeFlag && !lastEdgeFlag)	{
					pDC->LineTo(rx,ry);
					lastx = rx;
					lasty = ry;
				} else if (edgeFlag && lastEdgeFlag) {
					lastx = rx;
					lasty = ry;
				} else if (!edgeFlag && lastEdgeFlag) {
					pDC->MoveTo(lastx,lasty);
					pDC->LineTo(rx,ry);
				} else if (wrapFlag) {
					pDC->MoveTo(rx,ry);
					wrapFlag = FALSE;	
				} else 
					pDC->LineTo(rx,ry);
				lastlon = lon;
				lastEdgeFlag = edgeFlag;
				deccounter--;
				edgeFlag = FALSE;
				moveFlag = FALSE;
			}								
		} else {
			// max distance on display
			double efconv = 1/sqrt((1/convX)*(1/convX)+(1/convY)*(1/convY));
			dmax = min(asin(2/efconv)*earthRadius, 10001750) ;
			int lat, lon, rx, ry, lastx, lasty;
			double latf, lonf;
			double convertLat = 90.0/0x80000000; 
			double convertLon = 180.0/0x80000000;
			int loop = pDoc->mapTable.GetSize();
			int moveFlag = TRUE, lastEdgeFlag = FALSE, edgeFlag = FALSE;
			int decimate = 1;
			if (mag == 1)
				decimate = 4;
			else if (mag == 2)
				decimate = 3;
			else if (mag == 3)
				decimate = 2;
			int deccounter = decimate;

			for (int m=0; m<loop; m+=2) {
				lat = pDoc->mapTable[m];
				lon = pDoc->mapTable[m+1];
				if (lat == 0) {
					if (lon == 0) {
						moveFlag = TRUE;
						lastEdgeFlag = edgeFlag = FALSE;
						deccounter = decimate;
						continue;
					}
				}
				
				if ((decimate > 1) && (deccounter != decimate)) {
					if(deccounter == 0) 
						deccounter = decimate;
					else
						deccounter--;
					continue;
				} 

				latf = lat * convertLat;
				lonf = lon * convertLon;
				edgeFlag = !LtoC(lonf, latf, rx, ry);

				if (moveFlag && !edgeFlag) {			// previous entry was Move
					pDC->MoveTo(rx,ry);
				} else if (moveFlag && edgeFlag) {		// previous entry was Move, but current point is off edge
					deccounter--;
					lastx = rx;
					lasty = ry;
					lastEdgeFlag = TRUE;
					moveFlag = FALSE;
					continue;
				} else if (edgeFlag && !lastEdgeFlag) { // going over the edge
					pDC->LineTo(rx,ry);
				} else if (!edgeFlag && lastEdgeFlag) { // coming back from edge
					pDC->MoveTo(lastx,lasty);
					pDC->LineTo(rx,ry);
	//			} else if (edgeFlag && lastEdgeFlag) {	// still over the edge (no need to do anything)
	//				;
				} else if (!edgeFlag && !lastEdgeFlag)	// not over the edge
					pDC->LineTo(rx,ry);

				lastx = rx;
				lasty = ry;
				lastEdgeFlag = edgeFlag;
				moveFlag = FALSE;
				deccounter--;
			}								
		}
		pDC->SelectObject(oldPen);
	}
}

void CNavView::Mcalc() { // update viewpoint rotation matrix
	M11 =  cos(-centerLon * degToRad);
	M12 =  0;
	M13 =  sin(-centerLon * degToRad);
	M22 =  cos(-centerLat * degToRad);
	M32 = -sin(-centerLat * degToRad);
	M21 =  M32*M13;
	M23 = -M11*M32;
	M31 = -M22*M13;
	M33 =  M11*M22;
	Mready = TRUE;
}

void CNavView::DrawGrid(CDC *pDC) {
	char buffer[50];
	int rx, ry;
	CPen *oldPen;
	CPen grayPen(PS_SOLID, 0, RGB(192, 192, 192));
	if (pDC->IsPrinting()) { // light gray text does not print well
		oldPen = (CPen *) pDC->SelectStockObject(BLACK_PEN);
		pDC->SetTextColor(RGB(0, 0, 0));
	} else { 
		oldPen = pDC->SelectObject(&grayPen);
		pDC->SetTextColor(RGB(192, 192, 192));
	}
	if (flatWorld) {
		double gridsize;
		if (mag <3)
			gridsize = 30.0;
		else if (mag <5)
			gridsize = 10.0;
		else if (mag == 5)
			gridsize = 5.0;
		else if (mag == 6)
			gridsize = 2.0;
		else if (mag == 7)
			gridsize = 1.0;
		else if (mag > 7)
			gridsize = 1.0 / (2 << (mag-8));
		double start, end;
		int intgrid;
		intgrid = (int) (bottom / gridsize);
		if (bottom < 0.0)
			start = gridsize * (intgrid -1);
		else 
			start = gridsize * (intgrid);
		intgrid = (int) (top / gridsize);
		if (top < 0.0)
			end = gridsize * (intgrid);
		else 
			end = gridsize * (intgrid+1);
		if (start < -90) start = -90.0;
		if (start > 90) start = 90.0;
		if (end < -90) end = -90.0;
		if (end > 90) end = 90.0;
		for (double i= start+gridsize; i<end; i+=gridsize) { // latitude
			LtoC(-180, i, rx, ry);
			if ((ry>0) && (ry<screenHeight)) {
				if(gridsize < 1) {
					int deg = (int) (i);
					int min = (int) ((i-deg) * 60);
					int sec = (int) (((i-deg)*60-min)*60);
					if (i>=0)
						sprintf(buffer, "%d %d' %d\"", abs(deg), abs(min), abs(sec));
					else
						sprintf(buffer, "-%d %d' %d\"", abs(deg), abs(min), abs(sec));
				} else   
					sprintf(buffer, "%.lf", i);
				pDC->TextOut(max(rx+1,1), max(ry+1,1), buffer);
				pDC->MoveTo(max(rx,0), ry);
				LtoC(180, i, rx, ry);
				pDC->LineTo(min(rx, screenWidth), ry);
			}
		}
		intgrid = (int) (left / gridsize);
		if (left < 0.0)
			start = gridsize * (intgrid -1);
		else 
			start = gridsize * (intgrid);
		intgrid = (int) (right / gridsize);
		if (right < 0.0)
			end = gridsize * (intgrid);
		else 
			end = gridsize * (intgrid+1);
		if (start < -180) start = -180.0;
		if (start > 180) start = 180.0;
		if (end < -180) end = -180.0;
		if (end > 180) end = 180.0;
		for (i= start+gridsize; i<end; i+=gridsize) {   // longitude
			LtoC(i,90, rx, ry);
			if ((rx>0) && (rx<screenWidth)) {
				if(gridsize < 1) {
					int deg = (int) (i);
					int min = (int) ((i-deg) * 60);
					int sec = (int) (((i-deg)*60-min)*60);
					if (i>=0)
						sprintf(buffer, "%d %d' %d\"", abs(deg), abs(min), abs(sec));
					else
						sprintf(buffer, "-%d %d' %d\"", abs(deg), abs(min), abs(sec));
				} else   
					sprintf(buffer, "%.lf", i);
				pDC->TextOut(rx+1, max(ry+1,1), buffer);
				pDC->MoveTo(rx, max(ry,0));
				LtoC(i, -90, rx, ry);
				pDC->LineTo(rx, min(ry,screenHeight));
			}
		}
	} else { // !flatWorld
/* don't know a good algorithm for this yet

		double cx = screenWidth  / 2;
		double cy = screenHeight / 2;
		double hx = convX * screenWidth / 2;
		double hy = convY * screenHeight / 2;
		double lx = max(cx-hx, 0); // left and right edges
		double rx = min(cx+hx, screenWidth);
		int xnp, ynp, xsp, ysp, ix, reps=10;
		LtoC(centerLon,  90, xnp, ynp); // north pole
		LtoC(centerLon, -90, xsp, ysp); // south pole
		for (int i=1; i<reps; i++) {
			ix = lx + i*(rx-lx)/reps;
			gcLine(ix, cy, xnp, ynp, pDC);
		}
*/
  }
	pDC->SetTextColor(RGB(0, 0, 0));
	pDC->SelectObject(oldPen);
}

// check for number of IDs this one would overlap
int overlaps(idPlace idP,  CArray<idPlace, idPlace> *idlist) {
	int overlaps = 0;
	idPlace test;
	for (int i = 0; i < idlist->GetSize(); i++) {
		test = idlist->GetAt(i);
		if (idP.sx < test.x) continue;
		if (idP.x > test.sx) continue;
		if (idP.sy < test.y) continue;
		if (idP.y > test.sy) continue;
		overlaps++;
	}
	return overlaps;
}

// find "good" placement for IDs
void CNavView::idPlacement(int centerx, int centery, idPlace &idP, CArray<idPlace, idPlace> *idlist, int ils) {
	int overlap;
	overlap = overlaps(idP, idlist);
	if (overlap == 0) { // no problems
		idlist->Add(idP);
		return;
	}
	// try to find a better place
	idPlace a, best = idP;
	double d = sqrt((double)(idP.x-centerx)*(idP.x-centerx)+(idP.y-centery)*(idP.y-centery));
	double ang = atan2((double)(idP.y-centery), (double)(idP.x-centerx));
	d = max(d, idP.sy-idP.y);
	int dx, dy, o;
	double mul=1.0;
	while (mul<5) {
		if (ils == 0) {
			double tang = -pi/2 + pi/12;
	//		while (tang < ang) {
			while (tang < 0) {
				dx = (int) (mul*d*cos(tang));
				dy = (int) (mul*d*sin(tang));
				a = idP;
				a.x =  centerx + dx;
				a.sx = a.x + (idP.sx-idP.x);
				a.y =  centery + dy;
				a.sy = a.y + (idP.sy-idP.y);
				o = overlaps(a, idlist);
				if (overlap > o) {
					best = a;
					overlap = o;
				}
				if (overlap == 0)
					break;
				tang += pi/24;
			}
		} else { // special ils placement
			double tang = pi/12;
			while (tang <= pi/2) {
				dx = (int) (d*cos(tang+ang));
				dy = (int) (d*sin(tang+ang));
				a = idP;
				a.x =  centerx + dx;
				a.sx = a.x + (idP.sx-idP.x);
				a.y =  centery + dy;
				a.sy = a.y + (idP.sy-idP.y);
				o = overlaps(a, idlist);
				if (overlap > o) {
					best = a;
					overlap = o;
				}
				if (overlap == 0)
					break;
				tang = -tang;
				dx = (int) (d*cos(tang+ang));
				dy = (int) (d*sin(tang+ang));
				a = idP;
				a.x =  centerx + dx;
				a.sx = centerx + dx + (idP.sx-idP.x);
				a.y =  centery + dy;
				a.sy = centery + dy + (idP.sy-idP.y);
				o = overlaps(a, idlist);
				if (overlap > o) {
					best = a;
					overlap = o;
				}
				if (overlap == 0)
					break;
				tang = -tang;
				tang += pi/24;
			}
		}
		if (overlap == 0)
			break;
		mul *= 1.15;
	}
	idP = best;
	idlist->Add(idP);
}

void CNavView::OnInitialUpdate() {
	CView::OnInitialUpdate();
	DragAcceptFiles(TRUE); // register window for drag and drop support
}

void CNavView::OnDropFiles(HDROP hDropInfo) {
	char filename[_MAX_PATH];
	UINT nFiles = ::DragQueryFile(hDropInfo, (UINT) -1, NULL, 0);
	if (nFiles > 0) {
		CNavDoc* pDoc = GetDocument();
		FILE *fp;
		CString listname = orgDir + "\\" + "Dropped.lst";
		fp = fopen((LPCTSTR) listname, "w");
		for (UINT iFile = 0; iFile<nFiles; iFile++) {
			::DragQueryFile(hDropInfo, iFile, filename, _MAX_PATH);
			fprintf(fp, "%s\n", filename);	
		}
		fclose(fp);
		pDoc->OnFileLoad("Dropped.lst");
		DeleteFile((LPCTSTR) listname);
	}
	::DragFinish(hDropInfo);
}

void CNavView::OnFs6Link() {
	if (link) {
		linked = FALSE;
		link = FALSE;
	} else {
		hthread = AfxBeginThread(LinkThread, this);	// start link thread
		link = TRUE;
	}
	char buffer[30];
	sprintf(buffer, "%d", link);
	WritePrivateProfileString("View", "Link", buffer, "nav.ini");
}

void CNavView::OnUpdateFs6Link(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(link);	
}

void CNavView::OnFs6Set() {
	FSInfo* pFSInfo;	  // set this pointer to either FS95 or FS98 state offsets 
	HWND hWndFs6;
	if (hWndFs6 = FindWindowEx(NULL, NULL, fs95Info.pszWindowClass, NULL))
		pFSInfo = &fs95Info;
	else if (hWndFs6 = FindWindowEx(NULL, NULL, fs98Info.pszWindowClass, NULL))
		pFSInfo = &fs98Info;
	if (hWndFs6 == NULL) {		
		AfxMessageBox("Can't find FlightSim. Is it running?");
		return;
	}

	double lat, lon, fract;
	if (flatWorld) {
		lat = (top + bottom)/2.0;
		lon = (right + left)/2.0;
	} else {
		lat = centerLat;
		lon = centerLon;
	}
	lat /= 45.0 / 5000875.0;
	lon /= 90.0 / (256.0 * 4194304.0);
	fract = (int)lat - lat;
	flightData.dwLatLo = (int) (fract * 0x100000000);
	if (lat >= 0)
		flightData.dwLatHi = (int) (lat - fract);
	else 
		flightData.dwLatHi = (int) (lat + fract);
	fract = (int)lon - lon;
	flightData.dwLonLo = (int) (fract * 0x100000000);
	if (lat >= 0)
		flightData.dwLonHi = (int) (lon - fract);
	else 
		flightData.dwLonHi = (int) (lon + fract);

	EnterCriticalSection(&crit);
	FS6IPC_Client ipcClient;	
	ipcClient.Start(hWndFs6);
	ipcClient.BeginRequest();
	ipcClient.WriteStateData(pFSInfo->offset_dwLatLo, 4 * sizeof(DWORD), &flightData.dwLatLo);
	ipcClient.SendRequest();
	ipcClient.Stop();
	LeaveCriticalSection(&crit);
}

void CNavView::OnFs6Fs6linkoptions() {
	LinkOptions d;

	d.m_update = linkUpdateTime;
	d.m_flyupdate = linkFlyUpdateTime;
	d.m_flysetmode = linkFlySetMode;
	d.m_nbMilesToTurnBefore = linkFlyNbMilesToTurnBefore;
	if (d.DoModal() == IDOK) {
		linkUpdateTime = d.m_update;
		linkFlyUpdateTime = d.m_flyupdate;
		linkFlySetMode = d.m_flysetmode;
		linkFlyNbMilesToTurnBefore = d.m_nbMilesToTurnBefore;
		char buffer[30];
		sprintf(buffer, "%d", linkUpdateTime);
		WritePrivateProfileString("View", "linkUpdateTime", buffer, "nav.ini");
		sprintf(buffer, "%d", linkFlyUpdateTime);
		WritePrivateProfileString("View", "linkFlyUpdateTime", buffer, "nav.ini");
		sprintf(buffer, "%d", linkFlySetMode);
		WritePrivateProfileString("View", "linkFlySetMode", buffer, "nav.ini");
		sprintf(buffer, "%.2f", linkFlyNbMilesToTurnBefore);
		WritePrivateProfileString("View", "linkFlyNbMilesToTurnBefore", buffer, "nav.ini");
		SetEvent(hflyevent);
	}	
}

void CNavView::DrawLink(int undrawOld) {
	int xc, yc;
	if (!LtoC(linkLon, linkLat, xc, yc)) {
//		CenterAt(linkLat, linkLon);
		centerLat = linkLat;
		centerLon = linkLon;
		double xd = (right + left)/2.0 - linkLon;
		double yd = (top + bottom)/2.0 - linkLat;
		left -= xd;
		right -= xd;
		top -= yd;
		bottom -= yd;
		Mready = FALSE;
		abort = TRUE; 
		Invalidate(TRUE);
		return;
	}
	CDC *pDC;
	pDC = GetDC();
	CFont *oldFont;
	CFont font;
	LOGFONT lfnew = lf;
	font.CreateFontIndirect(&lfnew);
	oldFont = (CFont *) pDC->SelectObject(&font);
	int oldROP;
	oldROP = pDC->SetROP2(R2_NOTXORPEN);

	if (undrawOld) 
		DrawLinkHelper(pDC, linkHeadingOld, linkAltOld, linkTASOld, linkLatOld, linkLonOld);

	if (linked)
		DrawLinkHelper(pDC, linkHeading, linkAlt, linkTAS, linkLat, linkLon);

	pDC->SetROP2(oldROP);
	pDC->SelectObject(oldFont);
	//++ JYL
	ReleaseDC(pDC);
	//-- JYL
	linkHeadingOld = linkHeading;
	linkAltOld = linkAlt;
	linkTASOld = linkTAS;
	linkLatOld = linkLat;
	linkLonOld = linkLon;
}

void CNavView::DrawLinkHelper(CDC *pDC, double Heading, double Alt, double TAS, double Lat, double Lon) {
	int cx, cy;
	LtoC(Lon, Lat, cx, cy);
	double sa = sin(Heading*degToRad);
	double ca = cos(Heading*degToRad);
	double ux, uy, lx, ly, ldist = 1.0;
	lx = -100*(-sa);
	ly = -100*ca;
 	if(flatWorld)
		lx = (int) (lx * ldist);
	if(flatWorld) { // calculate latitude distortion due to flat map
		ldist = 1.0/cos((top + bottom) *.5 * degToRad);
		lx = (int) (lx * ldist);
	}
	CPen *oldPen;
	oldPen = (CPen *) pDC->SelectStockObject(BLACK_PEN);	
	pDC->MoveTo(cx, cy);
	pDC->LineTo(cx+(int) lx, cy+(int) ly);
	char buf[200];
	int chars;
	double mag = Heading + linkMag + .5;
	if (mag > 360) mag -= 360;
	if (mag < 0) mag += 360;
	if (!flying)
		chars = sprintf(buf, "%6.0f deg (%.0f mag), %6.0f ft, %4.0f kts", Heading, mag, Alt, TAS);
	else {
		double dist = gcDist(Lat, Lon, flyingLat, flyingLon)/1852;
		double hours = dist / TAS;
		double mins = 60 * (hours - floor(hours));
		hours = floor(hours);
		chars = sprintf(buf, "FlyTo: %.1f nm,  ETA: %0.lf:%04.1lf, %.0f deg (%.0f mag), %.0f ft, %.0f kts", dist, hours, mins, Heading, mag, Alt, TAS);
	}
	CSize tSize = pDC->GetOutputTextExtent(buf, chars);
	CRect client;
	GetClientRect(client);
	pDC->FillSolidRect(CRect(client.right-xLinkExtent, client.bottom-yLinkExtent, client.right, client.bottom), RGB(255,255,255));
	xLinkExtent = tSize.cx;
	yLinkExtent = tSize.cy;
	pDC->TextOut(client.right-tSize.cx, client.bottom-tSize.cy, buf);
	pDC->SelectObject(oldPen);
	LOGBRUSH lb;
	lb.lbStyle = BS_SOLID;
	lb.lbColor = RGB(0, 0, 0);
	CPen widePen(PS_GEOMETRIC | PS_ENDCAP_ROUND | PS_JOIN_BEVEL, 3, &lb);					
	oldPen = (CPen *) pDC->SelectObject(&widePen);	
	if(flatWorld) { // correct rotation angle for latitude
		double ap = atan2(lx, -1*ly);
		sa = sin(ap);
		ca = cos(ap);
	}
	ux = -7*(-sa);
	uy = -7*ca;
	lx = 12*(-sa);
	ly = 12*ca;
	pDC->MoveTo(cx+(int) ux, cy+(int) uy);
	pDC->LineTo(cx+(int) lx, cy+(int) ly);
	ux = -15*ca ;
	uy = -15*sa;
	lx = 15*ca;
	ly = 15*sa;
	pDC->MoveTo(cx+(int) ux, cy+(int) uy);
	pDC->LineTo(cx+(int) lx, cy+(int) ly);
	ux = -3*ca + 10*(-sa);
	uy = -3*sa + 10*ca;
	lx = 3*ca + 10*(-sa);
	ly = 3*sa + 10*ca;
	pDC->MoveTo(cx+(int) ux, cy+(int) uy);
	pDC->LineTo(cx+(int) lx, cy+(int) ly);
 	pDC->SelectObject(oldPen);
}

void CNavView::OnFs6Flyto() {
	if (flying) {
		flytoPlan = FALSE;
		flying = FALSE;
		SetEvent(hflyevent);
		Invalidate(TRUE);
		return;
	}
	flying = TRUE;
	if (!flatWorld) {
		flyingLat = centerLat;
		flyingLon = centerLon;
	} else {
		flyingLon = (right + left)/2.0;
		flyingLat = (top + bottom)/2.0;
	}
	StartFlyThread();
	Invalidate(TRUE);
}

void CNavView::OnUpdateFs6Flyto(CCmdUI* pCmdUI) {
	pCmdUI->SetCheck(flying);	
}

void CNavView::OnHelpviewdocs() {
	STARTUPINFO si;
	GetStartupInfo(&si);
	PROCESS_INFORMATION pinf;
	CreateProcess(NULL, "write Nav.wri", NULL, NULL, FALSE, NORMAL_PRIORITY_CLASS, NULL, NULL, &si, &pinf);
}

void CNavView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) {
	CRect clientRect;								
	GetClientRect(clientRect);
	CPoint point(clientRect.right/2, clientRect.bottom/2);
	int scrollPixels = 20;
	if ((nChar == 189) || (nChar == VK_SUBTRACT)) {		// zoom out
		PostMessage(WM_COMMAND, ID_VIEW_ZOOMOUT, 0);
	} else if ((nChar == 187) || (nChar == VK_ADD)) {	// zoom in
		PostMessage(WM_COMMAND, ID_VIEW_ZOOMIN, 0);		
	} else if ((nChar == 'n') || (nChar == 'N')) {		// find nearest (same as double right click)
		OnRButtonDblClk(0, point);
	} else 	if (nChar == VK_RIGHT) {					// scroll right
		point.x += scrollPixels;
		OnLButtonDown(0, point);
	} else if (nChar == VK_LEFT) {						// scroll left		
		point.x -= scrollPixels;
		OnLButtonDown(0, point);
	} else if (nChar == VK_UP) {						// scroll up		
		point.y -= scrollPixels;
		OnLButtonDown(0, point);
	} else if (nChar == VK_DOWN) {						// scroll down		
		point.y += scrollPixels;
		OnLButtonDown(0, point);
	}
	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CNavView::OnSetFocus(CWnd* pOldWnd) {
	CView::OnSetFocus(pOldWnd);
	
	if (linked)
		Invalidate(TRUE);	// re-draw whole map to avoid double images of plane
}

void CNavView::ConvertDistHeadToCoord(double distance,double heading, 
								double& latitude, double& longitude)
{
	double deltaLat;
	distance = distance / 1852.0;
	deltaLat = distance * sin((90-heading)* degToRad);

	longitude = longitude + ((distance * cos((90-heading)* degToRad))/(cos((latitude + deltaLat/120)*degToRad)))/60;
	latitude = latitude + (deltaLat/60);

}

void CNavView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	TRACE("\n CNavView::OnLButtonUp");	
	CView::OnLButtonUp(nFlags, point);
}


BOOL CNavView::SendToPlan(int index)
{
	if (plancount >= MAXPLAN-2) {
		AfxMessageBox("Too many nav-aids in plan");
		return FALSE;
	}
	plan[plancount] = index;
	plancount += 1;
	Invalidate(TRUE);
	return TRUE;

}


void CNavView::StartFlyThread()
{
	// start fly thread
	if (hflythread == 0)
		hflythread = AfxBeginThread(FlyThread, this);

}

void CNavView::OnViewWaypt() 
{
	wpt = !wpt;	
	abort = TRUE; // abort old redraw if necessary
	Invalidate(TRUE);
	
}

void CNavView::OnUpdateViewWaypt(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(wpt);	
	
}
