// datadial.cpp : implementation file
//
/*
    Copyright (C) 1997 Ted Wright 
    NASA Business: wright@lerc.nasa.gov, Remainder: wright@en.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "Nav.h"
#include <afxtempl.h>
#include "mainfrm.h"
#include "navdoc.h"
#include "flightpl.h"
#include "navview.h"
#include "datadial.h"
#include "math.h"
#include "details.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// DataDialog dialog


DataDialog::DataDialog(CWnd* pParent /*=NULL*/)
	: CDialog(DataDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(DataDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void DataDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(DataDialog)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(DataDialog, CDialog)
	//{{AFX_MSG_MAP(DataDialog)
	ON_BN_CLICKED(IDC_CENTER, OnCenter)
	ON_BN_CLICKED(IDC_DETAILS, OnDetails)
	ON_LBN_DBLCLK(IDC_LIST, OnDblclkList)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// DataDialog message handlers
void DataDialog::OnCenter() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	CNavDoc *doc = (CNavDoc *) main->GetActiveDocument();
	lb = (CListBox *) GetDlgItem(IDC_LIST);
	int lbindex = lb->GetCurSel();
	if (lbindex == LB_ERR)
		return;
	int dbindex = lb->GetItemData(lbindex);

	double lat = doc->db[dbindex].lat;
	double lon = doc->db[dbindex].lon;
	double cx = (view->left + view->right)/2;
	double cy = (view->top + view->bottom)/2;
	view->right += lon - cx;
	view->left += lon - cx;
	view->top += lat - cy;
	view->bottom += lat - cy;
	view->Invalidate(TRUE);
}

void DataDialog::OnDblclkList() {
	OnDetails();	
}

void DataDialog::OnDetails() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	CNavDoc *doc = (CNavDoc *) main->GetActiveDocument();
	lb = (CListBox *) GetDlgItem(IDC_LIST);
	int lbindex = lb->GetCurSel();
	if (lbindex == LB_ERR)
		return;
	int dbindex = lb->GetItemData(lbindex);

	CString details = "";
	doc->formatData(details, dbindex);
	Details d;
	d.index = dbindex;
	d.m_text = details;
	d.DoModal();
}

BOOL DataDialog::OnInitDialog() {
	CDialog::OnInitDialog();
	
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	CNavDoc *doc = (CNavDoc *) main->GetActiveDocument();
	lb = (CListBox *) GetDlgItem(IDC_LIST);

	lb->ResetContent();
	lb->SetTabStops();

	if (view->search == "") {	// we are not searching
		for (int i=0; i<doc->dbCount; i++) {
			if (view->planOnly) {
				int proceed = FALSE;
				for (int q=0; q<view->plancount; q++)
					if (i == view->plan[q]) {
						proceed = TRUE;
						break;
				}
				if (!proceed)
					goto next;
			}
			// limit to window being viewed
	 		if ((doc->db[i].lon) < view->left) goto next;
			if ((doc->db[i].lon) > view->right) goto next;
			if ((doc->db[i].lat) < view->bottom) goto next;
			if ((doc->db[i].lat) > view->top) goto next;
			if ((doc->db[i].type == 1) && (!view->ndb)) goto next;
			if ((doc->db[i].type == 2) && (!view->ils)) goto next;
			if ((doc->db[i].type == 3) && (!view->vor)) goto next;
			if ((doc->db[i].type == 4) && (!view->airports)) goto next;
			if ((doc->db[i].type == 5) && (!view->markers)) goto next;
			if ((doc->db[i].type == 6) && (!view->markers)) goto next;
			if ((doc->db[i].type == 7) && (!view->markers)) goto next;
			if ((doc->db[i].type == 8) && (!view->airports)) goto next;
			add(doc->db[i], i);
			next:;
		}
	} else { 					// we are searching
		char s[100], t[6], u[100];;
		strcpy(s, view->search);
		_strlwr(s);
		for (int i=0; i<doc->dbCount; i++) {
			if (doc->db[i].type == 9)
				strncpy(t, "gps  ", 5);
			else
				strncpy(t, doc->db[i].id, 5);
			t[5] = 0x00;
			_strlwr(t);
			strncpy(u, doc->db[i].name, 25);
			_strlwr(u);
			if ((NULL != strstr(t, s)) || (NULL != strstr(u, s)))
				add(doc->db[i], i);
		}
	}
		
	return TRUE;  // return TRUE unless you set the focus to a control
}

void DataDialog::add(dbRec dbr, int index) {
	char line[100];
	if (1 == dbr.type) {
		sprintf(line, "NDB  %s\t(%5.1f)\t%s", dbr.id, dbr.freq, dbr.name);
		lb->SetItemData(lb->AddString(line), index);
	} else if (2 == dbr.type) {
		sprintf(line, "ILS  %s\t(%6.2f)\t%s", dbr.id, dbr.freq, dbr.name);
		lb->SetItemData(lb->AddString(line), index);
	} else if (3 == dbr.type) {
		sprintf(line, "VOR  %s\t(%6.2f)\t%s", dbr.id, dbr.freq, dbr.name);
		lb->SetItemData(lb->AddString(line), index);
	} else if (4 == dbr.type) {
		sprintf(line, "Airport  %s", dbr.name);
		lb->SetItemData(lb->AddString(line), index);
	} else if (5 == dbr.type) {
		sprintf(line, "Inner marker");
		lb->SetItemData(lb->AddString(line), index);
	} else if (6 == dbr.type) {
		sprintf(line, "Middle marker");
		lb->SetItemData(lb->AddString(line), index);
	} else if (7 == dbr.type) {
		sprintf(line, "Outer marker");
		lb->SetItemData(lb->AddString(line), index);
	} else if (8 == dbr.type) {
		sprintf(line, "Runway  %s", dbr.id);
		lb->SetItemData(lb->AddString(line), index);
	} else if (9 == dbr.type) {
		sprintf(line, "GPS fix %s\t\t%s", dbr.id, dbr.name);
		lb->SetItemData(lb->AddString(line), index);
	} else if (10 == dbr.type) {
		sprintf(line, "ATIS station %s", dbr.name);
		lb->SetItemData(lb->AddString(line), index);
	}
	else if (11 == dbr.type) {
		sprintf(line, "Way Point %s\t\t%s", dbr.id, dbr.name);
		lb->SetItemData(lb->AddString(line), index);
	}

}

