// flightpl.cpp : implementation file
//
/*
    Copyright (C) 1997 Ted Wright 
    NASA Business: wright@lerc.nasa.gov, Remainder: wright@en.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "Nav.h"
#include <afxtempl.h>
#include "mainfrm.h"
#include "navdoc.h"
#include "flightpl.h"
#include "navview.h"
#include "string.h"
#include "datadial.h"
#include "search.h"
#include "details.h"
#include "Aircraft.h"
#include <math.h>
#include "Wind.h"
#include <afxdlgs.h>


#define pi			 3.1415946536
#define degToRad     (pi / 180.0)
#define CLIMB 1
#define CRUISE 2
#define DESCEND 3

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// FlightPlan dialog


FlightPlan::FlightPlan(CWnd* pParent /*=NULL*/)
	: CDialog(FlightPlan::IDD, pParent)
{
	plantable.SetSize(0,20);
	excessWind = FALSE;
	//{{AFX_DATA_INIT(FlightPlan)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_hIcon = AfxGetApp()->LoadIcon(IDR_NAVPLAN);

	flyToSelected = -1;
}

FlightPlan::~FlightPlan() {
	plantable.RemoveAll();
	plantable.FreeExtra();
}

void FlightPlan::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(FlightPlan)
	DDX_Control(pDX, IDC_LIST, m_list);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(FlightPlan, CDialog)
	//{{AFX_MSG_MAP(FlightPlan)
	ON_BN_CLICKED(IDC_ADD, OnAdd)
	ON_BN_CLICKED(IDC_CLEAR, OnClear)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_UP, OnUp)
	ON_BN_CLICKED(IDC_DOWN, OnDown)
	ON_BN_CLICKED(IDC_DELETE, OnDelete)
	ON_NOTIFY(NM_CLICK, IDC_LIST, OnClickList)
	ON_BN_CLICKED(IDC_AUTO, OnAuto)
	ON_BN_CLICKED(IDC_OPTIONS, OnOptions)
	ON_BN_CLICKED(IDC_PRINT, OnPrint)
	ON_NOTIFY(LVN_BEGINLABELEDIT, IDC_LIST, OnBeginlabeleditList)
	ON_NOTIFY(NM_DBLCLK, IDC_LIST, OnDblclkList)
	ON_BN_CLICKED(IDC_AIRCRAFT, OnAircraft)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_LIST, OnItemchangedList)
	ON_WM_PAINT()
	ON_BN_CLICKED(IDC_WIND, OnWind)
	ON_COMMAND(ID_FILE_PRINT, OnFilePrint)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT, OnUpdateFilePrint)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_FILE_EXPORT, OnFileExport)
	ON_COMMAND(ID_AIRCRAFT, OnAircraft)
	ON_COMMAND(ID_OPTIONS, OnOptions)
	ON_BN_CLICKED(IDC_FLYTO, OnFlyto)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CreateDefaultIniFile() { // should execute only once to add default planes
	int init;
	init = GetPrivateProfileInt("Plan", "initialized", FALSE, "nav.ini");
	if (!init) {
		WritePrivateProfileString("Plan", "initialized", "1", "nav.ini");
		WritePrivateProfileString("Plan", "lastCraft", "Cessna Skylane RG", "nav.ini");

		WritePrivateProfileString("Plan", "craftnumber", "3", "nav.ini");
		WritePrivateProfileString("Plan", "craft1", "Cessna Skylane RG", "nav.ini");
		WritePrivateProfileString("Plan", "craft2", "Learjet 35A", "nav.ini");
		WritePrivateProfileString("Plan", "craft3", "Boeing 737", "nav.ini");

		WritePrivateProfileString("Cessna Skylane RG", "cruisealtitude", "9000", "nav.ini");
		WritePrivateProfileString("Cessna Skylane RG", "climbfuel", "20", "nav.ini");
		WritePrivateProfileString("Cessna Skylane RG", "cruisefuel", "14", "nav.ini");
		WritePrivateProfileString("Cessna Skylane RG", "descentfuel", "10", "nav.ini");
		WritePrivateProfileString("Cessna Skylane RG", "climbspeed", "76", "nav.ini");
		WritePrivateProfileString("Cessna Skylane RG", "cruisespeed", "140", "nav.ini");
		WritePrivateProfileString("Cessna Skylane RG", "descentspeed", "120", "nav.ini");
		WritePrivateProfileString("Cessna Skylane RG", "climbrate", "900", "nav.ini");
		WritePrivateProfileString("Cessna Skylane RG", "descentrate", "800", "nav.ini");

		WritePrivateProfileString("Learjet 35A", "cruisealtitude", "32000", "nav.ini");
		WritePrivateProfileString("Learjet 35A", "climbfuel", "1000", "nav.ini");
		WritePrivateProfileString("Learjet 35A", "cruisefuel", "600", "nav.ini");
		WritePrivateProfileString("Learjet 35A", "descentfuel", "400", "nav.ini");
		WritePrivateProfileString("Learjet 35A", "climbspeed", "320", "nav.ini");
		WritePrivateProfileString("Learjet 35A", "cruisespeed", "420", "nav.ini");
		WritePrivateProfileString("Learjet 35A", "descentspeed", "420", "nav.ini");
		WritePrivateProfileString("Learjet 35A", "climbrate", "2200", "nav.ini");
		WritePrivateProfileString("Learjet 35A", "descentrate", "2200", "nav.ini");

		WritePrivateProfileString("Boeing 737", "cruisealtitude", "35000", "nav.ini");
		WritePrivateProfileString("Boeing 737", "climbfuel", "10000", "nav.ini");
		WritePrivateProfileString("Boeing 737", "cruisefuel", "5000", "nav.ini");
		WritePrivateProfileString("Boeing 737", "descentfuel", "3000", "nav.ini");
		WritePrivateProfileString("Boeing 737", "climbspeed", "300", "nav.ini");
		WritePrivateProfileString("Boeing 737", "cruisespeed", "480", "nav.ini");
		WritePrivateProfileString("Boeing 737", "descentspeed", "300", "nav.ini");
		WritePrivateProfileString("Boeing 737", "climbrate", "2500", "nav.ini");
		WritePrivateProfileString("Boeing 737", "descentrate", "2500", "nav.ini");
	}
}

/////////////////////////////////////////////////////////////////////////////
// FlightPlan message handlers

BOOL FlightPlan::OnInitDialog() {
	CDialog::OnInitDialog();
	long style = GetWindowLong(m_hWnd, GWL_STYLE);
	style |= LVS_SHOWSELALWAYS;
	SetWindowLong(m_hWnd, GWL_STYLE, style);
	style = GetWindowLong(m_hWnd, GWL_EXSTYLE);
	style |= WS_EX_APPWINDOW;
	SetWindowLong(m_hWnd, GWL_EXSTYLE, style);
	 
	// Set the icon for this dialog. 
	SetIcon(m_hIcon, TRUE);			// Set big icon

	sel = -1;
	LV_COLUMN c;
	c.mask = LVCF_TEXT | LVCF_WIDTH;
	c.pszText = "Location";
	c.cx = m_list.GetStringWidth("XXXX_XXXX_XXXX_XXXX_");
	m_list.InsertColumn(0, &c);
	c.pszText = "ID";
	c.cx = m_list.GetStringWidth("XXXXXXX");
	m_list.InsertColumn(1, &c);
	c.pszText = "Type";
	c.cx = 50;
	m_list.InsertColumn(2, &c);
	c.pszText = "Freq";
	c.cx = m_list.GetStringWidth("XXXXXXX");
	m_list.InsertColumn(3, &c);
	c.pszText = "Head";
	c.cx = m_list.GetStringWidth("XXXXXXXX");
	m_list.InsertColumn(4, &c);
	c.pszText = "Dist";
	c.cx = m_list.GetStringWidth("XXXXXXXX");
	m_list.InsertColumn(5, &c);
	c.pszText = "Total";
	c.cx = m_list.GetStringWidth("XXXXXXXX");
	m_list.InsertColumn(6, &c);
	c.pszText = "ETE";
	c.cx = m_list.GetStringWidth("XXXXXXXX");
	m_list.InsertColumn(7, &c);
	c.pszText = "Total";
	c.cx = m_list.GetStringWidth("XXXXXXXX");
	m_list.InsertColumn(8, &c);
	c.pszText = "Fuel";
	c.cx = m_list.GetStringWidth("XXXXXXXX");
	m_list.InsertColumn(9, &c);
	c.pszText = "Total";
	c.cx = m_list.GetStringWidth("XXXXXXXX");
	m_list.InsertColumn(10, &c);
	c.pszText = "Alt";
	c.cx = m_list.GetStringWidth("XXXXXXXX");
	m_list.InsertColumn(11, &c);
	c.pszText = "Wind Vel";
	c.cx = m_list.GetStringWidth("XXXXXXXX");
	m_list.InsertColumn(12, &c);
	c.pszText = "Wind Dir";
	c.cx = m_list.GetStringWidth("XXXXXXXX");
	m_list.InsertColumn(13, &c);

	m_list.SetItemState(0, LVNI_SELECTED, LVNI_SELECTED);
	m_list.SetItemState(0, LVIS_DROPHILITED, LVIS_DROPHILITED);
	m_list.SetItemState(0, 0, LVIS_FOCUSED);
	m_list.Update(0);
	m_list.EnsureVisible(0, FALSE);

	CreateDefaultIniFile();
	char buf[20];  // read default craft characteristics
	GetPrivateProfileString("Plan", "lastCraft", "Cessna Skylane RG", craftBuffer, 50, "nav.ini");
	GetPrivateProfileString(craftBuffer, "cruisealtitude", "9000", buf, 20, "nav.ini");
	sscanf(buf, "%ld", &cruisealtitude);
	GetPrivateProfileString(craftBuffer, "climbfuel", "14.1", buf, 20, "nav.ini");
	sscanf(buf, "%lf", &climbfuel);
	GetPrivateProfileString(craftBuffer, "cruisefuel", "14.1", buf, 20, "nav.ini");
	sscanf(buf, "%lf", &cruisefuel);
	GetPrivateProfileString(craftBuffer, "descentfuel", "14.1", buf, 20, "nav.ini");
	sscanf(buf, "%lf", &descentfuel);
	GetPrivateProfileString(craftBuffer, "climbspeed", "76", buf, 20, "nav.ini");
	sscanf(buf, "%d", &climbspeed);
	GetPrivateProfileString(craftBuffer, "cruisespeed", "140", buf, 20, "nav.ini");
	sscanf(buf, "%d", &cruisespeed);
	GetPrivateProfileString(craftBuffer, "descentspeed", "140", buf, 20, "nav.ini");
	sscanf(buf, "%d", &descentspeed);
	GetPrivateProfileString(craftBuffer, "climbrate", "900", buf, 20, "nav.ini");
	sscanf(buf, "%d", &climbrate);
	GetPrivateProfileString(craftBuffer, "descentrate", "1000", buf, 20, "nav.ini");
	sscanf(buf, "%d", &descentrate);

	update();
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void FlightPlan::OnAdd() {
	DataDialog d;	
	Search d2;
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();

	if (d2.DoModal() == IDOK) {
		view->search = d2.m_target;
		if (view->search != "")	{
			int c = view->plancount;
			d.DoModal();
			if (c != view->plancount) {
				view->abort = TRUE;
				view->Invalidate(TRUE);
			}
		}
	} 
	view->search = "";
}

// average two wind vectors (in magnitude and angle format)
void FlightPlan::averageWind(double m1, double a1, double m2, double a2, double &m3, double &a3, double percent) {
	ASSERT(percent >= 0.0);
	ASSERT(percent <= 1.0);
	int zero = ((m1 == 0) && (m2 == 0));
	if (zero) // special case to average angle correctly when magnitude is zero
		m1 = m2 = 1;
	double wx1 = m1 * sin(a1*degToRad);
	double wy1 = m1 * cos(a1*degToRad);
	double wx2 = m2 * sin(a2*degToRad);
	double wy2 = m2 * cos(a2*degToRad);
	double wx = (1-percent)*wx1 + percent*wx2;
	double wy = (1-percent)*wy1 + percent*wy2;
	m3 = sqrt(wx*wx+wy*wy);
	a3 = atan2(wx, wy) / degToRad;
	if (a3 < 0)
		a3 += 360;
	if (zero)
		m3 = 0;
}

// take wind into account for indicated heading and effective speed
int FlightPlan::wind(double &ihead, double ispeed, double ehead, double &espeed, double whead, double wspeed) {
	double wx = wspeed * sin(whead*degToRad);
	double wy = wspeed * cos(whead*degToRad);
	double st = sin(ehead*degToRad);
	double ct = cos(ehead*degToRad);
	double b = 2*(st*wx+ct*wy);
	double check = b*b-4*(wx*wx+wy*wy-ispeed*ispeed);
	if ((check < 0) || ((b<0) && (b*b > check))) {
		// clean up plan here
		if (!excessWind)
			AfxMessageBox("Wind exceeds airplane's capabilities");
		excessWind = TRUE;
		for (int p=1; p<plantable.GetSize(); p++) { 
			plantable[p].ete = 0;
			plantable[p].Tete  = 0;
			plantable[p].fuel = 0;
			plantable[p].Tfuel = 0;
		}
		return FALSE;
	}
	espeed = (b+sqrt(check))/2.0;
	ihead = atan2(espeed*st-wx, espeed*ct-wy) / degToRad;
	if (ihead<0)
		ihead += 360;
	return TRUE;
}

int FlightPlan::adjust(int i, int forward, int mode) {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	CNavDoc *doc = (CNavDoc *) main->GetActiveDocument();
	double wspeed, whead, speed, rate, fuel, percent, talt;
	planRec toc;
	if ((mode == CLIMB) && (itoc != -1) && (itoc < i))
		mode = CRUISE;
	if ((mode == DESCEND) && (ibod != -1) && (ibod >= i))
		mode = CRUISE;
	if (mode == CLIMB) {
		speed = climbspeed;
		rate = climbrate;
		fuel = climbfuel;
	} else if (mode == CRUISE) {
		speed = cruisespeed;
		fuel = cruisefuel;
	} else if (mode == DESCEND) {
		speed = descentspeed;
		rate = descentrate;
		fuel = descentfuel;
	}
	averageWind(plantable[i-1].windVel, plantable[i-1].windheading, plantable[i].windVel, plantable[i].windheading, wspeed, whead);
	if (FALSE == wind(plantable[i].correctedheading, speed, plantable[i].heading, plantable[i].correctedvelocity, whead, wspeed))
		return FALSE; 
	if (mode == CRUISE)	{
		if (((i != plantable.GetSize()-1) || !D) && ((i != 0) || !C)) {
			plantable[i].alt = cruisealtitude;
		}
	} else if (forward) {
		talt = plantable[i-1].alt + (plantable[i].dist / plantable[i].correctedvelocity) * 60 * rate;
//		if (D && (i == plantable.GetSize()-1)) {
//			; // do nothing for now
//		} else 
		if ((talt > cruisealtitude) && (itoc == -1)) { 
			// need to insert toc
			if (i != plantable.GetSize()-1)
				plantable[i].alt = cruisealtitude;
			toc.index = -1; //special value for toc
			percent = (cruisealtitude - plantable[i-1].alt) / (talt - plantable[i-1].alt);
			view->gcDistInterpolate(plantable[i-1].lat, plantable[i-1].lon, plantable[i].lat, plantable[i].lon, percent, toc.lat, toc.lon);
			toc.alt = cruisealtitude;
			averageWind(plantable[i-1].windVel, plantable[i-1].windheading, plantable[i].windVel, plantable[i].windheading, toc.windVel, toc.windheading, percent);
 			toc.dist = view->gcDist(plantable[i-1].lat, plantable[i-1].lon, toc.lat, toc.lon)/1852; // in nm
			toc.heading = view->bearing + doc->GetMagVar(toc.lat, toc.lon);
 			plantable[i].dist = view->gcDist(toc.lat, toc.lon, plantable[i].lat, plantable[i].lon)/1852; // in nm
			plantable[i].heading = view->bearing + doc->GetMagVar(plantable[i].lat, plantable[i].lon);
			itoc = i;
			if ((ibod != -1) && (ibod > i))
				ibod++;
			plantable.InsertAt(i, toc); // insert toc record, shift others 
			if (FALSE == adjust(i, forward, CLIMB)) return FALSE;
			if (FALSE == adjust(i+1, forward, CRUISE)) return FALSE;
			return TRUE;
		} else if ((i != plantable.GetSize()-1) || (C && !D)) {
			plantable[i].alt = talt;
			if (plantable[i].alt > cruisealtitude) {
				plantable[i].alt = cruisealtitude;
			}
		}
	} else {
		talt = plantable[i].alt + (plantable[i].dist / plantable[i].correctedvelocity) * 60 * rate;
		if (C && D && (talt > plantable[i-1].alt) && (plantable[i-1].alt < .99*cruisealtitude) && !((itoc==ibod)&&(itoc!=-1))) {
			//delete old toc & recalculate
			if (itoc != -1) {
				plantable.RemoveAt(itoc);
				if (i > itoc) 
					i -= 1;
				int del = itoc;
				itoc = ibod = -2; // fixup spot where toc used to be
 				plantable[del].dist = view->gcDist(plantable[del-1].lat, plantable[del-1].lon, plantable[del].lat, plantable[del].lon)/1852; // in nm
				plantable[del].heading = view->bearing + doc->GetMagVar(plantable[del].lat, plantable[del].lon);
				if (FALSE == adjust(del, FALSE, DESCEND)) return FALSE;
			}
			// need to insert toc/bod
			toc.index = -3; // special value for toc/bod
			double cspeed, dspeed; // take wind into account for speed calculations
			if (FALSE == wind(plantable[i].correctedheading, climbspeed, plantable[i].heading, cspeed, whead, wspeed))
				return FALSE; 
			if (FALSE == wind(plantable[i].correctedheading, descentspeed, plantable[i].heading, dspeed, whead, wspeed)) 
				return FALSE; 
			percent = plantable[i].alt - plantable[i-1].alt + plantable[i].dist*60*descentrate/dspeed;
			percent /= plantable[i].dist*60*(climbrate/cspeed+descentrate/dspeed);
			view->gcDistInterpolate(plantable[i-1].lat, plantable[i-1].lon, plantable[i].lat, plantable[i].lon, percent, toc.lat, toc.lon);
			averageWind(plantable[i-1].windVel, plantable[i-1].windheading, plantable[i].windVel, plantable[i].windheading, toc.windVel, toc.windheading, percent);
 			toc.dist = view->gcDist(plantable[i-1].lat, plantable[i-1].lon, toc.lat, toc.lon)/1852; // in nm
			toc.heading = view->bearing + doc->GetMagVar(toc.lat, toc.lon);
 			plantable[i].dist = view->gcDist(toc.lat, toc.lon, plantable[i].lat, plantable[i].lon)/1852; // in nm
			plantable[i].heading = view->bearing + doc->GetMagVar(plantable[i].lat, plantable[i].lon);
			itoc = ibod = i;
			plantable.InsertAt(i, toc); // insert toc/bod record, shift others 
			if (FALSE == adjust(i,   TRUE,  CLIMB))  return FALSE;
			if (FALSE == adjust(i+1, FALSE, DESCEND)) return FALSE;
			return TRUE;
		} else if ((talt > cruisealtitude) && (ibod == -1)) { 
			// need to insert bod
			toc.index = -2; // special value for bod
			percent = (talt - cruisealtitude) / (talt - plantable[i].alt);
			view->gcDistInterpolate(plantable[i-1].lat, plantable[i-1].lon, plantable[i].lat, plantable[i].lon, percent, toc.lat, toc.lon);
			toc.alt = cruisealtitude;
			averageWind(plantable[i-1].windVel, plantable[i-1].windheading, plantable[i].windVel, plantable[i].windheading, toc.windVel, toc.windheading, percent);
 			toc.dist = view->gcDist(plantable[i-1].lat, plantable[i-1].lon, toc.lat, toc.lon)/1852; // in nm
			toc.heading = view->bearing + doc->GetMagVar(toc.lat, toc.lon);
 			plantable[i].dist = view->gcDist(toc.lat, toc.lon, plantable[i].lat, plantable[i].lon)/1852; // in nm
			plantable[i].heading = view->bearing + doc->GetMagVar(plantable[i].lat, plantable[i].lon);
			ibod = i;
			plantable.InsertAt(i, toc); // insert toc record, shift others 
			if (FALSE == adjust(i,   forward, CRUISE))  return FALSE;
			if (FALSE == adjust(i+1, forward, DESCEND)) return FALSE;
			return TRUE;
		} else if ((itoc==-2)&&(ibod==-2))
			; // do nothing
		else 
			plantable[i-1].alt = min (talt, cruisealtitude) ;
	}
 	plantable[i].ete = plantable[i].dist / plantable[i].correctedvelocity * 60; //minutes
	plantable[i].fuel = fuel * plantable[i].ete / 60;

	return TRUE;
}

void FlightPlan::buildTable() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	CNavDoc *doc = (CNavDoc *) main->GetActiveDocument();
	plantable.RemoveAll();  // delete old table
	plantable.FreeExtra();
	if (view->plancount < 1)
		return;

	double windspeed, windir, oldLat, oldLon;
	int type;
	windStruct w;
	// insert known items into table
	for (int p=0; p<(view->plancount); p++) {
		planRec r;
		r.index = view->plan[p];
		type = doc->db[r.index].type;
		r.lat = doc->db[r.index].lat;
		r.lon = doc->db[r.index].lon;
		if (view->windtable.Lookup(r.index, w)) {
			windspeed = (double) w.speed;
			windir = (double) w.direction;
		} else
			windspeed = windir = 0.0;
		r.windVel = windspeed;	// correct wind heading for mag variation (& change "from" to "to")							
		r.windheading = windir + 180 + doc->GetMagVar(doc->db[r.index].lat, doc->db[r.index].lon);	
		if (r.windheading > 360)
			r.windheading -= 360;
		if (r.windheading < 0)
			r.windheading += 360;
		if (p == 0) {
			r.dist = r.Tdist = r.ete = r.Tete = r.fuel = r.Tfuel = 0.0; 
			r.heading = r.correctedheading = r.correctedvelocity = 0;
			r.alt = doc->db[r.index].alt/.3048; // convert to ft
			if ((r.alt <= 0) || (type != 2)&&(type != 4)&&(type != 8)) // some things don't have a valid altitude, so cruise
				r.alt = cruisealtitude;
		} else {
			r.dist = view->gcDist(oldLat, oldLon, doc->db[r.index].lat, doc->db[r.index].lon)/1852; // in nm
			r.heading = view->bearing + doc->GetMagVar(doc->db[r.index].lat, doc->db[r.index].lon);
			if (r.heading <= 0)
				r.heading += 360;
			if (p == view->plancount-1) {
				r.alt = doc->db[r.index].alt/.3048; // convert to ft
				if ((r.alt <= 0) || (type != 2)&&(type != 4)&&(type != 8)) // some things don't have a valid altitude, so cruise
					r.alt = cruisealtitude;
			}
		}
		oldLat = r.lat;
		oldLon = r.lon;
		plantable.Add(r);
	}
	if (view->plancount < 2)
		return;

	itoc = ibod = -1;
	D = C = TRUE;
	// ILS, runways and airports use their own altitude, others use cruise altitude
	int idp = doc->db[plantable[plantable.GetSize()-1].index].type; 
	if ( (idp != ILS_TYPE)&&(idp != AIRPORT_TYPE)&&(idp != RUNWAY_TYPE) )
		D = FALSE;
	idp = doc->db[plantable[0].index].type; 
	if ( (idp != ILS_TYPE)&&(idp != AIRPORT_TYPE)&&(idp != RUNWAY_TYPE) )
		C = FALSE;

	if (C || !D) {
		for (int p=1; p<plantable.GetSize(); p++) {
			if (p == itoc+1)
				continue;
			if (!C)
				adjust(p, TRUE, CRUISE);
			else
				adjust(p, TRUE, CLIMB);
		}
	}
	if (D) {
		for (p=plantable.GetSize()-1; p>0; p--) {
			if (p <= ibod)
				break;
			adjust(p, FALSE, DESCEND);
		}
	}

	for (p=1; p<plantable.GetSize(); p++) { // update totals
		plantable[p].Tdist = plantable[p-1].Tdist + plantable[p].dist;
		plantable[p].Tete  = plantable[p-1].Tete  + plantable[p].ete;
		plantable[p].Tfuel = plantable[p-1].Tfuel + plantable[p].fuel;
	}
}

void FlightPlan::update() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	CNavDoc *doc = (CNavDoc *) main->GetActiveDocument();
    m_list.DeleteAllItems();
	LV_ITEM i;
	i.mask = LVIF_TEXT;
	char line[50];
	int dbtype;

	buildTable();

	for (int k=0; k<plantable.GetSize(); k++) {
 		i.iItem = k;	   
		i.iSubItem = 0;
		if (plantable[k].index >= 0)
			dbtype = doc->db[plantable[k].index].type;
		else 
			dbtype = plantable[k].index; // toc, bod, toc/bob
		if (-1 == dbtype) {
			sprintf(line, "top of climb");
			i.pszText = line;
			m_list.InsertItem(&i);
		} else if (-2 == dbtype) {
			sprintf(line, "beginning of descent");
			i.pszText = line;
			m_list.InsertItem(&i);
		} else if (-3 == dbtype) {
			sprintf(line, "top climb/begin descent");
			i.pszText = line;
			m_list.InsertItem(&i);
		} else if (5 == dbtype) {
			strcpy(line, "Inner marker");	
			i.pszText = line;
			m_list.InsertItem(&i);
		} else if (6 == dbtype) {
			strcpy(line, "Middle marker");	
			i.pszText = line;
			m_list.InsertItem(&i);
		} else if (7 == dbtype) {
			strcpy(line, "Outer marker");	
			i.pszText = line;
			m_list.InsertItem(&i);
		} else if (8 == dbtype) {
			strcpy(line, "Runway ");
			strcat(line, doc->db[plantable[k].index].id);	
			i.pszText = line;
			m_list.InsertItem(&i);
		} else if (2 == dbtype) {
			CString name;
			strcpy(line, "ILS - ");
			name = line;
			name = name + doc->db[plantable[k].index].name;	
			name.TrimRight();
			i.pszText = name.GetBuffer(1);
			m_list.InsertItem(&i);
			name.ReleaseBuffer();
		} 
		else if (11 == dbtype)
		{
			CString name;
			name = "Way Point";
			i.pszText = name.GetBuffer(1);
			m_list.InsertItem(&i);
			name.ReleaseBuffer();

		}else {
			CString name;
			name = doc->db[plantable[k].index].name;
			name.TrimRight();
			i.pszText = name.GetBuffer(1);
			m_list.InsertItem(&i);
			name.ReleaseBuffer();
		}

		if ((1 == dbtype) || (2 == dbtype) || (3 == dbtype)) {
			i .iSubItem = 1;
			i.pszText = doc->db[plantable[k].index].id;
			m_list.SetItem(&i);
			sprintf(line, "%6.2f", doc->db[plantable[k].index].freq);
			i .iSubItem = 3;
			i.pszText = line;
			m_list.SetItem(&i);
		}

		if (10 == dbtype) {
			i .iSubItem = 1;
			i.pszText = "ATIS";
			m_list.SetItem(&i);
			sprintf(line, "%6.2f", doc->db[plantable[k].index].freq);
			i .iSubItem = 3;
			i.pszText = line;
			m_list.SetItem(&i);
		}

		if ((8 == dbtype) || (9 == dbtype) || (11 == dbtype)) {
			i .iSubItem = 1;
			i.pszText = doc->db[plantable[k].index].id;
			m_list.SetItem(&i);
		}

		if ((-1 == dbtype) || (-2 == dbtype) || (-3 == dbtype)) {
			i .iSubItem = 1;
			sprintf(line, "%dft", (int) (plantable[k].alt + .5));
			i.pszText = line;
			m_list.SetItem(&i);
		}

		if (1 == dbtype)
			strcpy(line, "NDB");	
		else if (2 == dbtype)
			strcpy(line, "ILS");	
		else if (3 == dbtype)
			strcpy(line, "VOR");	
		else if (4 == dbtype)
			strcpy(line, "Airport");	
		else if (5 == dbtype)
			strcpy(line, "Marker");	
		else if (6 == dbtype)
			strcpy(line, "Marker");	
		else if (7 == dbtype)
			strcpy(line, "Marker");	
		else if (8 == dbtype)
			strcpy(line, "Airport");	
		else if (9 == dbtype)
			strcpy(line, "GPS fix");	
		else if (10 == dbtype)
			strcpy(line, "ATIS");	
		else if (11 == dbtype)
			strcpy(line, "WPT");	
		else if (-1 == dbtype)
			strcpy(line, "TOC");	
		else if (-2 == dbtype)
			strcpy(line, "BOD");	
		else if (-3 == dbtype)
			strcpy(line, "TOC/BOD");	
		i .iSubItem = 2;
		i.pszText = line;
		m_list.SetItem(&i);

		sprintf(line, "%3.0f", plantable[k].correctedheading);
		i .iSubItem = 4;
		i.pszText = line;
		m_list.SetItem(&i);
		sprintf(line, "%.1f", plantable[k].dist);
		i .iSubItem = 5;
		i.pszText = line;
		m_list.SetItem(&i);
		sprintf(line, "%.1f", plantable[k].Tdist);
		i .iSubItem = 6;
		i.pszText = line;
		m_list.SetItem(&i);
		sprintf(line, "%.1f", plantable[k].ete);
		i .iSubItem = 7;
		i.pszText = line;
		m_list.SetItem(&i);
		sprintf(line, "%.1f", plantable[k].Tete);
		i .iSubItem = 8;
		i.pszText = line;
		m_list.SetItem(&i);
		sprintf(line, "%.1f", plantable[k].fuel);
		i .iSubItem = 9;
		i.pszText = line;
		m_list.SetItem(&i);
		sprintf(line, "%.1f", plantable[k].Tfuel);
		i .iSubItem = 10;
		i.pszText = line;
		m_list.SetItem(&i);
		sprintf(line, "%d", (int)(plantable[k].alt + .5));
		i .iSubItem = 11;
		i.pszText = line;
		m_list.SetItem(&i);
		sprintf(line, "%d", (int)(plantable[k].windVel + .5));
		i .iSubItem = 12;
		i.pszText = line;
		m_list.SetItem(&i);
		int wh = (int)(plantable[k].windheading + 180.49 - doc->GetMagVar(plantable[k].lat, plantable[k].lon));
		wh %= 360;
		sprintf(line, "%d", wh);
		i .iSubItem = 13;
		i.pszText = line;
		m_list.SetItem(&i);
	}
	setColumnWidth();

	if (sel != -1) {
		m_list.SetItemState(sel, LVNI_SELECTED, LVNI_SELECTED);
		m_list.SetItemState(sel, LVIS_DROPHILITED, LVIS_DROPHILITED);
		m_list.SetItemState(sel, 0, LVIS_FOCUSED);
		m_list.Update(sel);
		m_list.EnsureVisible(sel, FALSE);
	}
	checkButtons();
}

// set columns to be as small as possible
void FlightPlan::setColumnWidth() {
	int t, max;
	char buf[100];
	LV_ITEM pItem;
	pItem.mask = LVIF_TEXT;
	pItem.pszText = buf;
	pItem.cchTextMax = 100;
	LV_COLUMN pColumn;
	pColumn.mask = LVCF_TEXT;
	pColumn.pszText = buf;
	pColumn.cchTextMax = 100;
	for (int col=0; col<=13; col++) {
		pColumn.iSubItem = col;
		m_list.GetColumn(col, &pColumn);
		max = m_list.GetStringWidth(pColumn.pszText);
		for (int k=0; k<plantable.GetSize(); k++) {
			pItem.iItem = k;
			pItem.iSubItem = col;
			m_list.GetItem(&pItem);
			t = m_list.GetStringWidth(pItem.pszText);
			if (t>max)
				max = t;
		}
		// despite the docs, it does not take column margin into account
		m_list.SetColumnWidth(col, max + 13); 
	}
}

void FlightPlan::OnClear() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	view->plancount = 0;
	view->windtable.RemoveAll();
	view->abort = TRUE;
	view->Invalidate(TRUE);
	update();
}

void FlightPlan::OnOK() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	view->planning = FALSE;
	view->abort = TRUE;
	view->Invalidate(TRUE);
	CDialog::OnOK();
}

void FlightPlan::OnClose() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	view->planning = FALSE;
	view->abort = TRUE;
	view->Invalidate(TRUE);

	DestroyWindow(); // kill self, PostNcDestroy deletes memory	
}

void FlightPlan::OnUp() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	if (view->plancount < 2)
		return;
	// find the selected item
	int selected = -1;
	for (int i=0; i<plantable.GetSize(); i++) {
		if(m_list.GetItemState(i, LVNI_SELECTED) == LVNI_SELECTED)
			selected = i;
	}
	if (selected == -1) // nothing was selected
		selected = view->plancount-1;
	int t = plantable[selected].index; // skip toc/bod if selected
	if ((t == -1) || (t == -2) | (t == -3))
		return;
	if (selected == 0) // beginning of list
		return;
	int save = plantable[selected].index;
	int iplan = 0; // find index of selected item in view->plan
	for (int z=0; z<view->plancount; z++) 
		if (view->plan[z] == save)
			break;
		else
			iplan++;
	int temp = view->plan[iplan-1];
	view->plan[iplan-1] = view->plan[iplan];
	view->plan[iplan] = temp;
	m_list.SetItemState(selected, 0, LVNI_SELECTED);

	// need to recalc plan to know what selection should be
	iplan = 0;
	update();
	for (z=0; z<plantable.GetSize(); z++) 
		if (plantable[z].index == save)
			break;
		else
			iplan++;
	sel = iplan;
	checkButtons();

	m_list.SetItemState(sel, LVNI_SELECTED, LVNI_SELECTED);
	m_list.SetItemState(sel, LVIS_DROPHILITED, LVIS_DROPHILITED);
	m_list.SetItemState(sel, 0, LVIS_FOCUSED);
	view->Invalidate(TRUE);
}

void FlightPlan::OnDown() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	if (view->plancount < 2)
		return;
		
	// find the selected item
	int selected = -1;
	for (int i=0; i<plantable.GetSize(); i++) {
		if(m_list.GetItemState(i, LVNI_SELECTED) == LVNI_SELECTED)
			selected = i;
	}
	if (selected == -1)
		selected = 0;
	int t = plantable[selected].index; // skip toc/bod if selected
	if ((t == -1) || (t == -2) | (t == -3))
		return;
	if (selected == plantable.GetSize()-1) // end of list
		return;
	int save = plantable[selected].index;
	int iplan = 0; // find index of selected item in view->plan
	for (int z=0; z<view->plancount; z++)
		if (view->plan[z] == save)
			break;
		else
			iplan++;
	int temp = view->plan[iplan+1];
	view->plan[iplan+1] = view->plan[iplan];
	view->plan[iplan] = temp;
	m_list.SetItemState(selected, 0, LVNI_SELECTED);
	
	// need to recalc plan to know what selection should be
	iplan = 0;
	update();
	for (z=0; z<plantable.GetSize(); z++) 
		if (plantable[z].index == save)
			break;
		else
			iplan++;
	sel = iplan;
	checkButtons();

	m_list.SetItemState(sel, LVNI_SELECTED, LVNI_SELECTED);
	m_list.SetItemState(sel, LVIS_DROPHILITED, LVIS_DROPHILITED);
	m_list.SetItemState(sel, 0, LVIS_FOCUSED);
	view->Invalidate(TRUE);
}

void FlightPlan::OnDelete() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	if (view->plancount == 0)
		return;
		
	// find the selected item
	int selected = -1;
	for (int i=0; i<plantable.GetSize(); i++) {
		if(m_list.GetItemState(i, LVNI_SELECTED) == LVNI_SELECTED)
			selected = i;
	}
	if (selected == -1) // nothing was selected
		return;
	int t = plantable[selected].index; // skip toc/bod if selected
	if ((t == -1) || (t == -2) | (t == -3))
		return;

	view->windtable.RemoveKey(plantable[selected].index);

	int iplan = 0; // find index of selected item in view->plan
	for (int z=0; z<view->plancount; z++)
		if (view->plan[z] == plantable[selected].index)
			break;
		else
			iplan++;

	for (i=iplan+1; i<view->plancount; i++) 
		view->plan[i-1] = view->plan[i];
	view->plancount -=1;
	sel = -1;
	update();
	view->Invalidate(TRUE);
}

void FlightPlan::OnClickList(NMHDR* pNMHDR, LRESULT* pResult) {
	if (sel != -1) {
		// remove old highlighting
		m_list.SetItemState(sel, 0, LVNI_SELECTED);
		m_list.SetItemState(sel, 0, LVIS_DROPHILITED);
		m_list.SetItemState(sel, 0, LVIS_FOCUSED);
		m_list.Update(sel);
		sel = -1;
	}

	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	int selected = -1;
	for (int i=0; i<plantable.GetSize(); i++) {
		if(m_list.GetItemState(i, LVNI_SELECTED) == LVNI_SELECTED)
			selected = i;
	}
	checkButtons();
	*pResult = 0;
}

void FlightPlan::checkButtons() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();

	// find the selected item
	int selected = -1;
	for (int i=0; i<plantable.GetSize(); i++) {
		if(m_list.GetItemState(i, LVNI_SELECTED) == LVNI_SELECTED)
			selected = i;
	}
	if (selected >=0) {
		int t = plantable[selected].index; // toc/bod is selected
		if ((t == -1) || (t == -2) | (t == -3))
			selected = -1;  // act as if nothing is selected
	}

	

	CButton *clearButton;
	clearButton = (CButton *) GetDlgItem(IDC_CLEAR);
	if (plantable.GetSize() == 0) 
		clearButton->EnableWindow(FALSE);
	else
		clearButton->EnableWindow(TRUE);

	CButton *deleteButton;
	deleteButton = (CButton *) GetDlgItem(IDC_DELETE);
	if (selected == -1) 
		deleteButton->EnableWindow(FALSE);
	else
		deleteButton->EnableWindow(TRUE);

	CButton *upButton;
	upButton = (CButton *) GetDlgItem(IDC_UP);
	if (selected < 1) 
		upButton->EnableWindow(FALSE);
	else
		upButton->EnableWindow(TRUE);

	CButton *downButton;
	downButton = (CButton *) GetDlgItem(IDC_DOWN);
	if ((selected == plantable.GetSize()-1) || (selected < 0)) 
		downButton->EnableWindow(FALSE);
	else
		downButton->EnableWindow(TRUE);

	CButton *autoButton;
	autoButton = (CButton *) GetDlgItem(IDC_AUTO);
	if ((view->plancount < 2) || (selected == 0))
		autoButton->EnableWindow(FALSE);
	else
		autoButton->EnableWindow(TRUE);

	CButton *windButton; // need at least one item to set wind
	windButton = (CButton *) GetDlgItem(IDC_WIND);
	if (view->plancount < 1)
		windButton->EnableWindow(FALSE);
	else
		windButton->EnableWindow(TRUE);

	CButton *flytoButton;
	flytoButton = (CButton *) GetDlgItem(IDC_FLYTO);
	if (selected == -1) 
		flytoButton->EnableWindow(FALSE);
	else
		flytoButton->EnableWindow(TRUE);

}

void FlightPlan::OnAuto() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	CNavDoc *doc = (CNavDoc *) main->GetActiveDocument();
	char sbar[300];

	if (view->plancount < 2)
		return;
		
	// find the selected item
	int selected = -1;
	for (int i=0; i<view->plancount; i++) {
		if(m_list.GetItemState(i, LVNI_SELECTED) == LVNI_SELECTED)
			selected = i;
	}
	if (selected == -1)
		selected = view->plancount-1;
	else {
		int t = plantable[selected].index; // toc/bod is selected
		if ((t == -1) || (t == -2) | (t == -3))
			selected -= 1;  // act as if previous item were selected
		t = plantable[selected].index; // toc/bod is still selected
		if ((t == -1) || (t == -2) | (t == -3))
			selected -= 1;  // act as if previous item were selected
	}

	if (selected == 0)
		return;
 
	int start = view->plan[selected-1];
	int end = view->plan[selected];
	double startx = doc->db[start].lon;
	double starty = doc->db[start].lat;
	double endx = doc->db[end].lon;
	double endy = doc->db[end].lat;
	double ideal = view->idealDist*1000; // in km
	double percent, midx, midy, startDist, newdist;
	int index, type;
	int found, recover;
	MSG msg;  
 
	double dist = view->gcDist(starty, startx, endy, endx);

	CButton *autoButton;
	autoButton = (CButton *) GetDlgItem(IDC_AUTO);
	autoButton->EnableWindow(FALSE);
	while (dist > ideal) {
		if (dist > 2*ideal)
			percent = ideal / dist;
		else 
			percent = .5;
		view->gcDistInterpolate(starty, startx, endy, endx, percent, midy, midx);

		sprintf(sbar, "Routing from %s", doc->db[start].name);
		main->SetStatusBar(sbar);

		// find nearest station to midx, midy
		startDist = dist;
		newdist = 2.0 * ideal;
		found = FALSE;
		recover = FALSE;
			
		for (int i=0; i<doc->dbCount; i++) {
			type = doc->db[i].type;
			if (type == 2) goto next;
			if (type == 5) goto next;
			if (type == 6) goto next;
			if (type == 7) goto next;
			if (type == 9) goto next;
			if (type == 10) goto next;
			if ((type == 1) && (!view->planNDB)) goto next;
			if ((type == 3) && (!view->planVOR)) goto next;
			if ((type == 4) && (!view->planAirport)) goto next;
			if ((type == 8) && (!view->planAirport)) goto next;
			// take over message processing to simulate threading 
			while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) { 
				if (msg.message == WM_QUIT) { 
			 		PostQuitMessage(1);
			 		return;      
				}	 
			 	TranslateMessage(&msg);
			 	DispatchMessage(&msg);
	  		}
			// continue
			newdist = view->gcDist(midy, midx, doc->db[i].lat, doc->db[i].lon);
			if ((newdist < startDist) && (i != start)) {
				startDist = newdist;
				found = TRUE;
				index = i;
			}
			next:;
		} // index is closest navaid to midx, midy
	
		if (found == FALSE) { 
			recover = TRUE;
		} else { // found == TRUE
			newdist = view->gcDist(doc->db[index].lat, doc->db[index].lon, doc->db[end].lat, doc->db[end].lon);
			if (newdist > (dist - 1000))  // must make at least a 1 km improvement
				recover = TRUE;
		}
	
		if (recover == FALSE) {
			if(index == view->plan[selected])  // found end point, we must be done
				break;
			// insert the new navaid in the plan
			view->plancount += 1;
			for (int k=view->plancount-1; k>selected; k--) 
			 	view->plan[k] = view->plan[k-1];
			view->plan[k] = index;
			selected++;
			
			// repeat until done
			start = index;
			startx = doc->db[index].lon;
			starty = doc->db[index].lat;
			dist = newdist;
		} else { // recover by pretending we found something and move on or insert GPS fix
			if (view->planContinue) { //insert a GPS fix
				doc->OnMapInsertgpsfix(midy, midx);
				index = doc->dbCount - 1;
				// insert the new gps in the plan
				view->plancount += 1;
				for (int k=view->plancount-1; k>selected; k--) 
			 		view->plan[k] = view->plan[k-1];
				view->plan[k] = index;
				selected++;
			}
			recover = FALSE;
			startx = midx;
			starty = midy;
			dist = view->gcDist(starty, startx, endy, endx);
		}
	}

	autoButton->EnableWindow(TRUE);
	main->SetStatusBar("Ready");
	view->Invalidate(TRUE);
}

void FlightPlan::OnOptions() {
 	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	view->planOpts();
}

void FlightPlan::OnPrint() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	CNavDoc *doc = (CNavDoc *) main->GetActiveDocument();
	CFont font;
	TEXTMETRIC tm;

	if (view->plancount < 2)
		return;
	
	CPrintDialog pd(FALSE, PD_ALLPAGES | PD_USEDEVMODECOPIES | PD_NOPAGENUMS 
	              | PD_HIDEPRINTTOFILE | PD_NOSELECTION);

	if (pd.DoModal() == IDOK) {
		HDC hDC = pd.GetPrinterDC();
		if (hDC == NULL)
			return;
		CDC *pDC;
		pDC = pDC->FromHandle(hDC);

		typedef struct { // Win95 version since VC++ 2.1 has only NT version
		  int cbSize;
		  LPTSTR lpszDocName;
		  LPTSTR lpszOutput;
		  LPTSTR lpszDataType;
		  DWORD fwType;
		} W95DOCINFO;
		W95DOCINFO di;
	
		di.cbSize = sizeof(W95DOCINFO);
		di.lpszDocName = "NavPlan.doc";
		di.lpszOutput = (LPTSTR) NULL;
		di.lpszDataType = (LPTSTR) NULL; // secret new parameters for Win95?
		di.fwType = 0;					 // maybe this should be NULL?

		int startIndex = 1;
		pDC->StartDoc((char *) &di);
		beginPage:
		int newPage = FALSE;
		pDC->StartPage();

		pDC->SetMapMode(MM_TWIPS); 

		// 10 point proportional-pitch font
		font.CreateFont(-200, 0, 0, 0, 400, FALSE, FALSE,
						0, ANSI_CHARSET, OUT_TT_PRECIS,
						CLIP_DEFAULT_PRECIS, PROOF_QUALITY,
						DEFAULT_PITCH | FF_SWISS | TMPF_TRUETYPE, "Ariel");
		CFont *pOldFont = (CFont *) (pDC->SelectObject(&font));
		
 		double pagew = pDC->GetDeviceCaps(HORZRES) / (double) pDC->GetDeviceCaps(LOGPIXELSX);
		double pageh = pDC->GetDeviceCaps(VERTRES) / (double) pDC->GetDeviceCaps(LOGPIXELSY);

		pDC->GetOutputTextMetrics(&tm);
		int nHeight = tm.tmHeight + tm.tmExternalLeading;

		// ready to print

		// find minimum size for columns
		CSize locSize  = pDC->GetTextExtent("Location", sizeof("Location"));
		CSize idSize   = pDC->GetTextExtent("ID", sizeof("ID"));
		CSize freqSize = pDC->GetTextExtent("Freq", sizeof("Freq"));
		CSize headSize = pDC->GetTextExtent("Course", sizeof("Course"));
		CSize mheadSize = pDC->GetTextExtent("Mag Crs", sizeof("Mag Crs"));
		CSize recipSize = pDC->GetTextExtent("(recip)", sizeof("(recip)"));
		CSize distSize = pDC->GetTextExtent("Distance", sizeof("Distance"));
		CSize tdisSize = pDC->GetTextExtent("Total", sizeof("Total"));
		CSize eteSize = pDC->GetTextExtent("ETE", sizeof("ETE"));
		CSize teteSize = pDC->GetTextExtent("Total", sizeof("Total"));
		CSize fuelSize = pDC->GetTextExtent("Fuel", sizeof("Fuel"));
		CSize tfuelSize = pDC->GetTextExtent("Total", sizeof("Total"));

		CSize nameSize;
		CString name;
		LV_ITEM i;
		i.mask = LVIF_TEXT;
		int dbtype;
		for (int k=0; k<plantable.GetSize(); k++) {
			if (plantable[k].index >= 0)
				dbtype = doc->db[plantable[k].index].type;
			else 
				dbtype = plantable[k].index; // toc, bod, toc/bob
			CString bigName;
			name = readList(k, 0);
			bigName = name + "ILS - ";
			nameSize = pDC->GetTextExtent(name, name.GetLength());
			if (nameSize.cx < (pDC->GetTextExtent(bigName, bigName.GetLength())).cx)
				nameSize = pDC->GetTextExtent(bigName, bigName.GetLength());
			if (nameSize.cx > locSize.cx)
				locSize.cx = nameSize.cx;

			if ((dbtype < 4) || (dbtype == 8)) {
				name = readList(k, 1);
				nameSize = pDC->GetTextExtent(name, name.GetLength());
				if (nameSize.cx > idSize.cx)
					idSize.cx = nameSize.cx;
			}

			if ((dbtype < 4) || (dbtype == 10)){
				name = readList(k, 3);
				nameSize = pDC->GetTextExtent(name, name.GetLength());
				if (nameSize.cx > freqSize.cx)
					freqSize.cx = nameSize.cx;
			}

			name = readList(k, 4);
			nameSize = pDC->GetTextExtent(name, name.GetLength());
			if (nameSize.cx > headSize.cx)
				headSize.cx = nameSize.cx;

			name = readList(k, 5);
			nameSize = pDC->GetTextExtent(name, name.GetLength());
			if (nameSize.cx > distSize.cx)
				distSize.cx = nameSize.cx;

			name = readList(k, 6);
			nameSize = pDC->GetTextExtent(name, name.GetLength());
			if (nameSize.cx > tdisSize.cx)
				tdisSize.cx = nameSize.cx;

			name = readList(k, 7);
			nameSize = pDC->GetTextExtent(name, name.GetLength());
			if (nameSize.cx > eteSize.cx)
				eteSize.cx = nameSize.cx;

			name = readList(k, 8);
			nameSize = pDC->GetTextExtent(name, name.GetLength());
			if (nameSize.cx > teteSize.cx)
				teteSize.cx = nameSize.cx;

			name = readList(k, 9);
			nameSize = pDC->GetTextExtent(name, name.GetLength());
			if (nameSize.cx > fuelSize.cx)
				fuelSize.cx = nameSize.cx;

			name = readList(k, 10);
			nameSize = pDC->GetTextExtent(name, name.GetLength());
			if (nameSize.cx > tfuelSize.cx)
				tfuelSize.cx = nameSize.cx;
		}
		// leave space between columns (2 pts, 1 pt on either size of line)
 		locSize.cx   = locSize.cx   + 144; 
		idSize.cx    = idSize.cx    + 144;
		freqSize.cx  = freqSize.cx  + 144;
		headSize.cx  = headSize.cx  + 144;
		mheadSize.cx = mheadSize.cx + 144;
		distSize.cx  = distSize.cx  + 144;
		tdisSize.cx  = tdisSize.cx  + 144;
		eteSize.cx   = eteSize.cx   + 144;
		teteSize.cx  = teteSize.cx  + 144;
		fuelSize.cx  = fuelSize.cx  + 144;
		tfuelSize.cx = tfuelSize.cx + 144;
		int idfreq = max(idSize.cx, freqSize.cx);
		int disttotal = max(distSize.cx, tdisSize.cx);
		int etetotal = max(eteSize.cx, teteSize.cx);
		int fueltotal = max(fuelSize.cx, tfuelSize.cx);
		// resolution remaining
		int excess = (int) pagew * 1440 - (locSize.cx + idfreq + headSize.cx + mheadSize.cx + disttotal + etetotal + fueltotal);

		// do column headings
		int x = 72;
		pDC->TextOut(x, 0, "From");
		x += locSize.cx;
		pDC->TextOut(x, 0, "ID");
		pDC->TextOut(x, -nHeight - 20, "Freq");
		pDC->MoveTo(x, nHeight);
		pDC->LineTo(x+idfreq, nHeight);
		x += idfreq;
		pDC->TextOut(x, 0, "To");
		x += locSize.cx;
		pDC->TextOut(x, 0, "ID");
		pDC->TextOut(x, -nHeight - 20, "Freq");
		pDC->MoveTo(x, nHeight);
		pDC->LineTo(x+idfreq, nHeight);
		x += idfreq;
		pDC->TextOut(x, 0, "Course");
		pDC->TextOut(x, -nHeight - 20, "(recip)");
		x += headSize.cx;
		pDC->TextOut(x, 0, "Mag Crs");
		pDC->TextOut(x, -nHeight - 20, "(recip)");
		x += mheadSize.cx;
		pDC->TextOut(x, 0, "Distance");
		pDC->TextOut(x, -nHeight - 20, "Total");
		pDC->MoveTo(x, nHeight);
		pDC->LineTo(x+disttotal, nHeight);
		x += disttotal;
		pDC->TextOut(x, 0, "ETE");
		pDC->TextOut(x, -nHeight - 20, "Total");
		pDC->MoveTo(x, nHeight);
		pDC->LineTo(x+etetotal, nHeight);
		x += etetotal;
		pDC->TextOut(x, 0, "Fuel");
		pDC->TextOut(x, -nHeight - 20, "Total");
		pDC->MoveTo(x, nHeight);
		pDC->LineTo(x+fueltotal, nHeight);
		x += fueltotal;

		// do data
		int y = -2 * nHeight - 40, dbtypeLast; 
		for (k=startIndex; k<plantable.GetSize(); k++) {
			if (plantable[k].index >= 0)
				dbtype = doc->db[plantable[k].index].type;
			else 
				dbtype = plantable[k].index; // toc, bod, toc/bob
			if (plantable[k-1].index > 0)
				dbtypeLast = doc->db[plantable[k-1].index].type;
			else 
				dbtypeLast = plantable[k-1].index; // toc, bod, toc/bob

			x = 72;
			name = readList(k-1, 0);
			pDC->TextOut(x, y, name);
			x += locSize.cx;

			if ((dbtypeLast < 4) || (dbtypeLast == 10)) {
				name = readList(k-1, 1);
				pDC->TextOut(x, y, name);
				if (dbtypeLast > 0 ) {
					name = readList(k-1, 3);
					pDC->TextOut(x, y - nHeight, name);
				}
			} else if (dbtypeLast == 9) {
				name = "GPS";
				pDC->TextOut(x, y, name);
				name = readList(k-1, 1);
				pDC->TextOut(x, y - nHeight, name);
			}
			x += idfreq;

			name = readList(k, 0);
			pDC->TextOut(x, y, name);
			x += locSize.cx;

			if ((dbtype < 4) || (dbtype == 10)) {
				name = readList(k, 1);
				pDC->TextOut(x, y, name);
				if (dbtype > 0 ) {
					name = readList(k, 3);
					pDC->TextOut(x, y - nHeight, name);
				}
			} else if (dbtype == 9) {
				name = "GPS";
				pDC->TextOut(x, y, name);
				name = readList(k, 1);
				pDC->TextOut(x, y - nHeight, name);
			}
			x += idfreq;
			
			char mbuf[30];
			int magHeading, trueHeading;

			//  calculate true heading
			view->gcDist(plantable[k-1].lat, plantable[k-1].lon, plantable[k].lat, plantable[k].lon);
			trueHeading = (int) view->bearing;
			sprintf(mbuf, " %d", trueHeading);
			name = mbuf;
			name.TrimRight();
			pDC->TextOut(x, y, " " + name);

			// recip heading
			if (trueHeading >= 180) trueHeading -= 180;
			else if (trueHeading < 180) trueHeading += 180;
			sprintf(mbuf, "(%d)", trueHeading);
			name = mbuf;
			name.TrimRight();
			pDC->TextOut(x, y - nHeight, name);
			x += headSize.cx;

			// magnetic heading
			name = readList(k, 4); //  corrected (magnetic + wind) heading
			sscanf((LPCTSTR) name, "%d", &magHeading);
			pDC->TextOut(x, y, name);

			// recip heading
			if (magHeading >= 180) magHeading -= 180;
			else if (magHeading < 180) magHeading += 180;
			sprintf(mbuf, "(%d)", magHeading);
			name = mbuf;
			name.TrimRight();
			pDC->TextOut(x, y - nHeight, name);
			x += mheadSize.cx;

			name = readList(k, 5);	// distance
			pDC->TextOut(x, y, name);
			name = readList(k, 6);	// total distance
			pDC->TextOut(x, y - nHeight, name);
			x += disttotal;

			name = readList(k, 7);	// ete
			pDC->TextOut(x, y, name);
			name = readList(k, 8);	// total ete
			pDC->TextOut(x, y - nHeight, name);
			x += etetotal;

			name = readList(k, 9);	// fuel
			pDC->TextOut(x, y, name);
			name = readList(k, 10);	// total fuel
			pDC->TextOut(x, y - nHeight, name);

			y -= 2*nHeight-20;
 			// do horizontal line
			pDC->MoveTo(0, y);
			x += fueltotal;
			pDC->LineTo(x, y);
			y -= 20;

			// check to see if next item must go on new page
			if ((y < (int) (pageh * -1440)) && (k < plantable.GetSize()-1)) {
				newPage = TRUE;
				startIndex = k+1;
				break;
			}
		}

		// do vertical lines
		y += 20;
		pDC->MoveTo(0, 0);
		pDC->LineTo(0, y);
		x = locSize.cx;
		pDC->MoveTo(x, 0);
		pDC->LineTo(x, y);
		x += idfreq;
 		pDC->MoveTo(x, 0);
		pDC->LineTo(x, y);
		x += locSize.cx;
		pDC->MoveTo(x, 0);
		pDC->LineTo(x, y);
		x += idfreq;
 		pDC->MoveTo(x, 0);
		pDC->LineTo(x, y);
		x += headSize.cx;
 		pDC->MoveTo(x, 0);
		pDC->LineTo(x, y);
		x += mheadSize.cx;
 		pDC->MoveTo(x, 0);
		pDC->LineTo(x, y);
		x += disttotal;
 		pDC->MoveTo(x, 0);
		pDC->LineTo(x, y);
		x += etetotal;
 		pDC->MoveTo(x, 0);
		pDC->LineTo(x, y);
		x += fueltotal+72;
 		pDC->MoveTo(x, 0);
		pDC->LineTo(x, y);
 		// do horizontal lines
		pDC->MoveTo(0, 0);
		pDC->LineTo(x, 0);
		pDC->MoveTo(0, -2 * nHeight -20);
		pDC->LineTo(x, -2 * nHeight -20);

		if (newPage) {
			pDC->SelectObject(pOldFont);
			pDC->EndPage();
			goto beginPage;
		}

		// clean up
		pDC->SelectObject(pOldFont);
		pDC->EndPage();
		pDC->EndDoc();
		ReleaseDC(pDC);
		DeleteDC(hDC);		
	}
	GlobalFree(pd.m_pd.hDevMode);
	GlobalFree(pd.m_pd.hDevNames);
}

// disable editing the selected list item
void FlightPlan::OnBeginlabeleditList(NMHDR* pNMHDR, LRESULT* pResult) {
	LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;
		*pResult = 0;
}

// deallocate own memory since window can be closed from view or from here
void FlightPlan::PostNcDestroy() { 		
	plantable.RemoveAll();
	plantable.FreeExtra();
	delete this; // yes, Al Stevens would not hire me
}

void FlightPlan::OnDblclkList(NMHDR* pNMHDR, LRESULT* pResult) {
	OnClickList(pNMHDR, pResult);
	// find selected item
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavDoc *doc = (CNavDoc *) main->GetActiveDocument();
	int selected = -10;
	for (int i=0; i<plantable.GetSize(); i++) {
		if(m_list.GetItemState(i, LVNI_SELECTED) == LVNI_SELECTED)
			selected = plantable[i].index;
	}

	if (selected >= 0) { // show details dialog box
		CString details = "";
		doc->formatData(details, selected);
		Details d;
		d.index = selected;
		d.m_text = details;
		d.DoModal();
	}

	*pResult = 0;
}


void FlightPlan::OnAircraft() {
	Aircraft d;
	d.m_cruisealtitude = cruisealtitude;
	d.m_descentfuel = descentfuel;
	d.m_climbfuel = climbfuel;
	d.m_cruisefuel = cruisefuel;
	d.m_cruisespeed = cruisespeed;
	d.m_climbspeed = climbspeed;
	d.m_descentspeed = descentspeed;
	d.m_climbrate = climbrate;
	d.m_descentrate = descentrate;
	d.m_craft = craftBuffer;
	if (d.DoModal() == IDOK) {
		cruisealtitude = d.m_cruisealtitude;
		descentfuel = d.m_descentfuel;
		climbfuel = d.m_climbfuel;
		cruisefuel = d.m_cruisefuel;
		cruisespeed = d.m_cruisespeed;
		climbspeed = d.m_climbspeed;
		descentspeed = d.m_descentspeed;
		climbrate = d.m_climbrate;
		descentrate = d.m_descentrate;
		strcpy(craftBuffer, (LPCTSTR) d.m_craft);
		update();
	}	
}

void FlightPlan::OnItemchangedList(NMHDR* pNMHDR, LRESULT* pResult) {
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	// update buttons
	checkButtons();
	
	*pResult = 0;
}

void FlightPlan::OnPaint() {
	if (IsIconic())	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);
		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	} else {
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
//HCURSOR FlightPlan::OnQueryDragIcon() {
//	return (HCURSOR) m_hIcon;
//}

void FlightPlan::OnWind() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	Wind d;
	d.noselection = FALSE;
	windStruct w;

	// find the selected item
	int selected = -1;
	for (int i=0; i<plantable.GetSize(); i++) {
		if(m_list.GetItemState(i, LVNI_SELECTED) == LVNI_SELECTED)
			selected = i;
	}
	if (selected == -1) // nothing was selected
		d.noselection = TRUE;
	if (selected >= 0) {
		int t = plantable[selected].index; // toc/bod are not valid selections
		if ((t == -1) || (t == -2) | (t == -3))
			d.noselection = TRUE;
	}

	if (d.noselection == FALSE) { // initialize to current values
		if (view->windtable.Lookup(plantable[selected].index, w)) {
			d.m_speed = w.speed;
			d.m_direction = w.direction;
		}
	} else { // if all values the same, use that
		if (view->windtable.Lookup(plantable[0].index, w)) {
			int same = TRUE;
			int samespeed = w.speed;
			int samedirection = w.direction;
			for (int j=1; j<plantable.GetSize(); j++) {
				if (plantable[j].index < 0)
					continue;
				if (view->windtable.Lookup(plantable[j].index, w)) {
					if ((samespeed != w.speed) || (samedirection != w.direction)) {
						same = FALSE;
						break;
					}
				} else { 
					same = FALSE;
					break;
				}
			}
			if (same) {
				d.m_speed = w.speed;
				d.m_direction = w.direction;
			}
		}
	}

	if (d.DoModal() == IDOK) {
		if (d.m_global) { // update all entries
			for (int j=0; j<plantable.GetSize(); j++) {
				if (plantable[j].index < 0)
					continue;
				w.speed = d.m_speed;
				w.direction = d.m_direction;
				view->windtable.SetAt(plantable[j].index, w);
			}
		} else {
			w.speed = d.m_speed;
			w.direction = d.m_direction;
			view->windtable.SetAt(plantable[selected].index, w);
		}
		excessWind = FALSE; 
		update();
		view->Invalidate(TRUE);
	}
}

CString FlightPlan::readList(int row, int column) {
	char buf[100];
	LV_ITEM pItem;
	pItem.mask = LVIF_TEXT;
	pItem.pszText = buf;
	pItem.cchTextMax = 100;
	pItem.iItem = row;
	pItem.iSubItem = column;
	m_list.GetItem(&pItem);
	CString r;
	r = buf;
	r.TrimRight();
	return (r);
}

void FlightPlan::OnFilePrint() {
	OnPrint();	
}

void FlightPlan::OnUpdateFilePrint(CCmdUI* pCmdUI) {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	pCmdUI->Enable(view->plancount > 1); // works, but doesn't grey out
}


void FlightPlan::OnFileOpen() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	CNavDoc *doc = (CNavDoc *) main->GetActiveDocument();

	static char BASED_CODE szFilter[] = "Plan Files (*.pln)|*.pln|All Files (*.*)|*.*||";
	CFileDialog d(TRUE, ".pln", NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, szFilter);
	if (d.DoModal() != IDOK)
		return;
	CString filename = d.GetPathName();

	FILE *fp = fopen(filename, "r");
	if (fp == NULL) {
		char buf[256];
		sprintf(buf, "Could not open %s", filename);
		AfxMessageBox(buf);
		return;
	}

	int totalPoints = 0;
	int dbindex;
	double lat, lon;
	fscanf(fp, "%d\n", &(totalPoints));		// scan through file looking for start of file list
	for (int k=0; k<totalPoints; k++) {
		fscanf(fp, "%d", &dbindex);
		if (dbindex == -1)		// GPS indicator
			fscanf(fp, "%lf %lf", &lat, &lon);
	}

	int c;
	FILE *fp2 = fopen(view->orgDir + "\\OpenedPlan", "w");
	while ((c=fgetc(fp)) != EOF)
		fputc(c, fp2);
	fclose(fp);
	fclose(fp2);
	doc->OnFileLoad("OpenedPlan");
	DeleteFile(view->orgDir + "\\" + "OpenedPlan");

	view->plancount = 0;
	totalPoints = 0;
	fp = fopen(filename, "r");
	fscanf(fp, "%d\n", &(totalPoints));		// second pass through file to actually load data
	for (k=0; k<totalPoints; k++) {
		fscanf(fp, "%d", &dbindex);
		if (dbindex != -1)		// GPS indicator
		{
			view->plan[k] = dbindex;
			view->plancount++;
		}
		else {
			fscanf(fp, "%lf %lf", &lat, &lon);
			doc->OnMapInsertgpsfix(lat, lon);
			view->plan[k] = doc->dbCount - 1;
			view->plancount++;
		}
	}
	fclose(fp);
	view->Invalidate(TRUE);
}

void FlightPlan::OnFileSave() {
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	CNavDoc *doc = (CNavDoc *) main->GetActiveDocument();

	static char BASED_CODE szFilter[] = "Plan Files (*.pln)|*.pln|All Files (*.*)|*.*||";
	CFileDialog d(FALSE, ".pln", NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, szFilter);
	if (d.DoModal() != IDOK)
		return;
	CString filename = d.GetPathName();

	FILE *fp = fopen(filename, "w");
	if (fp == NULL) {
		char buf[256];
		sprintf(buf, "Could not open %s", filename);
		AfxMessageBox(buf);
		return;
	}
	fprintf(fp, "%d\n", view->plancount);
	for (int k=0; k<view->plancount; k++) {
		if (9 != doc->db[view->plan[k]].type)
			fprintf(fp, "%d\n", view->plan[k]);
		else 
			fprintf(fp, "-1 %lf %lf\n", doc->db[view->plan[k]].lat, doc->db[view->plan[k]].lon); // GPS fix
	}
	for (k=0; k<doc->dbFileIndex; k++) 
		fprintf(fp, "%s\n", (doc->dbFiles[k]).GetBuffer(1));	
	fclose(fp);
}

void FlightPlan::OnFileExport() {
	if (plantable.GetSize() < 2) {
		AfxMessageBox("Need at least 2 items in plan to export");
		return;
	}
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	CNavDoc *doc = (CNavDoc *) main->GetActiveDocument();

	static char BASED_CODE szFilter[] = "Text Files (*.txt)|*.txt|All Files (*.*)|*.*||";
	CFileDialog d(FALSE, ".pln", NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, szFilter);
	if (d.DoModal() != IDOK)
		return;
	CString filename = d.GetPathName();

	FILE *fp = fopen(filename, "w");
	if (fp == NULL) {
		char buf[256];
		sprintf(buf, "Could not open %s", filename);
		AfxMessageBox(buf);
		return;
	}

	// find minimum size for columns
	int locSize   = sizeof("Location");
	int idSize	  = sizeof("ID");
	int freqSize  = sizeof("Freq");
	int headSize  = sizeof("Course");
	int mheadSize = sizeof("Mag Crs");
	int recipSize = sizeof("(recip)");
	int distSize  = sizeof("Distance");
	int tdisSize  = sizeof("Total");
	int eteSize   = sizeof("ETE");
	int teteSize  = sizeof("Total");
	int fuelSize  = sizeof("Fuel");
	int tfuelSize = sizeof("Total");
	
	int nameSize;
	CString name;
	int dbtype, dbtypeLast;
	for (int k=0; k<plantable.GetSize(); k++) {
		if (plantable[k].index >= 0)
			dbtype = doc->db[plantable[k].index].type;
		else 
			dbtype = plantable[k].index; // toc, bod, toc/bob
		CString bigName;
		name = readList(k, 0);
		name.TrimRight();
		nameSize = name.GetLength() + 1;
		if (nameSize > locSize)
			locSize = nameSize;

		if ((dbtype < 4) || (dbtype == 8)) {
			name = readList(k, 1);
			nameSize = name.GetLength();
			if (nameSize > idSize)
				idSize = nameSize;
		}

		if ((dbtype < 4) || (dbtype == 10)){
			name = readList(k, 3);
			nameSize = name.GetLength() + 1;
			if (nameSize > freqSize)
				freqSize = nameSize;
		}

		name = readList(k, 4);
		nameSize = name.GetLength();
		if (nameSize > headSize)
			headSize = nameSize;

		name = readList(k, 5);
		nameSize = name.GetLength();
		if (nameSize > distSize)
			distSize = nameSize;

		name = readList(k, 6);
		nameSize = name.GetLength();
		if (nameSize > tdisSize)
			tdisSize = nameSize;

		name = readList(k, 7);
		nameSize = name.GetLength();
		if (nameSize > eteSize)
			eteSize = nameSize;

		name = readList(k, 8);
		nameSize = name.GetLength();
		if (nameSize > teteSize)
			teteSize = nameSize;

		name = readList(k, 9);
		nameSize = name.GetLength();
		if (nameSize > fuelSize)
			fuelSize = nameSize;

		name = readList(k, 10);
		nameSize = name.GetLength();
		if (nameSize > tfuelSize)
			tfuelSize = nameSize;
	}
	int idfreq = max(idSize, freqSize);
	int disttotal = max(distSize, tdisSize);
	int etetotal = max(eteSize, teteSize);
	int fueltotal = max(fuelSize, tfuelSize);

	// do column headings
	pspace(locSize -   fprintf(fp, "From"), fp);
	pspace(idfreq -    fprintf(fp, "ID") + 3, fp);
	pspace(locSize -   fprintf(fp, "To"), fp);
	pspace(idfreq -    fprintf(fp, "ID"), fp);
	pspace(headSize -  fprintf(fp, "Course"), fp);
	pspace(mheadSize - fprintf(fp, "Mag-Crs"), fp);
	pspace(disttotal - fprintf(fp, "Distance"), fp);
	pspace(etetotal -  fprintf(fp, "ETE"), fp);
	pspace(fueltotal - fprintf(fp, "Fuel"), fp);
	fprintf(fp, "\n");
	pspace(locSize -   0, fp);
	pspace(idfreq -    fprintf(fp, "Freq") + 3, fp);
	pspace(locSize -   0, fp);
	pspace(idfreq -    fprintf(fp, "Freq"), fp);
	pspace(headSize -  0, fp);
	pspace(mheadSize - 0, fp);
	pspace(disttotal - fprintf(fp, "Total"), fp);
	pspace(etetotal -  fprintf(fp, "Total"), fp);
	pspace(fueltotal - fprintf(fp, "Total"), fp);
	fprintf(fp, "\n\n");

	// do data
	for (k=1; k<plantable.GetSize(); k++) {
		if (plantable[k].index >= 0)
			dbtype = doc->db[plantable[k].index].type;
		else 
			dbtype = plantable[k].index; // toc, bod, toc/bod
		if (plantable[k-1].index > 0)
			dbtypeLast = doc->db[plantable[k-1].index].type;
		else 
			dbtypeLast = plantable[k-1].index; // toc, bod, toc/bod

		name = readList(k-1, 0);
		pspace(locSize -   fprintf(fp, "%s", name.GetBuffer(1)), fp);

		if ((dbtypeLast < 4) || (dbtypeLast == 10)) {
			name = readList(k-1, 1);
			pspace(idfreq -   fprintf(fp, "%s", name.GetBuffer(1)) + 3, fp);
		} else if (dbtypeLast == 9) {
			name = "GPS";
			pspace(idfreq -   fprintf(fp, "%s", name.GetBuffer(1)) + 3, fp);
		} else if (dbtypeLast == 4) {
			pspace(idfreq + 3, fp);
		}


		name = readList(k, 0);
		pspace(locSize -   fprintf(fp, "%s", name.GetBuffer(1)), fp);

		if ((dbtypeLast < 4) || (dbtypeLast == 10)) {
			name = readList(k, 1);
			pspace(idfreq -   fprintf(fp, "%s", name.GetBuffer(1)), fp);
		} else if (dbtypeLast == 9) {
			name = "GPS";
			pspace(idfreq -   fprintf(fp, "%s", name.GetBuffer(1)), fp);
		} else if (dbtypeLast == 4) {
			pspace(idfreq, fp);
		}

		
		char mbuf[30];
		int magHeading, trueHeading;

		//  calculate true heading
		view->gcDist(plantable[k-1].lat, plantable[k-1].lon, plantable[k].lat, plantable[k].lon);
		trueHeading = (int) view->bearing;
		sprintf(mbuf, " %d", trueHeading);
		name = mbuf;
		name.TrimRight();
		pspace(headSize -   fprintf(fp, "%s", name.GetBuffer(1)), fp);

		// magnetic heading
		name = readList(k, 4); //  corrected (magnetic + wind) heading
		sscanf((LPCTSTR) name, "%d", &magHeading);
		pspace(mheadSize -   fprintf(fp, "%s", name.GetBuffer(1)), fp);

		name = readList(k, 5);	// distance
		pspace(disttotal -   fprintf(fp, "%s", name.GetBuffer(1)), fp);

		name = readList(k, 7);	// ete
		pspace(etetotal -   fprintf(fp, "%s", name.GetBuffer(1)), fp);

		name = readList(k, 9);	// fuel
		pspace(fueltotal -   fprintf(fp, "%s", name.GetBuffer(1)), fp);
		fprintf(fp, "\n");

		pspace(locSize, fp);
		name = readList(k-1, 3);
		pspace(idfreq -   fprintf(fp, "%s", name.GetBuffer(1)) + 3, fp);

		pspace(locSize, fp);
		name = readList(k, 3);
		pspace(idfreq -   fprintf(fp, "%s", name.GetBuffer(1)), fp);
		
		pspace(headSize, fp);
		pspace(mheadSize, fp);

		name = readList(k, 6);	// total distance
		pspace(disttotal -   fprintf(fp, "%s", name.GetBuffer(1)), fp);

		name = readList(k, 8);	// total ete
		pspace(etetotal -   fprintf(fp, "%s", name.GetBuffer(1)), fp);

		name = readList(k, 10);	// total fuel
		pspace(fueltotal -   fprintf(fp, "%s", name.GetBuffer(1)), fp);
		fprintf(fp, "\n\n");
	}

	fclose(fp);
}

void FlightPlan::pspace(int s, FILE *fp) {
	if (s<0) 
		return;
	for (int i=0; i<s; i++)
		fprintf(fp, " ");
}

void FlightPlan::OnFlyto() 
{
	CMainFrame *main = (CMainFrame *) AfxGetMainWnd();
	CNavView *view = (CNavView *) main->GetActiveView();
	if (view->plancount == 0)
		return;
		
	// find the selected item
	int selected = -1;
	for (int i=0; i<plantable.GetSize(); i++) {
		if(m_list.GetItemState(i, LVNI_SELECTED) == LVNI_SELECTED)
			selected = i;
	}
	if (selected == -1) // nothing was selected
		return;
	int t = plantable[selected].index; // skip toc/bod if selected
	if ((t == -1) || (t == -2) | (t == -3))
		return;

	view->flytoPlan = TRUE;
	view->flying = TRUE;
	view->flyingLat = plantable[selected].lat;
	view->flyingLon = plantable[selected].lon;
	flyToSelected = selected;
	TRACE("Fly to selected %d\n",flyToSelected);
	view->Invalidate(TRUE);	
	view->StartFlyThread();
}

BOOL FlightPlan::SelectNextToFlyTo(CNavView *view)
{
	if (flyToSelected < 0)
		return FALSE;
	flyToSelected++;
	if (flyToSelected >= plantable.GetSize())
	{
		flyToSelected = plantable.GetSize() - 1;
		return FALSE;
	}
	view->flytoPlan = TRUE;
	view->flying = TRUE;
	view->flyingLat = plantable[flyToSelected].lat;
	view->flyingLon = plantable[flyToSelected].lon;
	TRACE("SelectNextToFlyTo %d\n",flyToSelected);
	view->Invalidate(TRUE);
	return TRUE;

}
