

#include "stdafx.h"
#include "formatedfile.h"


// ****************************************************************************
//@mfunc 
// constructor : reset file neame, comment and separator to 0 
// 
//@end
// ****************************************************************************

FormatedFile::FormatedFile()
: CStdioFile()
{
	fileName = "";
	comment = 0;
	separator = 0;
}


// ****************************************************************************
//@mfunc 
// constructor : set file name, comment char and separator 
// 
//@end
// ****************************************************************************


FormatedFile::FormatedFile( const char sepChar, 
							const char commentChar,
							const CString Filename)
	: CStdioFile()
{
	fileName = Filename;
	separator = sepChar;
	comment = commentChar;
}

//
// ****************************************************************************
//         Class : FormatedFile
//		function : Open
//
//@mfunc
//	Open a file
//	Display a dialog box if file name is empty
//	append typeText to open flags, only text files are managed
//
//@rdesc Returns one of the following values
//@flag TRUE | no error
//@end
//      Creation : JYL 25 MAR 1996
// ----------------------------------------------------------------------------
// Modifications : 
// ****************************************************************************
//

BOOL FormatedFile::Open(const CString filepath , //@parm filename to open
						UINT openFlags			//@parm flags to open file
						)			
{

	// empty buffer

	currentLine = "";
	writeBuffer = "";

	// we support only text files

	openFlags = openFlags |  CFile::typeText;
	fileName = filepath;
	
	// get a filename if empty
	
	if (fileName == "")
	{
		CFileDialog readFileName(TRUE,"*","*.*",
									(OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST),
									" AllFiles (*.*) | *.* ||");
		if (readFileName.DoModal() == IDOK)
		{
			fileName = readFileName.GetPathName( );
		}
		else return FALSE;
	}
	
	return CStdioFile::Open(fileName, openFlags,&ffException);
	
}


BOOL FormatedFile::NextRecord()
{
	
	// skip all comment line
	BOOL lineOk = TRUE;
	BOOL notEndOfFile = TRUE;
	currentLine = "";

	do
	{
		lineOk = TRUE;
		notEndOfFile = CStdioFile::ReadString(currentLine);
		if (notEndOfFile)
		{
			currentLine.TrimRight();
			currentLine.TrimLeft();
			if (!currentLine.IsEmpty())
			{
				// comment is defined so skip the comment line
				//
				if (comment)
				{
					if (currentLine.GetAt(0) == comment) lineOk = FALSE;
				}
			}
		}
	} 

	while ((!lineOk) && notEndOfFile);

	if (notEndOfFile)
	{
		recordPosition = 0;
		fieldCount = 0;
	}

	return notEndOfFile;
}


void FormatedFile::ProcessLine()
{
	/*
	int pos;

	lineFields.RemoveAll();

	const char* currentLinePtr;

	currentLinePtr =  (LPCTSTR)currentLine;
	
	while ( (pos = currentLine.Find(separator)) >= 0)
	{
		//lineFields.AddTail((void *)currentLine.Mid(0,pos));
		currentLine = currentLine.Mid(pos+1);	
	}
	//lineFields.AddTail(currentLine);
	
	// reset record position

	recordPosition = lineFields.GetHeadPosition();
	*/
	
}

int FormatedFile::GetFieldCount()
{
	return fieldCount;
}

BOOL FormatedFile::ReadNextField(CString& fieldToRead)
{

	if (currentLine != "")
	{
		fieldCount++;
		recordPosition = currentLine.Find(separator);
		if (recordPosition >=0)
		{
			
			fieldToRead = currentLine.Mid(0,recordPosition);
			currentLine = currentLine.Mid(recordPosition+1);
		
		}
		else
		{
			fieldToRead = currentLine;
			currentLine = "";
		}
		return TRUE;
	}

	return FALSE;
	
	/*
	if (recordPosition)
	{
		//fieldToRead = (char *)lineFields.GetNext(recordPosition);
		return TRUE;
	}
	else
		return FALSE;
	*/
}
//
// ****************************************************************************
//         Class : FormatedFile
//		function : NewRecord
//
//@mfunc
//	Prepare a new record to be written to disk
//	flush previous record to disk
//
//@rdesc Returns one of the following values
//@flag TRUE | no error
//@end
//      Creation : JYL 25 MAR 1996
// ----------------------------------------------------------------------------
// Modifications : 
// ****************************************************************************
//

BOOL FormatedFile::NewRecord()
{
	if (writeBuffer != "")
	{
		CStdioFile::WriteString(writeBuffer + "\n");
	}

	// empty current buffer

	writeBuffer = "";

	// do you see any reason to return FALSE ?
	return TRUE;
}

//
// ****************************************************************************
//         Class : FormatedFile
//		function : WriteField
//
//@mfunc
//	Write a new field, in fact append the string to current line after
//	a separator
//
//@rdesc Returns one of the following values
//@flag TRUE | no error
//@end
//      Creation : JYL 25 MAR 1996
// ----------------------------------------------------------------------------
// Modifications : 
// ****************************************************************************
//

BOOL FormatedFile::WriteField(CString fieldToWrite)
{
	if (writeBuffer == "")
	{
		writeBuffer = fieldToWrite;
	}
	else
	{
		writeBuffer = writeBuffer + separator + fieldToWrite;
	}
	return TRUE;
}

//
// ****************************************************************************
//         Class : FormatedFile
//		function : Close
//
//@mfunc
//	Flush write buffer to disk if needed and call the base class close
//@end
//      Creation : JYL 26 MAR 1996
// ----------------------------------------------------------------------------
// Modifications : 
// ****************************************************************************
//

void FormatedFile::Close()
{
	if (writeBuffer != "")
	{
		NewRecord();
	}
	CStdioFile::Close();
}

//
// ****************************************************************************
//         Class : FormatedFile
//		function : Find
//
//@mfunc
//		Find a string in a file, case insensitive search
//@end
//      Creation : JYL 28 MAR 1996
// ----------------------------------------------------------------------------
// Modifications : 
// ****************************************************************************
//

BOOL FormatedFile::Find(CString toFind)
{


	BOOL Found = FALSE;

	LPCTSTR pToFind = toFind;

	while( NextRecord() && !Found)
	{
		// case insensitive search in a string
		// see strstr.c
		LPCTSTR pCurrentLine = currentLine;
		// main loop go through current line
		while (*pCurrentLine)
		{
			int     i = -1;
			// check if we can find the strint toFind
			do
			{
				if (pToFind[++i] == '\0')
				{
					Found = TRUE;
					// we found it so get out this loop
					break;
					//   return (char *)s;
				}
			}
			while (toupper(pCurrentLine[i]) == toupper(pToFind[i]));
			if (!Found) ++pCurrentLine;
				else break;
		} // while
		if (Found) break;
	}

	
	return Found;

}


	