#ifndef _FORMATEDFILE_H_
#define _FORMATEDFILE_H_


#include <afxtempl.h>


// ****************************************************************************
// @class  FormatedFile
//		Manage formatedFile type text, each line is one record,
//		each record is composed by fields separated by a char separtor
//		line beginning by a comment char is skipped
// @base public | CStdioFile
// @end
//
//      Creation : JYL 25 MAR 1996
// ----------------------------------------------------------------------------
// Modifications : 
//
// ****************************************************************************


class FormatedFile : public CStdioFile

{
	char comment;			//@cmember comment char
	char separator;			//@cmember separator char
	CString fileName;		//@cmember file name
	CString currentLine;		//@cmember buffer for reading
	CString writeBuffer;		//@cmember buffer for writing
	CTypedPtrList<CPtrList, char*> lineFields;
	//CStringList lineFields;

	int recordPosition;		//@cmember 0 base index of a separator in current line
	int fieldCount;		//@cmember number of fields found in a record

	CFileException ffException; //@cmember catch file errors


private:

	void ProcessLine();
	//@cmember read a line
	BOOL ReadLine(CString& lineToRead);

public:

	//@cmember constructor
	FormatedFile();
	//@cmember constructor
	FormatedFile( const char sepChar, 
					const char commentChar = 0,
					const CString Filename = "");
	//@cmember open that file
	BOOL Open (const CString Filename = "", 
				UINT openFlags = (CFile::modeRead | CFile::typeText));
	//@cmember return number of fields in a record
	int GetFieldCount();
	//@cmember skip to next record
	BOOL NextRecord();
	//@cmember read next record in file
	BOOL ReadNextField(CString& fieldToRead);
	//@cmember write a field in file
	BOOL WriteField(CString fieldToWrite);
	//@cmember prepare a new record, last is flushed to file
	BOOL NewRecord();
	// inline functions
	//@cmember return TRUE if file was not found
	BOOL FileNotFound()
		{ return (ffException.m_cause == CFileException::fileNotFound);};

	//@cmember close the file
	void Close();

	//@cmember find a string in a file

	BOOL Find(CString toFind);

	
};






#endif	// _FORMATEDFILE_H_